@@ -1,6 +1,6 @@
 ;;;; c-platform.scm - Platform specific parameters and definitions
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -84,14 +84,16 @@
     check-syntax to-stdout no-usual-integrations case-insensitive no-lambda-info 
     profile inline keep-shadowed-macros ignore-repository
     fixnum-arithmetic disable-interrupts optimize-leaf-routines
-    lambda-lift compile-syntax tag-pointers accumulate-profile
-    disable-stack-overflow-checks raw 
+    compile-syntax tag-pointers accumulate-profile
+    disable-stack-overflow-checks raw specialize
     emit-external-prototypes-first release local inline-global
     analyze-only dynamic scrutinize no-argc-checks no-procedure-checks
-    no-procedure-checks-for-toplevel-bindings
+    no-procedure-checks-for-toplevel-bindings module
     no-bound-checks no-procedure-checks-for-usual-bindings no-compiler-syntax
     no-parentheses-synonyms no-symbol-escape r5rs-syntax emit-all-import-libraries
-    setup-mode unboxing no-module-registration) )
+    strict-types clustering
+    lambda-lift unboxing		; OBSOLETE
+    setup-mode no-module-registration) )
 
 (define valid-compiler-options-with-argument
   '(debug 
@@ -101,7 +103,9 @@
     parenthesis-synonyms
     prelude postlude prologue epilogue nursery extend feature no-feature types
     emit-import-library emit-inline-file static-extension consult-inline-file
-    heap-growth heap-shrinkage heap-initial-size ffi-define ffi-include-path) )
+    emit-type-file
+    heap-growth heap-shrinkage heap-initial-size ; DEPRECATED
+    ffi-define ffi-include-path) )
 
 
 ;;; Standard and extended bindings:
@@ -124,16 +128,19 @@
     read-char substring string-fill! vector-fill! make-string make-vector open-input-file
     open-output-file call-with-input-file call-with-output-file close-input-port close-output-port
     values call-with-values vector procedure? memq memv member assq assv assoc list-tail
-    list-ref abs char-ready? peek-char list->string string->list) )
+    list-ref abs char-ready? peek-char list->string string->list
+    current-input-port current-output-port) )
 
 (define default-extended-bindings
-  '(bitwise-and bitwise-ior bitwise-xor bitwise-not add1 sub1 fx+ fx- fx* fx/ fxmod o
+  '(bitwise-and alist-cons xcons
+    bitwise-ior bitwise-xor bitwise-not add1 sub1 fx+ fx- fx* fx/
+    fx+? fx-? fx*? fx/? fxmod o fp/?
     fx= fx> fx< fx>= fx<= fixnum? fxneg fxmax fxmin identity fp+ fp- fp* fp/ fpmin fpmax fpneg
     fp> fp< fp= fp>= fp<= fxand fxnot fxior fxxor fxshr fxshl bit-set? fxodd? fxeven?
     fpfloor fpceiling fptruncate fpround fpsin fpcos fptan fpasin fpacos fpatan
     fpatan2 fpexp fpexpt fplog fpsqrt fpabs fpinteger?
     arithmetic-shift void flush-output thread-specific thread-specific-set!
-    not-pair? atom? null-list? print print* error cpu-time proper-list? call/cc
+    not-pair? atom? null-list? print print* error proper-list? call/cc
     blob-size u8vector->blob/shared s8vector->blob/shared u16vector->blob/shared
     s16vector->blob/shared u32vector->blob/shared s32vector->blob/shared
     f32vector->blob/shared f64vector->blob/shared
@@ -143,25 +150,31 @@
     block-ref block-set! number-of-slots substring-index substring-index-ci
     hash-table-ref any? read-string substring=? substring-ci=?
     first second third fourth make-record-instance
-    u8vector-length s8vector-length u16vector-length s16vector-length u32vector-length s32vector-length
+    foldl foldr
+    u8vector-length s8vector-length u16vector-length s16vector-length u32vector-length 
+    s32vector-length
     f32vector-length f64vector-length setter
     u8vector-ref s8vector-ref u16vector-ref s16vector-ref u32vector-ref s32vector-ref
     f32vector-ref f64vector-ref f32vector-set! f64vector-set!
     u8vector-set! s8vector-set! u16vector-set! s16vector-set! u32vector-set! s32vector-set!
-    locative-ref locative-set! locative->object locative? global-ref
+    locative-ref locative-set! locative->object locative?
     null-pointer? pointer->object flonum? finite? address->pointer pointer->address
     pointer+ pointer=?
     pointer-u8-ref pointer-s8-ref pointer-u16-ref pointer-s16-ref
     pointer-u32-ref pointer-s32-ref pointer-f32-ref pointer-f64-ref
     pointer-u8-set! pointer-s8-set! pointer-u16-set! pointer-s16-set!
     pointer-u32-set! pointer-s32-set! pointer-f32-set! pointer-f64-set!
-    printf sprintf format get-keyword) )
+    current-error-port current-thread
+    printf sprintf format fprintf get-keyword) )
 
 (define internal-bindings
   '(##sys#slot ##sys#setslot ##sys#block-ref ##sys#block-set!
     ##sys#call-with-current-continuation ##sys#size ##sys#byte ##sys#setbyte
     ##sys#pointer? ##sys#generic-structure? ##sys#structure? ##sys#check-structure
-    ##sys#check-exact ##sys#check-number ##sys#check-list ##sys#check-pair ##sys#check-string ##sys#check-symbol 
+    ##sys#check-exact ##sys#check-number ##sys#check-list ##sys#check-pair ##sys#check-string
+    ##sys#check-symbol ##sys#check-boolean ##sys#check-locative
+    ##sys#check-port ##sys#check-input-port ##sys#check-output-port
+    ##sys#check-open-port
     ##sys#check-char ##sys#check-vector ##sys#check-byte-vector ##sys#list ##sys#cons
     ##sys#call-with-values ##sys#fits-in-int? ##sys#fits-in-unsigned-int? ##sys#flonum-in-fixnum-range? 
     ##sys#fudge ##sys#immediate? ##sys#direct-return ##sys#context-switch
@@ -169,11 +182,12 @@
     ##sys#bytevector? ##sys#make-vector ##sys#setter ##sys#car ##sys#cdr ##sys#pair?
     ##sys#eq? ##sys#list? ##sys#vector? ##sys#eqv? ##sys#get-keyword
     ##sys#foreign-char-argument ##sys#foreign-fixnum-argument ##sys#foreign-flonum-argument
-    ##sys#foreign-block-argument ##sys#foreign-number-vector-argument
+    ##sys#foreign-block-argument ##sys#foreign-struct-wrapper-argument
     ##sys#foreign-string-argument ##sys#foreign-pointer-argument ##sys#void
     ##sys#foreign-integer-argument ##sys#foreign-unsigned-integer-argument ##sys#double->number
     ##sys#peek-fixnum ##sys#setislot ##sys#poke-integer ##sys#permanent? ##sys#values ##sys#poke-double
-    ##sys#intern-symbol ##sys#make-symbol ##sys#null-pointer? ##sys#peek-byte) )
+    ##sys#intern-symbol ##sys#make-symbol ##sys#null-pointer? ##sys#peek-byte
+    ##sys#file-exists?) )
 
 (define non-foldable-bindings
   '(vector
@@ -196,7 +210,16 @@
     f32vector-set! f64vector-set!
     u8vector-ref s8vector-ref u16vector-ref s16vector-ref u32vector-ref s32vector-ref
     u8vector-set! s8vector-set! u16vector-set! s16vector-set! u32vector-set! s32vector-set!
-    ##sys#intern-symbol ##sys#make-symbol make-record-instance error cpu-time ##sys#block-set!) )
+    ##sys#intern-symbol ##sys#make-symbol make-record-instance error ##sys#block-set!
+    current-error-port current-thread
+    pointer-u8-ref pointer-u8-set!
+    pointer-s8-ref pointer-s8-set!
+    pointer-u16-ref pointer-u16-set!
+    pointer-s16-ref pointer-s16-set!
+    pointer-u32-ref pointer-u32-set!
+    pointer-s32-ref pointer-s32-set!
+    pointer-f32-ref pointer-f32-set!
+    pointer-f64-ref pointer-f64-set!))
 
 (define foldable-bindings
   (lset-difference 
@@ -204,28 +227,14 @@
    (lset-union eq? default-standard-bindings default-extended-bindings)
    non-foldable-bindings) )
 
-(for-each
- (cut mark-variable <> '##compiler#pure 'standard)
- '(not boolean? eq? eqv? equal? pair? null? list? zero?
-       char? eof-object? symbol? number? complex? real? integer? rational? string?
-       procedure?))
- 
-(for-each
- (cut mark-variable <> '##compiler#pure 'extended)
- '(fx+ fx- fx* fx/ fxmod
-       fx= fx> fx< fx>= fx<= fixnum? fxneg fxmax fxmin identity
-    fxand fxnot fxior fxxor fxshr fxshl fxodd? fxeven?
-    void not-pair? atom? any? u8vector? s8vector? u16vector? s16vector?
-    u32vector? s32vector? f32vector? f64vector?
-    locative?) )
-    
 (for-each
  (cut mark-variable <> '##compiler#pure '#t)
  '(##sys#slot ##sys#block-ref ##sys#size ##sys#byte
     ##sys#pointer? ##sys#generic-structure? ##sys#fits-in-int? ##sys#fits-in-unsigned-int? ##sys#flonum-in-fixnum-range? 
     ##sys#fudge ##sys#immediate?
     ##sys#bytevector? ##sys#pair?
-    ##sys#eq? ##sys#list? ##sys#vector? ##sys#eqv? ##sys#get-keyword
+    ##sys#eq? ##sys#list? ##sys#vector? ##sys#eqv? 
+    ##sys#get-keyword			; ok it isn't, but this is only used for ext. llists
     ##sys#void
     ##sys#permanent?))
 
@@ -246,19 +255,19 @@
 	  (remove
 	   (lambda (x)
 	     (and (eq? 'quote (node-class x))
-		  (= 1 (first (node-parameters x))) ) ) 
+		  (eq? 1 (first (node-parameters x))) ) ) 
 	   callargs) ] )
-     (cond [(null? callargs) (make-node '##core#call '(#t) (list cont (qnode 0)))]
+     (cond [(null? callargs) (make-node '##core#call (list #t) (list cont (qnode 0)))]
 	   [(null? (cdr callargs))
-	    (make-node '##core#call '(#t) (list cont (first callargs))) ]
+	    (make-node '##core#call (list #t) (list cont (first callargs))) ]
 	   [(eq? number-type 'fixnum)
 	    (make-node 
-	     '##core#call '(#t)
+	     '##core#call (list #t)
 	     (list
 	      cont
 	      (fold-inner
 	       (lambda (x y)
-		 (if (and (eq? 'quote (node-class y)) (= 2 (first (node-parameters y))))
+		 (if (and (eq? 'quote (node-class y)) (eq? 2 (first (node-parameters y))))
 		     (make-node '##core#inline '("C_fixnum_shift_left") (list x (qnode 1)))
 		     (make-node '##core#inline '("C_fixnum_times") (list x y)) ) )
 	       callargs) ) ) ]
@@ -276,7 +285,7 @@
    (cond [(null? callargs) #f]
 	 [(and (null? (cdr callargs)) (eq? number-type 'fixnum))
 	  (make-node
-	   '##core#call '(#t)
+	   '##core#call (list #t)
 	   (list cont
 		 (make-node '##core#inline
 			    (if unsafe '("C_u_fixnum_negate") '("C_fixnum_negate"))
@@ -292,7 +301,7 @@
 	    (and (eq? number-type 'fixnum)
 		 (>= (length callargs) 2)
 		 (make-node
-		  '##core#call '(#t)
+		  '##core#call (list #t)
 		  (list 
 		   cont
 		   (fold-inner
@@ -314,17 +323,17 @@
 		     (remove
 		      (lambda (x)
 			(and (eq? 'quote (node-class x))
-			     (= 1 (first (node-parameters x))) ) ) 
+			     (eq? 1 (first (node-parameters x))) ) ) 
 		      (cdr callargs) ) ) ] )
 	  (and (eq? number-type 'fixnum)
 	       (>= (length callargs) 2)
 	       (make-node
-		'##core#call '(#t)
+		'##core#call (list #t)
 		(list
 		 cont
 		 (fold-inner
 		  (lambda (x y)
-		    (if (and (eq? 'quote (node-class y)) (= 2 (first (node-parameters y))))
+		    (if (and (eq? 'quote (node-class y)) (eq? 2 (first (node-parameters y))))
 			(make-node '##core#inline '("C_fixnum_shift_right") (list x (qnode 1)))
 			(make-node '##core#inline '("C_fixnum_divide") (list x y)) ) )
 		  callargs) ) ) ) ) ) ) )
@@ -338,17 +347,17 @@
    (and (= (length callargs) 2)
 	(if (eq? 'fixnum number-type)
 	    (make-node
-	     '##core#call '(#t)
+	     '##core#call (list #t)
 	     (let ([arg2 (second callargs)])
 	       (list cont 
 		     (if (and (eq? 'quote (node-class arg2)) 
-			      (= 2 (first (node-parameters arg2))) )
+			      (eq? 2 (first (node-parameters arg2))) )
 			 (make-node 
 			  '##core#inline '("C_fixnum_shift_right") 
 			  (list (first callargs) (qnode 1)) )
 			 (make-node '##core#inline '("C_fixnum_divide") callargs) ) ) ) )
 	    (make-node
-	     '##core#call '(#t)
+	     '##core#call (list #t)
 	     (cons* (make-node '##core#proc '("C_quotient" #t) '()) cont callargs) ) ) ) ) )
 
 (let ()
@@ -361,7 +370,7 @@
   (define ((op1 fiop ufiop aiop) db classargs cont callargs)
     (and (= (length callargs) 1)
 	 (make-node
-	  '##core#call '(#t)
+	  '##core#call (list #t)
 	  (list 
 	   cont
 	   (if (eq? 'fixnum number-type)
@@ -382,13 +391,13 @@
 	   (or (and (eq? '##core#variable (node-class arg1))
 		    (eq? '##core#variable (node-class arg2))
 		    (equal? (node-parameters arg1) (node-parameters arg2))
-		    (make-node '##core#call '(#t) (list cont (qnode #t))) )
+		    (make-node '##core#call (list #t) (list cont (qnode #t))) )
 	       (and (or (and (eq? 'quote (node-class arg1))
 			     (not (flonum? (first (node-parameters arg1)))) )
 			(and (eq? 'quote (node-class arg2))
 			     (not (flonum? (first (node-parameters arg2)))) ) )
 		    (make-node
-		     '##core#call '(#t) 
+		     '##core#call (list #t) 
 		     (list cont (make-node '##core#inline '("C_eqp") callargs)) ) ) ) ) ) )
   (rewrite 'eqv? 8 eqv?-id)
   (rewrite '##sys#eqv? 8 eqv?-id))
@@ -405,7 +414,7 @@
 	  (or (and (eq? '##core#variable (node-class arg1))
 		   (eq? '##core#variable (node-class arg2))
 		   (equal? (node-parameters arg1) (node-parameters arg2))
-		   (make-node '##core#call '(#t) (list cont (qnode #t))) )
+		   (make-node '##core#call (list #t) (list cont (qnode #t))) )
 	      (and (or (and (eq? 'quote (node-class arg1))
 			    (let ([f (first (node-parameters arg1))])
 			      (or (immediate? f) (symbol? f)) ) )
@@ -413,10 +422,10 @@
 			    (let ([f (first (node-parameters arg2))])
 			      (or (immediate? f) (symbol? f)) ) ) )
 		   (make-node
-		    '##core#call '(#t) 
+		    '##core#call (list #t) 
 		    (list cont (make-node '##core#inline '("C_eqp") callargs)) ) )
 	      (make-node
-	       '##core#call '(#t) 
+	       '##core#call (list #t) 
 	       (list cont (make-node '##core#inline '("C_i_equalp") callargs)) ) ) ) ) ) )
 
 (let ()
@@ -430,7 +439,7 @@
 	       [proc (car callargs)] )
 	   (if (eq? 'quote (node-class lastarg))
 	       (make-node
-		'##core#call '(#f)
+		'##core#call (list #f)
 		(cons* (first callargs)
 		       cont 
 		       (append (cdr (butlast callargs)) (map qnode (first (node-parameters lastarg)))) ) )
@@ -440,12 +449,12 @@
 			  (and (memq name '(values ##sys#values))
 			       (intrinsic? name)
 			       (make-node
-				'##core#call '(#t)
+				'##core#call (list #t)
 				(list (make-node '##core#proc '("C_apply_values" #t) '())
 				      cont
 				      (cadr callargs) ) ) ) ) ) 
 		   (make-node
-		    '##core#call '(#t)
+		    '##core#call (list #t)
 		    (cons* (make-node '##core#proc '("C_apply" #t) '())
 			   cont callargs) ) ) ) ) ) )
   (rewrite 'apply 8 rewrite-apply)
@@ -464,7 +473,7 @@
 	     (lambda (return)
 	       (let ([arg (first callargs)])
 		 (make-node
-		  '##core#call '(#t)
+		  '##core#call (list #t)
 		  (list
 		   cont
 		   (cond [(and (eq? '##core#variable (node-class arg))
@@ -494,7 +503,7 @@
        (lambda (db classargs cont callargs)
 	 ;; (values <x>) -> <x>
 	 (and (= (length callargs) 1)
-	      (make-node '##core#call '(#t) (cons cont callargs) ) ) ) ] )
+	      (make-node '##core#call (list #t) (cons cont callargs) ) ) ) ] )
   (rewrite 'values 8 rvalues)
   (rewrite '##sys#values 8 rvalues) )
 
@@ -522,10 +531,10 @@
 				       '##core#lambda
 				       (list (gensym 'f_) #f (list tmpk) 0)
 				       (list (make-node
-					      '##core#call '(#t)
+					      '##core#call (list #t)
 					      (list arg2 cont (varnode tmpk)) ) ) ) 
 				      (make-node
-				       '##core#call '(#t)
+				       '##core#call (list #t)
 				       (list arg1 (varnode tmp)) ) ) ) ) ) ) ) ) ) ) ) )
   (rewrite 'call-with-values 8 rewrite-c-w-v)
   (rewrite '##sys#call-with-values 8 rewrite-c-w-v) )
@@ -536,7 +545,6 @@
 (rewrite 'call-with-values 13 "C_call_with_values" #t)
 (rewrite '##sys#call-with-values 13 "C_u_call_with_values" #f)
 (rewrite '##sys#call-with-values 13 "C_call_with_values" #t)
-(rewrite 'cpu-time 13 "C_cpu_time" #t)
 (rewrite 'locative-ref 13 "C_locative_ref" #t)
 (rewrite '##sys#continuation-graft 13 "C_continuation_graft" #t)
 
@@ -624,16 +632,20 @@
 (rewrite 'string-set! 2 3 "C_i_string_set" #t)
 (rewrite 'vector-ref 2 2 "C_slot" #f)
 (rewrite 'vector-ref 2 2 "C_i_vector_ref" #t)
-(rewrite 'char=? 2 2 "C_eqp" #t)
-(rewrite 'char>? 2 2 "C_fixnum_greaterp" #t)
-(rewrite 'char<? 2 2 "C_fixnum_lessp" #t)
-(rewrite 'char>=? 2 2 "C_fixnum_greater_or_equal_p" #t)
-(rewrite 'char<=? 2 2 "C_fixnum_less_or_equal_p" #t)
+(rewrite 'char=? 2 2 "C_i_char_equalp" #t)   ; a bit of a lie: won't crash but accepts garbage
+(rewrite 'char>? 2 2 "C_i_char_greaterp" #t)
+(rewrite 'char<? 2 2 "C_i_char_lessp" #t)
+(rewrite 'char>=? 2 2 "C_i_char_greater_or_equal_p" #t)
+(rewrite 'char<=? 2 2 "C_i_char_less_or_equal_p" #t)
 (rewrite '##sys#slot 2 2 "C_slot" #t)		; consider as safe, the primitive is unsafe anyway.
-(rewrite '##sys#block-ref 2 2 "C_i_block_ref" #t) ;*** must be safe for pattern matcher (anymore?)
+(rewrite '##sys#block-ref 2 2 "C_i_block_ref" #t) ;XXX must be safe for pattern matcher (anymore?)
 (rewrite '##sys#size 2 1 "C_block_size" #t)
 (rewrite 'fxnot 2 1 "C_fixnum_not" #t)
 (rewrite 'fx* 2 2 "C_fixnum_times" #t)
+(rewrite 'fx+? 2 2 "C_i_o_fixnum_plus" #t)
+(rewrite 'fx-? 2 2 "C_i_o_fixnum_difference" #t)
+(rewrite 'fx*? 2 2 "C_i_o_fixnum_times" #t)
+(rewrite 'fx/? 2 2 "C_i_o_fixnum_quotient" #t)
 (rewrite 'fx= 2 2 "C_eqp" #t)
 (rewrite 'fx> 2 2 "C_fixnum_greaterp" #t)
 (rewrite 'fx< 2 2 "C_fixnum_lessp" #t)
@@ -684,11 +696,12 @@
 
 (rewrite 'bitwise-not 22 1 "C_a_i_bitwise_not" #t words-per-flonum "C_fixnum_not")
 
-(rewrite 'fp+ 16 2 "C_a_i_flonum_plus" #t words-per-flonum)
-(rewrite 'fp- 16 2 "C_a_i_flonum_difference" #t words-per-flonum)
-(rewrite 'fp* 16 2 "C_a_i_flonum_times" #t words-per-flonum)
-(rewrite 'fp/ 16 2 "C_a_i_flonum_quotient" #t words-per-flonum)
-(rewrite 'fpneg 16 1 "C_a_i_flonum_negate" #t words-per-flonum)
+(rewrite 'fp+ 16 2 "C_a_i_flonum_plus" #f words-per-flonum)
+(rewrite 'fp- 16 2 "C_a_i_flonum_difference" #f words-per-flonum)
+(rewrite 'fp* 16 2 "C_a_i_flonum_times" #f words-per-flonum)
+(rewrite 'fp/ 16 2 "C_a_i_flonum_quotient" #f words-per-flonum)
+(rewrite 'fp/? 16 2 "C_a_i_flonum_quotient_checked" #f words-per-flonum)
+(rewrite 'fpneg 16 1 "C_a_i_flonum_negate" #f words-per-flonum)
 
 (rewrite 'exp 16 1 "C_a_i_exp" #t words-per-flonum)
 (rewrite 'sin 16 1 "C_a_i_sin" #t words-per-flonum)
@@ -727,6 +740,8 @@
 (rewrite '##sys#check-number 2 1 "C_i_check_number" #t)
 (rewrite '##sys#check-list 2 1 "C_i_check_list" #t)
 (rewrite '##sys#check-pair 2 1 "C_i_check_pair" #t)
+(rewrite '##sys#check-boolean 2 1 "C_i_check_boolean" #t)
+(rewrite '##sys#check-locative 2 1 "C_i_check_locative" #t)
 (rewrite '##sys#check-symbol 2 1 "C_i_check_symbol" #t)
 (rewrite '##sys#check-string 2 1 "C_i_check_string" #t)
 (rewrite '##sys#check-byte-vector 2 1 "C_i_check_bytevector" #t)
@@ -737,6 +752,8 @@
 (rewrite '##sys#check-number 2 2 "C_i_check_number_2" #t)
 (rewrite '##sys#check-list 2 2 "C_i_check_list_2" #t)
 (rewrite '##sys#check-pair 2 2 "C_i_check_pair_2" #t)
+(rewrite '##sys#check-boolean 2 2 "C_i_check_boolean_2" #t)
+(rewrite '##sys#check-locative 2 2 "C_i_check_locative_2" #t)
 (rewrite '##sys#check-symbol 2 2 "C_i_check_symbol_2" #t)
 (rewrite '##sys#check-string 2 2 "C_i_check_string_2" #t)
 (rewrite '##sys#check-byte-vector 2 2 "C_i_check_bytevector_2" #t)
@@ -764,11 +781,6 @@
 (rewrite 'vector-set! 11 3 '##sys#setslot #f)
 (rewrite 'vector-set! 2 3 "C_i_vector_set" #t)
 
-(rewrite '##sys#vector->list 11 1 'vector->list #t)
-(rewrite '##sys#list->vector 11 1 'list->vector #t)
-(rewrite '##sys#>= 11 2 '>= #t)
-(rewrite '##sys#= 11 2 '= #t)
-
 (rewrite 'gcd 12 '##sys#gcd #t 2)
 (rewrite 'lcm 12 '##sys#lcm #t 2)
 (rewrite 'identity 12 #f #t 1)
@@ -783,6 +795,8 @@
 (rewrite '+ 16 2 "C_a_i_plus" #t 4)	; words-per-flonum
 (rewrite '- 16 2 "C_a_i_minus" #t 4)	; words-per-flonum
 (rewrite '/ 16 2 "C_a_i_divide" #t 4)	; words-per-flonum
+(rewrite 'exact->inexact 16 1 "C_a_i_exact_to_inexact" #t 4) ; words-per-flonum
+
 (rewrite '= 17 2 "C_i_nequalp")
 (rewrite '> 17 2 "C_i_greaterp")
 (rewrite '< 17 2 "C_i_lessp")
@@ -799,8 +813,6 @@
 (rewrite '>= 13 "C_greater_or_equal_p" #t)
 (rewrite '<= 13 "C_less_or_equal_p" #t)
 
-(rewrite 'exact->inexact 13 "C_exact_to_inexact" #t)
-(rewrite 'string->number 13 "C_string_to_number" #t)
 (rewrite 'number->string 13 "C_number_to_string" #t)
 (rewrite '##sys#call-with-current-continuation 13 "C_call_cc" #t)
 (rewrite '##sys#allocate-vector 13 "C_allocate_vector" #t)
@@ -817,13 +829,12 @@
 (rewrite 'even? 17 1 "C_i_evenp" "C_u_i_evenp")
 (rewrite 'odd? 17 1 "C_i_oddp" "C_u_i_oddp")
 
-(rewrite 'fxodd? 2 1 "C_fixnumoddp" #t)
-(rewrite 'fxeven? 2 1 "C_fixnumevenp" #t)
+(rewrite 'fxodd? 2 1 "C_i_fixnumoddp" #t)
+(rewrite 'fxeven? 2 1 "C_i_fixnumevenp" #t)
 
 (rewrite 'floor 15 'flonum 'fixnum 'fpfloor #f)
 (rewrite 'ceiling 15 'flonum 'fixnum 'fpceiling #f)
 (rewrite 'truncate 15 'flonum 'fixnum 'fptruncate #f)
-(rewrite 'round 15 'flonum 'fixnum 'fpround #f)
 
 (rewrite 'fpsin 16 1 "C_a_i_flonum_sin" #f words-per-flonum)
 (rewrite 'fpcos 16 1 "C_a_i_flonum_cos" #f words-per-flonum)
@@ -838,9 +849,27 @@
 (rewrite 'fpsqrt 16 1 "C_a_i_flonum_sqrt" #f words-per-flonum)
 (rewrite 'fpabs 16 1 "C_a_i_flonum_abs" #f words-per-flonum)
 (rewrite 'fptruncate 16 1 "C_a_i_flonum_truncate" #f words-per-flonum)
-(rewrite 'fpround 16 1 "C_a_i_flonum_truncate" #f words-per-flonum)
-(rewrite 'fpceiling 16 1 "C_a_i_flonum_truncate" #f words-per-flonum)
-(rewrite 'fpround 16 1 "C_a_i_flonum_truncate" #f words-per-flonum)
+(rewrite 'fpround 16 1 "C_a_i_flonum_round" #f words-per-flonum)
+(rewrite 'fpceiling 16 1 "C_a_i_flonum_ceiling" #f words-per-flonum)
+(rewrite 'fpround 16 1 "C_a_i_flonum_floor" #f words-per-flonum)
+
+(rewrite
+ 'string->number 8
+ (lambda (db classargs cont callargs)
+   ;; (string->number X) -> (##core#inline_allocate ("C_a_i_string_to_number" 4) X 10)
+   ;; (string->number X Y) -> (##core#inline_allocate ("C_a_i_string_to_number" 4) X Y)
+   (define (build x y)
+     (make-node
+      '##core#call (list #t)
+      (list cont
+	    (make-node
+	     '##core#inline_allocate 
+	     '("C_a_i_string_to_number" 4) ; words-per-flonum
+	     (list x y)))))
+   (case (length callargs)
+     ((1) (build (first callargs) (qnode 10)))
+     ((2) (build (first callargs) (second callargs)))
+     (else #f))))
 
 (rewrite 'cons 16 2 "C_a_i_cons" #t 3)
 (rewrite '##sys#cons 16 2 "C_a_i_cons" #t 3)
@@ -879,7 +908,7 @@
    ;; (##sys#setslot <x> <y> <z>) -> (##core#inline "C_i_setslot" <x> <y> <z>)
    (and (= (length callargs) 3)
 	(make-node 
-	 '##core#call '(#t)
+	 '##core#call (list #t)
 	 (list cont
 	       (make-node
 		'##core#inline
@@ -911,7 +940,7 @@
    (and (= 2 (length callargs))
 	(let ([val (second callargs)])
 	  (make-node
-	   '##core#call '(#t)
+	   '##core#call (list #t)
 	   (list cont
 		 (or (and-let* ([(eq? 'quote (node-class val))]
 				[(eq? number-type 'fixnum)]
@@ -954,7 +983,7 @@
 (rewrite '##sys#foreign-char-argument 17 1 "C_i_foreign_char_argumentp")
 (rewrite '##sys#foreign-flonum-argument 17 1 "C_i_foreign_flonum_argumentp")
 (rewrite '##sys#foreign-block-argument 17 1 "C_i_foreign_block_argumentp")
-(rewrite '##sys#foreign-number-vector-argument 17 2 "C_i_foreign_number_vector_argumentp")
+(rewrite '##sys#foreign-struct-wrapper-argument 17 2 "C_i_foreign_struct_wrapper_argumentp")
 (rewrite '##sys#foreign-string-argument 17 1 "C_i_foreign_string_argumentp")
 (rewrite '##sys#foreign-pointer-argument 17 1 "C_i_foreign_pointer_argumentp")
 (rewrite '##sys#foreign-integer-argument 17 1 "C_i_foreign_integer_argumentp")
@@ -968,11 +997,11 @@
 (rewrite 'u16vector-ref 2 2 "C_u_i_u16vector_ref" #f)
 (rewrite 's16vector-ref 2 2 "C_u_i_s16vector_ref" #f)
 
-(rewrite 'f32vector-ref 16 2 "C_a_i_f32vector_ref" #f words-per-flonum)
-(rewrite 'f64vector-ref 16 2 "C_a_i_f64vector_ref" #f words-per-flonum)
+(rewrite 'f32vector-ref 16 2 "C_a_u_i_f32vector_ref" #f words-per-flonum)
+(rewrite 'f64vector-ref 16 2 "C_a_u_i_f64vector_ref" #f words-per-flonum)
 
-(rewrite 'u32vector-ref 22 2 "C_a_i_u32vector_ref" #f words-per-flonum "C_u_i_u32vector_ref")
-(rewrite 's32vector-ref 22 2 "C_a_i_s32vector_ref" #f words-per-flonum "C_u_i_s32vector_ref")
+(rewrite 'u32vector-ref 22 2 "C_a_u_i_u32vector_ref" #f words-per-flonum "C_u_i_u32vector_ref")
+(rewrite 's32vector-ref 22 2 "C_a_u_i_s32vector_ref" #f words-per-flonum "C_u_i_s32vector_ref")
 
 (rewrite 'u8vector-set! 2 3 "C_u_i_u8vector_set" #f)
 (rewrite 's8vector-set! 2 3 "C_u_i_s8vector_set" #f)
@@ -1016,7 +1045,7 @@
 		  (let ([tmp (gensym)]
 			[c (first (node-parameters n))] )
 		    (and (fixnum? c)
-			 (<= c 32)
+			 (<= 0 c 32)
 			 (let ([val (if (pair? (cdr callargs))
 					(second callargs)
 					(make-node '##core#undefined '() '()) ) ] )
@@ -1025,7 +1054,7 @@
 			    (list tmp)
 			    (list val
 				  (make-node
-				   '##core#call '(#t)
+				   '##core#call (list #t)
 				   (list cont
 					 (make-node
 					  '##core#inline_allocate 
@@ -1055,7 +1084,7 @@
 				   (not (get db var 'assigned)) 
 				   (not (get db var 'inline-transient))
 				   (make-node
-				    '##core#call '(#t)
+				    '##core#call (list #t)
 				    (list val cont (qnode #f)) ) ) ) ) ) ) ) ) ) ) ) )
   (rewrite 'call-with-current-continuation 8 rewrite-call/cc)
   (rewrite 'call/cc 8 rewrite-call/cc) )
@@ -1085,7 +1114,6 @@
     (pointer-f32-ref . pointer-f32-set!)
     (pointer-f64-ref . pointer-f64-set!)
     (string-ref . string-set!)
-    (global-ref . global-set!)
     (vector-ref . vector-set!) ) )
 
 (rewrite
@@ -1099,11 +1127,15 @@
 		 (and (intrinsic? sym)
 		      (and-let* ((a (assq sym setter-map)))
 			(make-node
-			 '##core#call '(#t)
+			 '##core#call (list #t)
 			 (list cont (varnode (cdr a))) ) ) ) ) ) ) ) ) )
 			       
-(rewrite 'void 3 '##sys#undefined-value)
-(rewrite '##sys#void 3 '##sys#undefined-value)
+(rewrite 'void 3 '##sys#undefined-value 0)
+(rewrite '##sys#void 3 '##sys#undefined-value #f)
+(rewrite 'current-thread 3 '##sys#current-thread 0)
+(rewrite 'current-input-port 3 '##sys#standard-input 0)
+(rewrite 'current-output-port 3 '##sys#standard-output 0)
+(rewrite 'current-error-port 3 '##sys#standard-error 0)
 
 (rewrite
  'any? 8
@@ -1111,7 +1143,7 @@
    (and (= 1 (length callargs))
 	(let ((arg (car callargs)))
 	  (make-node
-	   '##core#call '(#t) 
+	   '##core#call (list #t) 
 	   (list cont
 		 (if (and (eq? '##core#variable (node-class arg))
 			  (not (get db (car (node-parameters arg)) 'global)) )
@@ -1125,7 +1157,7 @@
  (lambda (db classargs cont callargs)
    (and (= 2 (length callargs))
 	(make-node
-	 '##core#call '(#t)
+	 '##core#call (list #t)
 	 (list cont
 	       (make-node
 		'##core#inline 
@@ -1142,3 +1174,36 @@
 
 (rewrite 'get-keyword 7 2 "C_i_get_keyword" #f #t)
 (rewrite '##sys#get-keyword 7 2 "C_i_get_keyword" #f #t)
+
+(rewrite 
+ 'alist-cons 8
+ (lambda (db classargs cont callargs)
+   (and (= 3 (length callargs))
+	(make-node
+	 '##core#call (list #t)
+	 (list cont
+	       (make-node
+		'##core#inline_allocate
+		'("C_a_i_cons" 3) 
+		(list (make-node
+		       '##core#inline_allocate
+		       '("C_a_i_cons" 3)
+		       (list (first callargs) (second callargs)))
+		      (third callargs))))))))
+
+(rewrite 
+ 'xcons 8
+ (lambda (db classargs cont callargs)
+   (and (= 2 (length callargs))
+	(let ((tmp (gensym)))
+	  (make-node 
+	   'let (list tmp)		; preserve order of argument evaluation
+	   (list
+	    (first callargs)
+	    (make-node
+	     '##core#call (list #t)
+	     (list cont
+		   (make-node
+		    '##core#inline_allocate
+		    '("C_a_i_cons" 3) 
+		    (list (second callargs) (varnode tmp)))))))))))
