@@ -1,6 +1,6 @@
 ;;;; chicken-install.scm
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -25,19 +25,20 @@
 
 
 (require-library setup-download setup-api)
-(require-library srfi-1 posix data-structures utils regex ports extras srfi-13 files)
+(require-library srfi-1 posix data-structures utils irregex ports extras srfi-13 files)
 (require-library chicken-syntax)	; in case an import library reexports chicken syntax
 (require-library chicken-ffi-syntax)	; same reason, also for filling modules.db
 
 (module main ()
 
-  (import scheme chicken srfi-1 posix data-structures utils regex ports extras
+  (import scheme chicken srfi-1 posix data-structures utils irregex ports extras
           srfi-13 files)
   (import setup-download setup-api)
 
   (import foreign)
 
   (define +default-repository-files+
+    ;;XXX keep this up-to-date!
     '("setup-api.so" "setup-api.import.so"
       "setup-download.so" "setup-download.import.so"
       "chicken.import.so"
@@ -51,11 +52,9 @@
       "srfi-13.import.so"
       "srfi-69.import.so"
       "extras.import.so"
-      "regex.import.so"
       "srfi-14.import.so"
       "tcp.import.so"
       "foreign.import.so"
-      "scheme.import.so"
       "srfi-18.import.so"
       "utils.import.so"
       "csi.import.so"
@@ -66,12 +65,20 @@
   (define-constant +module-db+ "modules.db")
   (define-constant +defaults-file+ "setup.defaults")
 
+  (define-foreign-variable C_TARGET_LIB_HOME c-string)
+  (define-foreign-variable C_INSTALL_BIN_HOME c-string)
+  (define-foreign-variable C_TARGET_PREFIX c-string)
+  (define-foreign-variable C_BINARY_VERSION int)
+  (define-foreign-variable C_WINDOWS_SHELL bool)
+  (define-foreign-variable C_CSI_PROGRAM c-string)
+
   (define *program-path*
     (or (and-let* ((p (get-environment-variable "CHICKEN_PREFIX")))
           (make-pathname p "bin") )
-        (foreign-value "C_INSTALL_BIN_HOME" c-string) ) )
+        C_INSTALL_BIN_HOME))
 
   (define *keep* #f)
+  (define *keep-existing* #f)
   (define *force* #f)
   (define *run-tests* #f)
   (define *retrieve-only* #f)
@@ -81,9 +88,10 @@
   (define *default-sources* '())
   (define *default-location* #f)
   (define *default-transport* 'http)
-  (define *windows-shell* (foreign-value "C_WINDOWS_SHELL" bool))
+  (define *windows-shell* C_WINDOWS_SHELL)
   (define *proxy-host* #f)
   (define *proxy-port* #f)
+  (define *proxy-user-pass* #f)
   (define *running-test* #f)
   (define *mappings* '())
   (define *deploy* #f)
@@ -97,12 +105,22 @@
   (define *target-extension* *cross-chicken*)
   (define *debug-setup* #f)
   (define *keep-going* #f)
+  (define *override* '())
+  (define *reinstall* #f)
+  (define *show-depends* #f)
+  (define *show-foreign-depends* #f)
+  (define *hacks* '())
 
-  (define (get-prefix)
+  (define (repo-path)
+    (if (and *cross-chicken* (not *host-extension*))
+	(make-pathname C_TARGET_LIB_HOME (sprintf "chicken/~a" C_BINARY_VERSION))
+	(repository-path)))
+
+  (define (get-prefix #!optional runtime)
     (cond ((and *cross-chicken*
 		(not *host-extension*))
-	   (or *prefix*
-	       (foreign-value "C_TARGET_PREFIX" c-string)))
+	   (or (and (not runtime) *prefix*)
+	       C_TARGET_PREFIX))
 	  (else *prefix*)))
 
   (define (load-defaults)
@@ -149,6 +167,13 @@
 				 (cons (car a) (cadr a))
 				 (broken x)))
 			   (cdr x)))))
+		  ((override)
+		   (set! *override* 
+		     (if (and (pair? (cdr x)) (string? (cadr x)))
+			 (read-file (cadr x))
+			 (cdr x))))
+		  ((hack)
+		   (set! *hacks* (append *hacks* (list (eval (cadr x))))))
 		  (else (broken x))))
 	      (read-file deff))))
       (pair? *default-sources*) ))
@@ -157,7 +182,7 @@
     (cond ((assoc name *aliases*) => 
 	   (lambda (a)
 	     (let ((new (cdr a)))
-	       (print "resolving alias `" name "' to: " new)
+	       ;(print "resolving alias `" name "' to: " new)
 	       (resolve-location new))))
 	  (else name)))
 
@@ -193,7 +218,9 @@
   (define (ext-version x)
     (cond ((or (eq? x 'chicken)
                (equal? x "chicken")
-               (member (->string x) ##sys#core-library-modules))
+               (let ((xs (->string x)))
+		 (or (member xs ##sys#core-library-modules)
+		     (member xs ##sys#core-syntax-modules))))
            (chicken-version) )
           ((extension-information x) =>
            (lambda (info)
@@ -209,49 +236,78 @@
      (deps 'needs meta)
      (if *run-tests* (deps 'test-depends meta) '())))
 
-  (define (outdated-dependencies meta)
+  (define (check-dependency dep)
+    (cond ((or (symbol? dep) (string? dep))
+	   (values
+	    (and (not (ext-version dep))
+		 (->string dep))
+	    #f))
+	  ((and (list? dep) (eq? 'or (car dep)))
+	   (let scan ((ordeps (cdr dep)) (bestm #f) (bestu #f))
+	     (if (null? ordeps)
+		 (values
+		  (cond (bestu #f)	; upgrade overrides new
+			(bestm bestm)
+			(else #f))
+		  bestu)
+		 (let-values (((m u) (check-dependency (car ordeps))))
+		   (if (and (not m) (not u))
+		       (values #f #f)
+		       (scan (cdr ordeps)
+			     (if (and m (not bestm))
+				 m
+				 bestm)
+			     (if (and u (not bestu))
+				 u
+				 bestu)))))))
+	  ((and (list? dep) (= 2 (length dep))
+		(or (string? (car dep)) (symbol? (car dep))))
+	   (let ((v (ext-version (car dep))))
+	     (cond ((not v)
+		    (values
+		     (->string (car dep))
+		     #f))
+		   ((not (version>=? v (->string (cadr dep))))
+		    (cond ((string=? "chicken" (->string (car dep)))
+			   (if *force*
+			       (values #f #f)
+			       (error
+				(string-append 
+				 "Your CHICKEN version is not recent enough to use this extension - version "
+				 (cadr dep) 
+				 " or newer is required"))))
+			  (else
+			   (values 
+			    #f
+			    (cons (->string (car dep)) (->string (cadr dep)))))))
+		   (else (values #f #f)))))
+	  (else
+	   (warning
+	    "invalid dependency syntax in extension meta information"
+	    dep)
+	   (values #f #f))))
+
+  (define (outdated-dependencies egg meta)
     (let ((ds (meta-dependencies meta)))
+      (for-each
+       (lambda (h) (set! ds (h egg ds)))
+       *hacks*)
       (let loop ((deps ds) (missing '()) (upgrade '()))
         (if (null? deps)
             (values (reverse missing) (reverse upgrade))
             (let ((dep (car deps))
                   (rest (cdr deps)))
-              (cond ((or (symbol? dep) (string? dep))
-                     (loop rest
-                           (if (ext-version dep)
-                               missing
-                               (cons (->string dep) missing))
-                           upgrade))
-                    ((and (list? dep) (= 2 (length dep))
-                          (or (string? (car dep)) (symbol? (car dep))))
-                     (let ((v (ext-version (car dep))))
-                       (cond ((not v)
-                              (loop rest (cons (->string (car dep)) missing) upgrade))
-                             ((not (version>=? v (->string (cadr dep))))
-			      (when (and (string=? "chicken" (->string (car dep)))
-					 (not *force*))
-				(error
-				 (string-append 
-				  "Your CHICKEN version is not recent enough to use this extension - version "
-				  (cadr dep) 
-				  " or newer is required")))
-                              (loop rest missing
-                                    (alist-cons
-                                     (->string (car dep)) (->string (cadr dep))
-                                     upgrade)))
-                             (else (loop rest missing upgrade)))))
-                    (else
-                     (warning
-                      "invalid dependency syntax in extension meta information"
-                      dep)
-                     (loop rest missing upgrade))))))))
+	      (let-values (((m u) (check-dependency dep)))
+		(loop rest
+		      (if m (cons m missing) missing)
+		      (if u (cons u upgrade) upgrade))))))))
 
   (define *eggs+dirs+vers* '())
   (define *dependencies* '())
   (define *checked* '())
 
   (define *csi* 
-    (shellpath (make-pathname *program-path* (foreign-value "C_CSI_PROGRAM" c-string))))
+    (shellpath (make-pathname *program-path* C_CSI_PROGRAM)))
 
   (define (try-extension name version trans locn)
     (condition-case
@@ -264,7 +320,9 @@
          password: *password*
 	 trunk: *trunk*
 	 proxy-host: *proxy-host*
-	 proxy-port: *proxy-port*)
+	 proxy-port: *proxy-port*
+	 proxy-user-pass: *proxy-user-pass*
+	 clean: (and (not *retrieve-only*) (not *keep*)))
       [(exn net)
        (print "TCP connect timeout")
        (values #f "") ]
@@ -278,22 +336,37 @@
       [e ()
        (abort e) ] ) )
 
-  (define (try-default-sources name version)
+  (define (with-default-sources proc)
     (let trying-sources ([defs (known-default-sources)])
       (if (null? defs)
-          (values #f "")
+          (proc #f #f
+                (lambda ()
+                  (with-output-to-port (current-error-port)
+                    (lambda ()
+                      (print "Could not determine a source of extensions. "
+                             "Please, specify a location and a transport for "
+                             "a source.")))
+                  (exit 1)))
           (let* ([def (car defs)]
                  [locn (resolve-location
 			(cadr (or (assq 'location def)
 				  (error "missing location entry" def))))]
                  [trans (cadr (or (assq 'transport def)
                                   (error "missing transport entry" def)))])
-            (let-values ([(dir ver) (try-extension name version trans locn)])
-              (if dir
-                  (values dir ver)
-                  (begin
+	    (proc trans locn
+		  (lambda ()
                     (invalidate-default-source! def)
-                    (trying-sources (cdr defs)) ) ) ) ) ) ) )
+                    (trying-sources (cdr defs)) ) ) ) ) ) )
+
+  (define (try-default-sources name version)
+    (with-default-sources
+     (lambda (trans locn next)
+       (if (not trans)
+	   (values #f "")
+	   (let-values (((dir ver) (try-extension name version trans locn)))
+	     (if dir
+		 (values dir ver)
+		 (next)))))))
 
   (define (make-replace-extension-question e+d+v upgrade)
     (string-concatenate
@@ -301,15 +374,60 @@
       (list "The following installed extensions are outdated, because `"
             (car e+d+v)
             "' requires later versions:\n")
-      (map
+      (filter-map
        (lambda (e)
-         (conc
-          "  " (car e)
-          " (" (let ([v (assq 'version (extension-information (car e)))]) (if v (cadr v) "???"))
-               " -> " (cdr e) ")"
-          #\newline) )
+	 (cond ((assq (string->symbol (car e)) *override*) =>
+		(lambda (a)
+		  (unless (equal? (cadr a) (cdr e))
+		    (warning
+		     (sprintf "version `~a' of extension `~a' overrides required version `~a'"
+		       (cadr a) (car a) (cdr e))))
+		  #f))
+	       (else
+		(conc
+		 "  " (car e)
+		 " (" (let ((v (assq 'version (extension-information (car e))))) 
+			(if v (cadr v) "???"))
+		 " -> " (cdr e) ")"
+		 #\newline) )))
        upgrade)
-      '("\nDo you want to replace the existing extensions?"))) )
+      '("\nDo you want to replace the existing extensions?"))))
+
+  (define (override-version egg)
+    (let ((name (string->symbol (if (pair? egg) (car egg) egg))))
+      (cond ((assq name *override*) =>
+	     (lambda (a)
+	       (cond ((and (pair? egg) (not (equal? (cadr a) (cdr egg))))
+		      (warning
+		       (sprintf 
+			   "version `~a' of extension `~a' overrides explicitly given version `~a'"
+			 (cadr a) name (cdr egg))))
+		     (else
+		      (print "overriding: " a)))
+	       (cadr a)))
+	    ((pair? egg) (cdr egg))
+	    (else #f))))
+
+  (define (show-depends eggs . type)
+    (print "fetching meta information...")
+    (retrieve eggs)
+    (let ((type (optional type 'depends)))
+      (printf "~a dependencies as reported in .meta:\n"
+	      (case type ((depends) "Egg")
+			 ((foreign-depends) "Foreign")))
+      (for-each
+       (lambda (egg)
+	 (and-let* ((meta-file (make-pathname (cadr egg) (car egg) "meta"))
+		    (m (and (file-exists? meta-file) (with-input-from-file meta-file read)))
+		    (ds (if (eq? type 'depends)
+			  (append (deps 'needs m) (deps type m))
+			  (deps type m))))
+	   (unless (null? ds)
+	     (print (car egg) ": ")
+	     (for-each (cut print "\t" <>) ds))))
+       *eggs+dirs+vers*)
+      (cleanup)
+      (exit 0)))
 
   (define (retrieve eggs)
     (print "retrieving ...")
@@ -320,10 +438,12 @@
                 ;; push to front
                 (set! *eggs+dirs+vers* (cons a (delete a *eggs+dirs+vers* eq?))) ) ]
              [else
-              (let ([name (if (pair? egg) (car egg) egg)]
-                    [version (and (pair? egg) (cdr egg))])
-                (let-values ([(dir ver) (try-default-sources name version)])
-                  (unless dir (error "extension or version not found"))
+              (let ((name (if (pair? egg) (car egg) egg))
+                    (version (override-version egg)))
+                (let-values (((dir ver) (try-default-sources name version)))
+                  (when (or (not dir)
+                            (null? (directory dir)))
+                    (error "extension or version not found"))
                   (print " " name " located at " dir)
                   (set! *eggs+dirs+vers* (cons (list name dir ver) *eggs+dirs+vers*)) ) ) ] ) )
      eggs)
@@ -332,13 +452,14 @@
        (lambda (e+d+v)
          (unless (member (car e+d+v) *checked*)
            (set! *checked* (cons (car e+d+v) *checked*))
-           (let ([mfile (make-pathname (cadr e+d+v) (car e+d+v) "meta")])
+           (let ((mfile (make-pathname (cadr e+d+v) (car e+d+v) "meta")))
              (cond [(file-exists? mfile)
-                    (let ([meta (with-input-from-file mfile read)])
+                    (let ((meta (with-input-from-file mfile read)))
 		      (print "checking platform for `" (car e+d+v) "' ...")
 		      (check-platform (car e+d+v) meta)
                       (print "checking dependencies for `" (car e+d+v) "' ...")
-                      (let-values ([(missing upgrade) (outdated-dependencies meta)])
+                      (let-values (((missing upgrade) 
+				    (outdated-dependencies (car e+d+v) meta)))
 			(set! missing (apply-mappings missing)) ;XXX only missing - wrong?
 			(set! *dependencies*
 			  (cons
@@ -357,7 +478,7 @@
                                        (yes-or-no?
                                         (make-replace-extension-question e+d+v upgrade)
                                         "no"
-					abort: abort-setup) ) )
+					abort: (abort-setup) ) ) )
                           (let ([ueggs (unzip1 upgrade)])
                             (print " upgrade: " (string-intersperse ueggs ", "))
                             (for-each
@@ -395,8 +516,9 @@
     (conc
      *csi*
      " -bnq "
-     (if (and *cross-chicken* ; disable -setup-mode when cross-compiling,
-	      (not *host-extension*)) ; host-repo must always take precedence
+     (if (or *deploy*
+	     (and *cross-chicken* ; disable -setup-mode when cross-compiling,
+		  (not *host-extension*))) ; host-repo must always take precedence
 	 ""
 	 "-setup-mode ")
      "-e \"(require-library setup-api)\" -e \"(import setup-api)\" "
@@ -414,6 +536,11 @@
 	   (sprintf " -e \"(destination-prefix \\\"~a\\\")\"" 
 	     (normalize-pathname prefix 'unix))
 	   ""))
+     (let ((prefix (get-prefix #t)))
+       (if prefix
+	   (sprintf " -e \"(runtime-prefix \\\"~a\\\")\"" 
+	     (normalize-pathname prefix 'unix))
+	   ""))
      (if (pair? *csc-features*)
 	 (sprintf " -e \"(extra-features '~s)\"" *csc-features*)
 	 "")
@@ -439,6 +566,11 @@
 	     (tmp))))))
 
   (define (install eggs)
+    (when *keep-existing*
+      (set! eggs
+	(remove 
+	 (lambda (egg) (extension-information (if (pair? egg) (car egg) egg)))
+	 eggs)))
     (retrieve eggs)
     (unless *retrieve-only*
       (let* ((dag (reverse (topological-sort *dependencies* string=?)))
@@ -448,16 +580,17 @@
 	(pp dag)
 	(for-each
 	 (lambda (e+d+v i)
-	   (let ((isdep (> i 1)))
-	     (when (and (not depinstall-ok)
-			isdep
-			(= i num))
+	   (let ((isdep (and (pair? eggs)
+			     (not (find (lambda (e)
+					  (equal? (if (pair? e) (car e) e) (car e+d+v)))
+					eggs)))))
+	     (when (and (not depinstall-ok) isdep)
 	       (when (and *no-install*
 			   (not (yes-or-no?
 				 (string-append
 				  "You specified `-no-install', but this extension has dependencies"
 				  " that are required for building.\nDo you still want to install them?")
-				 abort: abort-setup)))
+				 abort: (abort-setup))))
 		 (print "aborting installation.")
 		 (cleanup)
 		 (exit 1)))
@@ -515,17 +648,17 @@
     (let* ((files (glob (make-pathname (repository-path) "*.import.*")))
            (tmpdir (create-temporary-directory))
            (dbfile (make-pathname tmpdir +module-db+))
-           (rx (regexp ".*/([^/]+)\\.import\\.(scm|so)")))
+           (rx (irregex ".*/([^/]+)\\.import\\.(scm|so)")))
       (print "loading import libraries ...")
       (fluid-let ((##sys#warnings-enabled #f))
         (for-each
          (lambda (f)
-           (let ((m (string-match rx f)))
+           (let ((m (irregex-match rx f)))
 	     (handle-exceptions ex
 		 (print-error-message 
 		  ex (current-error-port) 
 		  (sprintf "Failed to import from `~a'" f))
-	       (eval `(import ,(string->symbol (cadr m)))))))
+	       (eval `(import ,(string->symbol (irregex-match-substring m 1)))))))
          files))
       (print "generating database")
       (let ((db
@@ -550,8 +683,13 @@
         (remove-directory tmpdir))))
 
   (define (apply-mappings eggs)
+    (define (canonical x)
+      (cond ((symbol? x) (cons (symbol->string x) #f))
+	    ((string? x) (cons x #f))
+	    ((pair? x) x)
+	    (else (error "internal error - bad egg spec" x))))
     (define (same? e1 e2)
-      (string=? (->string e1) (->string e2)))
+      (equal? (car (canonical e1)) (car (canonical e2))))
     (let ((eggs2
 	   (delete-duplicates
 	    (append-map
@@ -561,11 +699,17 @@
 			    (lambda (m) (map ->string (cdr m))))
 		     (else (list egg))))
 	     eggs)
-	    string=?)))
+	    same?)))
       (unless (lset= same? eggs eggs2)
 	(print "mapped " eggs " to " eggs2))
       eggs2))
 
+  (define (scan-directory dir)
+    (for-each
+     (lambda (info)
+       (pp (cons (car info) (cadadr info))))
+     (gather-egg-information dir)))      
+
   (define ($system str)
     (let ((r (system
               (if *windows-shell*
@@ -574,6 +718,37 @@
       (unless (zero? r)
         (error "shell command terminated with nonzero exit code" r str))))
 
+  (define (installed-extensions)
+    (delete-duplicates
+     (filter-map
+      (lambda (sf)
+	(let* ((info (first (read-file sf)))
+	       (v (cond ((assq 'version info) => cadr)
+			(else ""))))
+	  (cond ((assq 'egg-name info) => 
+		 (lambda (a) (cons (cadr a) (->string v))))
+		(else 
+		 (warning 
+		  "installed extension has no information about which egg it belongs to"
+		  (pathname-file sf))
+		 #f))))
+      (glob (make-pathname (repo-path) "*" "setup-info")))
+     equal?))
+
+  (define (list-available-extensions trans locn)
+    (with-default-sources
+     (lambda (trans locn next)
+       (if trans
+	   (list-extensions
+	    trans locn
+	    quiet: #t
+	    username: *username*
+	    password: *password*
+	    proxy-host: *proxy-host*
+	    proxy-port: *proxy-port*
+	    proxy-user-pass: *proxy-user-pass*)
+	   (next)))))
+
   (define (command fstr . args)
     (let ((cmd (apply sprintf fstr args)))
       (print "  " cmd)
@@ -584,9 +759,11 @@
 usage: chicken-install [OPTION | EXTENSION[:VERSION]] ...
 
   -h   -help                    show this message and exit
-  -v   -version                 show version and exit
+       -version                 show version and exit
        -force                   don't ask, install even if versions don't match
   -k   -keep                    keep temporary files
+  -x   -keep-installed          install only if not already installed
+       -reinstall               reinstall all currently installed extensions
   -l   -location LOCATION       install from given location instead of default
   -t   -transport TRANSPORT     use given transport instead of default
        -proxy HOST[:PORT]       download via HTTP proxy
@@ -594,6 +771,7 @@ usage: chicken-install [OPTION | EXTENSION[:VERSION]] ...
   -r   -retrieve                only retrieve egg into current directory, don't install
   -n   -no-install              do not install, just build (implies `-keep')
   -p   -prefix PREFIX           change installation prefix to PREFIX
+       -list                    list extensions available over selected transport and location
        -host                    when cross-compiling, compile extension only for host
        -target                  when cross-compiling, compile extension only for target
        -test                    run included test-cases, if available
@@ -607,25 +785,42 @@ usage: chicken-install [OPTION | EXTENSION[:VERSION]] ...
   -D   -feature FEATURE         features to pass to sub-invocations of `csc'
        -debug                   enable full display of error message information
        -keep-going              continue installation even if dependency fails
+       -scan DIRECTORY          scan local directory for highest available egg versions
+       -override FILENAME       override versions for installed eggs with information from file
+       -csi FILENAME            use given pathname for invocations of "csi"
+       -show-depends            display a list of egg dependencies for the given egg(s)
+       -show-foreign-depends    display a list of foreign dependencies for the given egg(s)
+
+chicken-install recognizes the http_proxy, and proxy_auth environment variables, if set.
+
 EOF
 );|
     (exit code))
 
   (define (setup-proxy uri)
     (if (string? uri)
-        (cond ((string-match "(.+)\\:([0-9]+)" uri) =>
-               (lambda (m)
-                 (set! *proxy-host* (cadr m))
-                 (set! *proxy-port* (string->number (caddr m))))
-               (else
-                (set! *proxy-host* uri)
-                (set! *proxy-port* 80))))))
+        (begin 
+          (set! *proxy-user-pass* (get-environment-variable "proxy_auth"))
+          (cond ((irregex-match "(.+)\\:([0-9]+)" uri) =>
+                 (lambda (m)
+                   (set! *proxy-host* (irregex-match-substring m 1))
+                   (set! *proxy-port* (string->number (irregex-match-substring m 2))))
+                 (else
+                  (set! *proxy-host* uri)
+                  (set! *proxy-port* 80)))))))
+
+  (define (info->egg info)
+    (if (member (cdr info) '("" "unknown" "trunk"))
+	(car info)
+	info))
   
   (define *short-options* '(#\h #\k #\l #\t #\s #\p #\r #\n #\v #\i #\u #\D))
 
   (define (main args)
     (let ((update #f)
-          (rx (regexp "([^:]+):(.+)")))
+	  (scan #f)
+	  (listeggs #f)
+          (rx (irregex "([^:]+):(.+)")))
       (setup-proxy (get-environment-variable "http_proxy"))
       (let loop ((args args) (eggs '()))
         (cond ((null? args)
@@ -633,20 +828,33 @@ EOF
 		      (error 
 		       "`-deploy' only makes sense in combination with `-prefix DIRECTORY`"))
 		     (update (update-db))
+		     (scan (scan-directory scan))
                      (else
 		      (let ((defaults (load-defaults)))
 			(when (null? eggs)
-			  (let ((setups (glob "*.setup")))
-			    (cond ((pair? setups)
-				   (set! *eggs+dirs+vers*
-				     (append
-				      (map
-				       (lambda (s) (cons (pathname-file s) (list "." "")))
-				       setups)
-				      *eggs+dirs+vers*)))
-				  (else
-				   (print "no setup-scripts to process")
-				   (exit 1))) ) )
+			  (cond (*reinstall*
+				 (let ((egginfos (installed-extensions)))
+				   (if (or *force*
+					   (yes-or-no? 
+					    (sprintf
+						"About to re-install all ~a currently installed extensions - do you want to proceed?"
+					      (length egginfos))
+					    abort: #f))
+				       (set! eggs (map info->egg egginfos))
+				       (exit 1))))
+				((not listeggs)
+				 (let ((setups (glob "*.setup")))
+				   (cond ((pair? setups)
+					  (set! *eggs+dirs+vers*
+					    (append
+					     (map
+					      (lambda (s) 
+						(cons (pathname-file s) (list "." "")))
+					      setups)
+					     *eggs+dirs+vers*)))
+					 (else
+					  (print "no setup-scripts to process")
+					  (exit 1))) ) )))
 			(unless defaults
 			  (unless *default-transport*
 			    (error
@@ -654,7 +862,17 @@ EOF
 			  (unless *default-location*
 			    (error
 			     "no default location defined - please use `-location' option")))
-			(install (apply-mappings (reverse eggs)))))))
+                        (cond (listeggs
+                               (display
+                                (list-available-extensions
+                                 *default-transport* *default-location*)))
+                              (*show-depends*
+                               (show-depends eggs 'depends))
+                              (*show-foreign-depends*
+                               (show-depends eggs 'foreign-depends))
+                              (else
+                               (install (apply-mappings (reverse eggs)))))
+                        ))))
               (else
                (let ((arg (car args)))
                  (cond ((or (string=? arg "-help")
@@ -697,7 +915,8 @@ EOF
                         (set! *keep* #t)
                         (set! *no-install* #t)
                         (loop (cdr args) eggs))
-                       ((or (string=? arg "-v") (string=? arg "-version"))
+                       ((or (string=? arg "-v") ; DEPRECATED
+			    (string=? arg "-version"))
                         (print (chicken-version))
                         (exit 0))
                        ((or (string=? arg "-u") (string=? arg "-update-db"))
@@ -740,16 +959,43 @@ EOF
                         (unless (pair? (cdr args)) (usage 1))
                         (set! *username* (cadr args))
                         (loop (cddr args) eggs))
+		       ((string=? "-scan" arg)
+                        (unless (pair? (cdr args)) (usage 1))
+			(set! scan (cadr args))
+			(loop (cddr args) eggs))
+		       ((string=? "-override" arg)
+                        (unless (pair? (cdr args)) (usage 1))
+			(set! *override* (read-file (cadr args)))
+			(loop (cddr args) eggs))
+		       ((or (string=? "-x" arg) (string=? "-keep-installed" arg))
+			(set! *keep-existing* #t)
+			(loop (cdr args) eggs))
+		       ((string=? "-reinstall" arg)
+			(set! *reinstall* #t)
+			(loop (cdr args) eggs))
 		       ((string=? "-trunk" arg)
 			(set! *trunk* #t)
 			(loop (cdr args) eggs))
 		       ((string=? "-keep-going" arg)
 			(set! *keep-going* #t)
 			(loop (cdr args) eggs))
+		       ((string=? "-list" arg)
+			(set! listeggs #t)
+			(loop (cdr args) eggs))
+		       ((string=? "-csi" arg)
+			(unless (pair? (cdr args)) (usage 1))
+			(set! *csi* (cadr args))
+			(loop (cddr args) eggs))
                        ((string=? "-password" arg)
                         (unless (pair? (cdr args)) (usage 1))
                         (set! *password* (cadr args))
                         (loop (cddr args) eggs))
+		       ((string=? "-show-depends" arg)
+                        (set! *show-depends* #t)
+                        (loop (cdr args) eggs))
+                       ((string=? "-show-foreign-depends" arg)
+                        (set! *show-foreign-depends* #t)
+                        (loop (cdr args) eggs))
                        ((and (positive? (string-length arg))
                              (char=? #\- (string-ref arg 0)))
                         (if (> (string-length arg) 2)
@@ -776,9 +1022,14 @@ EOF
                               "")
                              *eggs+dirs+vers*))
                           (loop (cdr args) (cons egg eggs))))
-                       ((string-match rx arg) =>
+                       ((irregex-match rx arg) =>
                         (lambda (m)
-                          (loop (cdr args) (alist-cons (cadr m) (caddr m) eggs))))
+                          (loop 
+			   (cdr args) 
+			   (alist-cons
+			    (irregex-match-substring m 1)
+			    (irregex-match-substring m 2)
+			    eggs))))
                        (else (loop (cdr args) (cons arg eggs))))))))))
 
   (register-feature! 'chicken-install)
