@@ -1,6 +1,6 @@
 ;;;; chicken-profile.scm - Formatted display of profile outputs - felix -*- Scheme -*-
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -61,6 +61,7 @@ Usage: chicken-profile [FILENAME | OPTION] ...
  the highest modification time, in case multiple profiles exist.
 
 EOF
+;|
 )
  (exit 64) )
 
@@ -91,7 +92,8 @@ EOF
 	      (if (and n (> n 0)) n (error "invalid argument to option" arg))))
 	  (cond 
 	   [(member arg '("-h" "-help" "--help")) (print-usage)]
-	   [(member arg '("-v" "-version"))
+	   [(member arg '("-v" 		; DEPRECATED
+			  "-version"))
 	    (print "chicken-profile - Version " (chicken-version))
 	    (exit) ]
 	   [(string=? arg "-release")
@@ -182,28 +184,28 @@ EOF
 
 (define (write-profile)
   (print "reading `" file "' ...\n")
-  (let* ([data0 (with-input-from-file file read-profile)]
-	 [max-t (fold (lambda (t result)
-			(max (third t) result))
-		      0
-		      data0)]
-	 [data (sort (map
-		      (lambda (t) (append t (let ((c (second t))
-						  (t (third t)))
-					      (list (or (and c (> c 0) (/ t c))
-							0)
-						    (or (and (> max-t 0) (* (/ t max-t) 100))
-							0)
-						    ))))
+  (let* ((data0 (with-input-from-file file read-profile))
+	 (max-t (foldl (lambda (r t) (max r (third t))) 0 data0))
+	 (data (sort (map
+		      (lambda (t)
+			(append
+			 t
+			 (let ((c (second t)) ; count
+			       (t (third t))) ; total time
+			   (list (or (and c (> c 0) (/ t c)) ; time / count
+				     0)
+				 (or (and (> max-t 0) (* (/ t max-t) 100)) ; % of max-time
+				     0)
+				 ))))
 		      data0)
-                     sort-by)])
+                     sort-by)))
     (if (< 0 top (length data))
 	(set! data (take data top)))
     (set! data (map (lambda (entry)
-		      (let ([c (second entry)]
-			    [t (third entry)]
-			    [a (cadddr entry)]
-			    [p (list-ref entry 4)] )
+		      (let ([c (second entry)] ; count
+			    [t (third entry)]  ; total time
+			    [a (fourth entry)] ; average time
+			    [p (fifth entry)] ) ; % of max time
 			(list (##sys#symbol->qualified-string (first entry))
 			      (if (not c) "overflow" (number->string c))
 			      (format-real (/ t 1000) seconds-digits)
