@@ -1,6 +1,6 @@
 ;;;; chicken-status.scm
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -24,14 +24,14 @@
 ; POSSIBILITY OF SUCH DAMAGE.
 
 
-(require-library setup-api srfi-1 posix data-structures utils ports regex files)
+(require-library setup-api srfi-1 posix data-structures utils ports irregex files)
 
 
 (module main ()
   
   (import scheme chicken foreign)
-  (import srfi-1 posix data-structures utils ports regex
-	  files setup-api)
+  (import srfi-1 posix data-structures utils ports irregex
+	  files setup-api extras)
 
   (define-foreign-variable C_TARGET_LIB_HOME c-string)
   (define-foreign-variable C_BINARY_VERSION int)
@@ -45,13 +45,18 @@
 	(make-pathname C_TARGET_LIB_HOME (sprintf "chicken/~a" C_BINARY_VERSION))
 	(repository-path)))
 
+  (define (grep rx lst)
+    (filter (cut irregex-search rx <>) lst))
+
   (define (gather-eggs patterns)
-    (let ((eggs (map pathname-file 
-		     (glob (make-pathname (repo-path) "*" "setup-info")))))
+    (let ((eggs (gather-all-eggs)))
       (delete-duplicates
        (concatenate (map (cut grep <> eggs) patterns))
        string=?)))
 
+  (define (gather-all-eggs)
+    (map pathname-file (glob (make-pathname (repo-path) "*" "setup-info"))))
+
   (define (format-string str cols #!optional right (padc #\space))
     (let* ((len (string-length str))
 	   (pad (make-string (fxmax 0 (fx- cols len)) padc)) )
@@ -97,16 +102,24 @@
        eggs)
       string<?)))
 
+  (define (dump-installed-versions)
+    (for-each
+     (lambda (egg)
+       (let ((version (assq 'version (read-info egg (repo-path)))))
+	 (pp (list (string->symbol egg) (->string (and version (cadr version)))))))
+     (gather-all-eggs)))
+
   (define (usage code)
     (print #<<EOF
 usage: chicken-status [OPTION | PATTERN] ...
 
   -h   -help                    show this message
-  -v   -version                 show version and exit
+       -version                 show version and exit
   -f   -files                   list installed files
        -exact                   treat PATTERN as exact match (not a pattern)
        -host                    when cross-compiling, show status of host extensions only
        -target                  when cross-compiling, show status of target extensions only
+       -list                    dump installed eggs and their versions in "override" format
 EOF
 );|
     (exit code))
@@ -115,6 +128,7 @@ EOF
 
   (define (main args)
     (let ((files #f)
+	  (dump #f)
 	  (exact #f))
       (let loop ((args args) (pats '()))
 	(if (null? args)
@@ -122,19 +136,19 @@ EOF
 		   (lambda ()
 		     (let* ((patterns
 			     (map
-			      regexp
+			      irregex
 			      (cond ((null? pats) '(".*"))
-				    ;;XXX change for total-irregex branch:
 				    (exact (map (lambda (p)
-						  (string-append "^" (regexp-escape p) "$"))
+						  (string-append "^" (irregex-quote p) "$"))
 						pats))
-				    (else pats))))
+				    (else (map ##sys#glob->regexp pats)))))
 			    (eggs (gather-eggs patterns)))
 		       (if (null? eggs)
 			   (print "(none)")
 			   ((if files list-installed-files list-installed-eggs)
 			    eggs))))))
-	      (cond ((and *host-extensions* *target-extensions*)
+	      (cond (dump (dump-installed-versions))
+	            ((and *host-extensions* *target-extensions*)
 		     (print "host at " (repo-path) ":\n")
 		     (status)
 		     (fluid-let ((*host-extensions* #f))
@@ -155,10 +169,14 @@ EOF
 		    ((string=? arg "-exact")
 		     (set! exact #t)
 		     (loop (cdr args) pats))
+		    ((string=? arg "-list")
+		     (set! dump #t)
+		     (loop (cdr args) pats))
 		    ((or (string=? arg "-f") (string=? arg "-files"))
 		     (set! files #t)
 		     (loop (cdr args) pats))
-		    ((or (string=? arg "-v") (string=? arg "-version"))
+		    ((or (string=? arg "-v") ; DEPRECATED
+			 (string=? arg "-version"))
 		     (print (chicken-version))
 		     (exit 0))
 		    ((and (positive? (string-length arg))
