@@ -1,6 +1,6 @@
 ;;;; chicken-syntax.scm - non-standard syntax extensions
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -57,48 +57,67 @@
  'define-record '()
  (##sys#er-transformer
   (lambda (x r c)
-    (##sys#check-syntax 'define-record x '(_ symbol . #(symbol 0)))
+    (##sys#check-syntax 'define-record x '(_ symbol . _))
     (let* ((name (cadr x))
 	   (slots (cddr x))
 	   (prefix (symbol->string name))
-	   (setters (memq #:record-setters ##sys#features))
 	   (%define (r 'define))
-	   (%getter-with-setter (r 'getter-with-setter)))
+	   (%setter (r 'setter))
+	   (%getter-with-setter (r 'getter-with-setter))
+	   (slotnames
+	    (map (lambda (slot)
+		   (cond ((symbol? slot) slot)
+			 ((and (pair? slot)
+			       (c (car slot) %setter) 
+			       (pair? (cdr slot))
+			       (symbol? (cadr slot))
+			       (null? (cddr slot)))
+			  (cadr slot))
+			 (else
+			  (syntax-error 
+			   'define-record "invalid slot specification" slot))))
+		 slots)))
       `(##core#begin
-	  (,%define 
-	   ,(string->symbol (string-append "make-" prefix))
-	   (##core#lambda ,slots (##sys#make-structure (##core#quote ,name) ,@slots)) )
-	  (,%define
-	   ,(string->symbol (string-append prefix "?"))
-	   (##core#lambda (x) (##sys#structure? x ',name)) )
-	  ,@(let mapslots ((slots slots) (i 1))
-	      (if (eq? slots '())
-		  slots
-		  (let* ((slotname (symbol->string (##sys#slot slots 0)))
-			 (setr (string->symbol (string-append prefix "-" slotname "-set!")))
-			 (getr (string->symbol (string-append prefix "-" slotname)) ) )
-		    (cons
-		     `(##core#begin
-			(,%define
-			 ,setr
-			 (##core#lambda 
+	(,%define 
+	 ,(string->symbol (string-append "make-" prefix))
+	 (##core#lambda 
+	  ,slotnames
+	  (##sys#make-structure (##core#quote ,name) ,@slotnames)))
+	(,%define
+	 ,(string->symbol (string-append prefix "?"))
+	 (##core#lambda (x) (##sys#structure? x ',name)) )
+	,@(let mapslots ((slots slots) (i 1))
+	    (if (eq? slots '())
+		slots
+		(let* ((a (car slots))
+		       (has-setter (not (symbol? a)))
+		       (slotname (symbol->string (if has-setter (cadr a) a)))
+		       (setr (string->symbol (string-append prefix "-" slotname "-set!")))
+		       (getr (string->symbol (string-append prefix "-" slotname)))
+		       (setrcode
+			`(##core#lambda 
 			  (x val)
 			  (##core#check (##sys#check-structure x (##core#quote ,name)))
-			  (##sys#block-set! x ,i val) ) )
-			(,%define
-			 ,getr
-			 ,(if setters
-			      `(,%getter-with-setter
-				(##core#lambda
-				 (x) 
-				 (##core#check (##sys#check-structure x (##core#quote ,name)))
-				 (##sys#block-ref x ,i) )
-				,setr)
-			      `(##core#lambda 
-				(x)
-				(##core#check (##sys#check-structure x (##core#quote ,name)))
-				(##sys#block-ref x ,i) ) ) ) )
-		     (mapslots (##sys#slot slots 1) (fx+ i 1)) ) ) ) ) ) ) ) ) )
+			  (##sys#block-set! x ,i val) ) ))
+		  (cons
+		   `(##core#begin
+		     ,@(if has-setter
+			   '()
+			   `((,%define ,setr ,setrcode)))
+		     (,%define
+		      ,getr
+		      ,(if has-setter
+			   `(,%getter-with-setter
+			     (##core#lambda
+			      (x) 
+			      (##core#check (##sys#check-structure x (##core#quote ,name)))
+			      (##sys#block-ref x ,i) )
+			     ,setrcode)
+			   `(##core#lambda 
+			     (x)
+			     (##core#check (##sys#check-structure x (##core#quote ,name)))
+			     (##sys#block-ref x ,i) ) ) ) )
+		   (mapslots (##sys#slot slots 1) (fx+ i 1)) ) ) ) ) ) ) ) ) )
 
 (##sys#extend-macro-environment
  'receive
@@ -122,22 +141,22 @@
 (##sys#extend-macro-environment
  'time '()
  (##sys#er-transformer
- (lambda (form r c)
-   (let ((rvar (r 't)))
-    `(##core#begin
-       (##sys#start-timer)
-       (##sys#call-with-values 
-	(##core#lambda () ,@(cdr form))
-	(##core#lambda 
-	 ,rvar
-	 (##sys#display-times (##sys#stop-timer))
-	 (##sys#apply ##sys#values ,rvar) ) ) ) ) ) ) )
+  (lambda (form r c)
+    (let ((rvar (r 't)))
+      `(##core#begin
+	(##sys#start-timer)
+	(##sys#call-with-values 
+	 (##core#lambda () ,@(cdr form))
+	 (##core#lambda 
+	  ,rvar
+	  (##sys#display-times (##sys#stop-timer))
+	  (##sys#apply ##sys#values ,rvar) ) ) ) ) ) ) )
 
 (##sys#extend-macro-environment
  'declare '()
  (##sys#er-transformer
- (lambda (form r c)
-   `(##core#declare ,@(cdr form)))))
+  (lambda (form r c)
+    `(##core#declare ,@(cdr form)))))
 
 (##sys#extend-macro-environment
  'include '()
@@ -153,19 +172,16 @@
     (lambda (form r c)
       (##sys#check-syntax 'assert form '#(_ 1))
       (let* ((exp (cadr form))
-	     (ln (get-line-number form))
 	     (msg-and-args (cddr form))
-	     (msg  (if (null? msg-and-args)
-		       "assertion failed"
-		       (car msg-and-args)))
-	     (msg (if ln
-		      (string-append "(" ln ") " msg)
-		      msg)))
+	     (msg (optional msg-and-args "assertion failed")))
+	(when (string? msg)
+	  (and-let* ((ln (get-line-number form)))
+	    (set! msg (string-append "(" ln ") " msg))))
 	`(##core#if (##core#check ,exp)
 		    (##core#undefined)
 		    (##sys#error 
 		     ,msg 
-		     ,@(if (fx> (length msg-and-args) 1)
+		     ,@(if (pair? msg-and-args)
 			   (cdr msg-and-args)
 			   `((##core#quote ,(##sys#strip-syntax exp)))))))))))
 
@@ -187,8 +203,8 @@
 		    #:type-error
 		    ,@(if (pair? args)
 			  args
-			  `((##core#immutable '"argument has incorrect type")
-			    ,tmp ',pred) ) ) ) ) ) ) ) )
+			  `((##core#immutable (##core#quote "argument has incorrect type"))
+			    ,tmp (##core#quote ,pred)) ) ) ) ) ) ) ) )
 
 (##sys#extend-macro-environment
  'fluid-let '()
@@ -256,29 +272,37 @@
  'parameterize '()
  (##sys#er-transformer
   (lambda (form r c)
+    (define (pname p)
+      (if (symbol? p)
+	  (gensym p)
+	  (gensym "parameter")))
     (##sys#check-syntax 'parameterize form '#(_ 2))
-     (let* ((bindings (cadr form))
-	    (body (cddr form))
-	    (swap (r 'swap))
-	    [params (##sys#map car bindings)]
-	    [vals (##sys#map cadr bindings)]
-	    [aliases (##sys#map (lambda (z) (r (gensym))) params)]
-	    [aliases2 (##sys#map (lambda (z) (r (gensym))) params)] )
-       `(##core#let
-	 ,(##sys#append 
-	   (map ##sys#list aliases params)
-	   (map ##sys#list aliases2 vals))
+    (let* ((bindings (cadr form))
+	   (body (cddr form))
+	   (swap (r 'swap))
+	   (mode (r 'mode))
+	   (params (##sys#map car bindings))
+	   (vals (##sys#map cadr bindings))
+	   (aliases (##sys#map (lambda (z) (r (pname z))) params))
+	   (aliases2 (##sys#map (lambda (z) (r (gensym))) params)) )
+      `(##core#let
+	,(map ##sys#list aliases params)
+	(##core#let
+	 ,(map ##sys#list aliases2 vals)
+	 (##core#let
+	  ((,mode #f))
 	  (##core#let
 	   ((,swap (##core#lambda
 		    ()
 		    ,@(map (lambda (a a2)
-			     `(##core#let ((t (,a))) (,a ,a2)
+			     `(##core#let ((t (,a))) (,a ,a2 ,mode)
 					  (##core#set! ,a2 t)))
-			   aliases aliases2) ) ) )
+			   aliases aliases2)
+		    (##core#set! ,mode #t))))
 	   (##sys#dynamic-wind 
 	    ,swap
 	    (##core#lambda () ,@body)
-	    ,swap) ) ) ) )))
+	    ,swap) ) ) ) )))))
 
 (##sys#extend-macro-environment
  'when '()
@@ -320,13 +344,17 @@
 				    `(##core#set! ,v ,a))
 				  vars aliases) ) ) ) ) ) ))))
 
-(##sys#extend-macro-environment
- 'define-values '()
- (##sys#er-transformer
-  (lambda (form r c)
-    (##sys#check-syntax 'define-values form '(_ #(variable 0) _))
-    (for-each (cut ##sys#register-export <> (##sys#current-module)) (cadr form))
-    `(,(r 'set!-values) ,@(cdr form)))))
+(set! ##sys#define-values-definition
+  (##sys#extend-macro-environment
+   'define-values '()
+   (##sys#er-transformer
+    (lambda (form r c)
+      (##sys#check-syntax 'define-values form '(_ #(variable 0) _))
+      (for-each (lambda (nm)
+                  (let ((name (##sys#get nm '##core#macro-alias nm)))
+                    (##sys#register-export name (##sys#current-module))))
+                (cadr form))
+      `(,(r 'set!-values) ,@(cdr form))))))
 
 (##sys#extend-macro-environment
  'let-values '()
@@ -483,7 +511,7 @@
 	   (cond ((null? clauses)
 		  '(##core#undefined) )
 		 ((not (pair? clauses))
-		  (##sys#syntax-error 'select "invalid syntax" clauses))
+		  (syntax-error 'select "invalid syntax" clauses))
 		 (else
 		  (let ((clause (##sys#slot clauses 0))
 			(rclauses (##sys#slot clauses 1)) )
@@ -538,7 +566,6 @@
 ;;; The expression takes a rest list ARG-LIST and binds the VARi to
 ;;; the elements of the rest list. When there are no more elements, then
 ;;; the remaining VARi are bound to their corresponding DEFAULTi values.
-;;; It is an error if there are more args than variables.
 ;;;
 ;;; - The default expressions are *not* evaluated unless needed.
 ;;;
@@ -616,9 +643,7 @@
       (define (make-if-tree vars defaulters body-proc rest rename)
 	(let recur ((vars vars) (defaulters defaulters) (non-defaults '()))
 	  (if (null? vars)
-	      `(##core#if (##core#check (,(r 'null?) ,rest))
-		     (,body-proc . ,(reverse non-defaults))
-		     (##sys#error (##core#immutable '"too many optional arguments") ,rest))
+	      `(,body-proc . ,(reverse non-defaults))
 	      (let ((v (car vars)))
 		`(##core#if (null? ,rest)
 		       (,(car defaulters) . ,(reverse non-defaults))
@@ -670,7 +695,6 @@
 ;;;     (lambda (a b . r) ...)
 ;;; - If REST-ARG has 0 elements, evaluate DEFAULT-EXP and return that.
 ;;; - If REST-ARG has 1 element, return that element.
-;;; - If REST-ARG has >1 element, error.
 
 (##sys#extend-macro-environment
  'optional 
@@ -683,12 +707,8 @@
     (let ((var (r 'tmp)))
       `(##core#let ((,var ,(cadr form)))
 	(##core#if (,(r 'null?) ,var) 
-	      ,(optional (cddr form) #f)
-	      (##core#if (##core#check (,(r 'null?) (,(r 'cdr) ,var)))
-		    (,(r 'car) ,var)
-		    (##sys#error
-		     (##core#immutable '"too many optional arguments") 
-		     ,var))))))))
+		   ,(optional (cddr form) #f)
+		   (,(r 'car) ,var)))))))
 
 
 ;;; (LET-OPTIONALS* args ((var1 default1) ... [rest]) body1 ...)
@@ -703,8 +723,6 @@
 ;;;   VAR7, and ARGS has 9 values, then REST will be bound to the list of
 ;;;   the two values of ARGS. If ARGS is too short, causing defaults to
 ;;;   be used, then REST is bound to '().
-;;; - If there is no REST variable, then it is an error to have excess
-;;;   values in the ARGS list.
 
 (##sys#extend-macro-environment
  'let-optionals* 
@@ -723,11 +741,7 @@
 	  ((,rvar ,args))
 	  ,(let loop ([args rvar] [vardefs var/defs])
 	     (if (null? vardefs)
-		 `(##core#if (##core#check (,%null? ,args))
-			     (##core#let () ,@body)
-			     (##sys#error 
-			      (##core#immutable '"too many optional arguments") 
-			      ,args) )
+		 `(##core#let () ,@body)
 		 (let ([head (car vardefs)])
 		   (if (pair? head)
 		       (let ((rvar2 (r 'tmp2)))
@@ -748,7 +762,8 @@
  `((>= . ,(##sys#primitive-alias '>=))
    (car . ,(##sys#primitive-alias 'car))
    (cdr . ,(##sys#primitive-alias 'cdr))
-   (eq? . ,(##sys#primitive-alias 'eq?)))
+   (eq? . ,(##sys#primitive-alias 'eq?))
+   (length . ,(##sys#primitive-alias 'length)))
  (##sys#er-transformer
   (lambda (form r c)
     (##sys#check-syntax 'case-lambda form '(_ . _))
@@ -769,11 +784,12 @@
 	   (%>= (r '>=))
 	   (%eq? (r 'eq?))
 	   (%car (r 'car))
-	   (%cdr (r 'cdr)))
+	   (%cdr (r 'cdr))
+	   (%length (r 'length)))
       `(##core#lambda
 	,(append minvars rvar)
 	(##core#let
-	 ((,lvar (length ,rvar)))
+        ((,lvar (,%length ,rvar)))
 	 ,(fold-right
 	   (lambda (c body)
 	     (##sys#decompose-lambda-list
@@ -804,7 +820,7 @@
 				     bindings
 				     `(##core#let ,(map list vars1 minvars) ,bindings) ) ) )
 			    ,body) ) ) )
-	   '(##core#check (##sys#error (##core#immutable '"no matching clause in call to 'case-lambda' form")))
+	   '(##core#check (##sys#error (##core#immutable (##core#quote "no matching clause in call to 'case-lambda' form"))))
 	   (cdr form))))))))
 
 
@@ -969,6 +985,8 @@
     (let ((%<> (r '<>))
 	  (%<...> (r '<...>))
 	  (%apply (r 'apply)))
+      (when (null? (cdr form))
+        (syntax-error 'cut "you need to supply at least a procedure" form))
       (let loop ([xs (cdr form)] [vars '()] [vals '()] [rest #f])
 	(if (null? xs)
 	    (let ([rvars (reverse vars)]
@@ -978,11 +996,17 @@
 		    `(##core#lambda
 		      (,@rvars . ,rv)
 		      (,%apply ,(car rvals) ,@(cdr rvals) ,rv) ) )
+		  ;;XXX should we drop the begin?
 		  `(##core#lambda ,rvars ((##core#begin ,(car rvals)) ,@(cdr rvals)) ) ) )
 	    (cond ((c %<> (car xs))
 		   (let ([v (r (gensym))])
 		     (loop (cdr xs) (cons v vars) (cons v vals) #f) ) )
-		  ((c %<...> (car xs)) (loop '() vars vals #t))
+		  ((c %<...> (car xs))
+                   (if (null? (cdr xs))
+                       (loop '() vars vals #t)
+                       (syntax-error 'cut
+                                     "tail patterns after <...> are not supported"
+                                     form)))
 		  (else (loop (cdr xs) vars (cons (car xs) vals) #f)) ) ) ) ) )))
 
 (##sys#extend-macro-environment
@@ -993,6 +1017,8 @@
     (let ((%apply (r 'apply))
 	  (%<> (r '<>))
 	  (%<...> (r '<...>)))
+      (when (null? (cdr form))
+        (syntax-error 'cute "you need to supply at least a procedure" form))
       (let loop ([xs (cdr form)] [vars '()] [bs '()] [vals '()] [rest #f])
 	(if (null? xs)
 	    (let ([rvars (reverse vars)]
@@ -1008,7 +1034,12 @@
 	    (cond ((c %<> (car xs))
 		   (let ([v (r (gensym))])
 		     (loop (cdr xs) (cons v vars) bs (cons v vals) #f) ) )
-		  ((c %<...> (car xs)) (loop '() vars bs vals #t))
+		  ((c %<...> (car xs))
+                   (if (null? (cdr xs))
+                       (loop '() vars bs vals #t)
+                       (syntax-error 'cute
+                                     "tail patterns after <...> are not supported"
+                                     form)))
 		  (else 
 		   (let ([v (r (gensym))])
 		     (loop (cdr xs) 
@@ -1058,35 +1089,232 @@
 
 (##sys#extend-macro-environment
  'define-compiler-syntax '()
- (##sys#er-transformer
-  (syntax-rules ()
-    ((_ name)
-     (##core#define-compiler-syntax name #f))
-    ((_ (name . llist) body ...)
-     (define-compiler-syntax name (lambda llist body ...)))
-    ((_ name transformer)
-     (##core#define-compiler-syntax name transformer)))))
+ (syntax-rules ()
+   ((_ name)
+    (##core#define-compiler-syntax name #f))
+   ((_ (name . llist) body ...)		; DEPRECATED
+    (define-compiler-syntax name
+      (##sys#er-transformer (lambda llist body ...) 'name)))
+   ((_ name transformer)
+    (##core#define-compiler-syntax name transformer))))
 
 (##sys#extend-macro-environment
  'let-compiler-syntax '()
- (##sys#er-transformer
-  (syntax-rules ()
-    ((_ (binding ...) body ...)
-     (##core#let-compiler-syntax (binding ...) body ...)))))
+ (syntax-rules ()
+   ((_ (binding ...) body ...)
+    (##core#let-compiler-syntax (binding ...) body ...))))
 
 
-;;; Just in case someone forgets
+;;; interface definition
 
 (##sys#extend-macro-environment
- 'define-macro '()
+ 'define-interface '()
  (##sys#er-transformer
-  (lambda (form r c)
-    (syntax-error 'define-macro "`define-macro' is not supported - please use `define-syntax'"))))
+  (lambda (x r c)
+    (##sys#check-syntax 'define-interface x '(_ variable _))
+    (let ((name (##sys#strip-syntax (cadr x)))
+	  (%quote (r 'quote)))
+      (when (eq? '* name)
+	(syntax-error-hook
+	 'define-interface "`*' is not allowed as a name for an interface"))
+      `(##core#elaborationtimeonly
+	(##sys#put/restore!
+	 (,%quote ,name)
+	 (,%quote ##core#interface)
+	 (,%quote
+	  ,(let ((exps (##sys#strip-syntax (caddr x))))
+	     (cond ((eq? '* exps) '*)
+		   ((symbol? exps) `(#:interface ,exps))
+		   ((list? exps) 
+		    (##sys#validate-exports exps 'define-interface))
+		   (else
+		    (syntax-error-hook
+		     'define-interface "invalid exports" (caddr x))))))))))))
 
 
+;;; functor definition
+
+(##sys#extend-macro-environment
+ 'functor '()
+ (##sys#er-transformer
+  (lambda (x r c)
+    (##sys#check-syntax 'functor x '(_ (symbol . #((symbol _) 0)) _ . _))
+    (let* ((x (##sys#strip-syntax x))
+	   (head (cadr x))
+	   (name (car head))
+	   (exps (caddr x))
+	   (body (cdddr x))
+	   (registration
+	    `(##sys#register-functor
+	      ',name
+	      ',(map (lambda (arg)
+		       (let ((argname (car arg))
+			     (exps (##sys#validate-exports (cadr arg) 'functor)))
+			 (cons argname exps)))
+		     (cdr head))
+	      ',(##sys#validate-exports exps 'functor)
+	      ',body)))
+      `(##core#module
+	,name
+	#t
+	(import scheme chicken)
+	(begin-for-syntax ,registration))))))
+
+
+;;; type-related syntax
+
+(##sys#extend-macro-environment
+ ': '()
+ (##sys#er-transformer
+  (lambda (x r c)
+    (##sys#check-syntax ': x '(_ symbol _ . _))
+    (if (not (memq #:compiling ##sys#features)) 
+	'(##core#undefined)
+	(let* ((type1 (##sys#strip-syntax (caddr x)))
+	       (name1 (cadr x)))
+	  ;; we need pred/pure info, so not using "##compiler#check-and-validate-type"
+	  (let-values (((type pred pure)
+			(##compiler#validate-type type1 (##sys#strip-syntax name1))))
+	    (cond ((not type)
+		   (syntax-error ': "invalid type syntax" name1 type1))
+		  (else
+		   `(##core#declare 
+		     (type (,name1 ,type1 ,@(cdddr x)))
+		     ,@(if pure `((pure ,name1)) '())
+		     ,@(if pred `((predicate (,name1 ,pred))) '()))))))))))
+
+(##sys#extend-macro-environment
+ 'the '()
+ (##sys#er-transformer
+  (lambda (x r c)
+    (##sys#check-syntax 'the x '(_ _ _))
+    (if (not (memq #:compiling ##sys#features)) 
+	(caddr x)
+	`(##core#the ,(##compiler#check-and-validate-type (cadr x) 'the)
+		     #t
+		     ,(caddr x))))))
+
+(##sys#extend-macro-environment
+ 'assume '()
+ (syntax-rules ()
+   ((_ ((var type) ...) body ...)
+    (let ((var (the type var)) ...) body ...))))
+
+(##sys#extend-macro-environment
+ 'define-specialization '()
+ (##sys#er-transformer
+  (lambda (x r c)
+    (cond ((not (memq #:compiling ##sys#features)) '(##core#undefined))
+	  (else
+	   (##sys#check-syntax 'define-specialization x '(_ (variable . #(_ 0)) _ . #(_ 0 1)))
+	   (let* ((head (cadr x))
+		  (name (car head))
+		  (gname (##sys#globalize name '())) ;XXX correct?
+		  (args (cdr head))
+		  (alias (gensym name))
+		  (galias (##sys#globalize alias '())) ;XXX and this?
+		  (rtypes (and (pair? (cdddr x)) (##sys#strip-syntax (caddr x))))
+		  (%define (r 'define))
+		  (body (if rtypes (cadddr x) (caddr x))))
+	     (let loop ((args args) (anames '()) (atypes '()))
+	       (cond ((null? args)
+		      (let ((anames (reverse anames))
+			    (atypes (reverse atypes))
+			    (spec
+			     `(,galias ,@(let loop2 ((anames anames) (i 1))
+					   (if (null? anames)
+					       '()
+					       (cons (vector i)
+						     (loop2 (cdr anames) (fx+ i 1))))))))
+			(##sys#put! 
+			 gname '##compiler#local-specializations
+			 (##sys#append
+			  (list
+			   (cons atypes
+				 (if (and rtypes (pair? rtypes))
+				     (list
+				      (map (cut ##compiler#check-and-validate-type 
+					     <>
+					     'define-specialization)
+					   rtypes)
+				      spec)
+				     (list spec))))
+			  (or (##compiler#variable-mark 
+			       gname
+			       '##compiler#local-specializations)
+			      '())))
+			`(##core#begin
+			  (##core#declare (inline ,alias) (hide ,alias))
+			  (,%define (,alias ,@anames)
+				    (##core#let ,(map (lambda (an at)
+							(list an `(##core#the ,at #t ,an)))
+						      anames atypes)
+						,body)))))
+		     (else
+		      (let ((arg (car args)))
+			(cond ((symbol? arg)
+			       (loop (cdr args) (cons arg anames) (cons '* atypes)))
+			      ((and (list? arg) (fx= 2 (length arg)) (symbol? (car arg)))
+			       (loop
+				(cdr args)
+				(cons (car arg) anames)
+				(cons 
+				 (##compiler#check-and-validate-type 
+				  (cadr arg) 
+				  'define-specialization)
+				 atypes)))
+			      (else (syntax-error
+				     'define-specialization
+				     "invalid argument syntax" arg head)))))))))))))
+
+(##sys#extend-macro-environment
+ 'compiler-typecase '()
+ (##sys#er-transformer
+  (lambda (x r c)
+    (##sys#check-syntax 'compiler-typecase x '(_ _ . #((_ . #(_ 1)) 1)))
+    (let ((val (memq #:compiling ##sys#features))
+	  (var (gensym))
+	  (ln (get-line-number x)))
+      `(##core#let ((,var ,(cadr x)))
+		   (##core#typecase 
+		    ,ln
+		    ,var		; must be variable (see: CPS transform)
+		    ,@(map (lambda (clause)
+			     (let ((hd (##sys#strip-syntax (car clause))))
+			       (list
+				(if (eq? hd 'else)
+				    'else
+				    (if val
+					(##compiler#check-and-validate-type
+					 hd
+					 'compiler-typecase)
+					hd))
+				`(##core#begin ,@(cdr clause)))))
+			   (cddr x))))))))
+
+(##sys#extend-macro-environment
+ 'define-type '()
+ (##sys#er-transformer
+  (lambda (x r c)
+    (##sys#check-syntax 'define-type x '(_ variable _))
+    (cond ((not (memq #:compiling ##sys#features)) '(##core#undefined))
+	  (else
+	   (let ((name (##sys#strip-syntax (cadr x)))
+		 (%quote (r 'quote))
+		 (t0 (##sys#strip-syntax (caddr x))))
+	     `(##core#elaborationtimeonly
+	       (##sys#put/restore!
+		(,%quote ,name)
+		(,%quote ##compiler#type-abbreviation)
+		(,%quote ,(##compiler#check-and-validate-type t0 'define-type name))))))))))
+
+
+;; capture current macro env
+
 (##sys#macro-subset me0 ##sys#default-macro-environment)))
 
+
 ;; register features
 
 (eval-when (compile load eval)
-  (register-feature! 'srfi-8 'srfi-16 'srfi-26 'srfi-31 'srfi-15 'srfi-11) )
+  (register-feature! 'srfi-8 'srfi-11 'srfi-15 'srfi-16 'srfi-26 'srfi-31) )
