@@ -1,6 +1,6 @@
 ;;;; common-declarations.scm - settings for core libraries
 ;
-; Copyright (c) 2010, The CHICKEN Team
+; Copyright (c) 2011-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -25,8 +25,7 @@
 
 
 (declare 
-  (usual-integrations)
-  (hide d))
+  (usual-integrations))
 
 (cond-expand
  (debugbuild
@@ -38,10 +37,11 @@
        (when (##sys#fudge 13)
 	 (print arg1 more ...))))))
  (else
-  (declare
-    (safe-globals)
-    (no-bound-checks))
-  (define-syntax d (syntax-rules () ((_ . _) (void))))))
+  (begin
+    (declare
+      (safe-globals)
+      (no-bound-checks))
+    (define-syntax d (syntax-rules () ((_ . _) (void)))))))
 
 (define-syntax define-alias
   (syntax-rules ()
