@@ -1,6 +1,6 @@
 ;;;; compiler-namespace.scm - private namespace declarations for compiler units
 ;
-; Copyright (c) 2009-2010, The Chicken Team
+; Copyright (c) 2009-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -34,6 +34,7 @@
  block-compilation
  block-variable-literal-name
  block-variable-literal?
+ bootstrap-mode
  bomb
  broken-constant-nodes
  build-expression-tree
@@ -41,10 +42,12 @@
  build-node-graph
  c-ify-string
  callback-names
+ call-info
+ canonicalize-list-type
  canonicalize-begin-body
  canonicalize-expression
  check-and-open-input-file
- check-for-unsafe-toplevel-procedure-calls
+ check-and-validate-type
  check-signature
  chop-extension
  chop-separator
@@ -52,6 +55,7 @@
  close-checked-input-file
  collapsable-literal?
  collect!
+ collected-debugging-output
  compile-format-string
  compiler-arguments
  compiler-cleanup-hook
@@ -59,6 +63,7 @@
  compiler-syntax-enabled
  compiler-syntax-statistics
  compute-database-statistics
+ constant-form-eval
  constant-table
  constant?
  constants-used
@@ -76,11 +81,11 @@
  debugging
  debugging-chicken
  debugging-executable
+ determine-loop-and-dispatch
  decompose-lambda-list
  default-debugging-declarations
  default-declarations
  default-default-target-heap-size
- default-default-target-stack-size
  default-extended-bindings
  default-optimization-iterations
  default-optimization-passes
@@ -96,8 +101,6 @@
  display-analysis-database
  display-line-number-database
  display-real-name-table
- do-lambda-lifting
- do-scrutinize
  dump-defined-globals
  dump-global-refs
  dump-nodes
@@ -108,10 +111,11 @@
  emit-profile
  emit-syntax-trace-info
  emit-trace-info
+ emit-type-file
  enable-inline-files
+ enable-specialization
  encode-literal
  eq-inline-operator
- error-is-extended-binding
  estimate-foreign-result-location-size
  estimate-foreign-result-size
  expand-debug-assignment
@@ -147,6 +151,7 @@
  foreign-lambda-stubs
  foreign-result-conversion
  foreign-string-result-reserve
+ foreign-type->scrutiny-type
  foreign-type-check
  foreign-type-convert-argument
  foreign-type-convert-result
@@ -175,18 +180,19 @@
  inline-lambda-bindings
  inline-locally
  inline-max-size
- inline-output-file
  inline-substitutions-enabled
  inline-table
  inline-table-used
  inlining
  insert-timer-checks
+ install-specializations
  installation-home
  internal-bindings
  intrinsic?
  line-number-database-2
  line-number-database-size
  llist-length
+ llist-match?
  load-identifier-database
  load-inline-file
  load-type-database
@@ -217,13 +223,13 @@
  perform-cps-conversion
  perform-high-level-optimizations
  perform-inlining!
- perform-lambda-lifting!
  perform-pre-optimization!
- perform-unboxing!
+ posv
  posq
  postponed-initforms
  pprint-expressions-to-file
  prepare-for-code-generation
+ print-debug-options
  print-program-statistics
  print-usage
  print-version
@@ -235,6 +241,7 @@
  profiled-procedures
  put!
  qnode
+ quit
  r-c-s
  read/source-info
  real-name
@@ -251,32 +258,35 @@
  scan-toplevel-assignments
  scan-used-variables
  scrutinize
+ scrutiny-debug
  set-real-name!
  sexpr->node
  simple-lambda-node?
  simplifications
  simplified-ops
  simplify-named-call
+ simplify-type
+ slashify
  sort-symbols
  source-filename
  source-info->string
  source-info->line
+ specialize-node!
  standalone-executable
+ strict-variable-types
  string->c-identifier
  string->expr
  stringify
  substitution-table
  symbolify
- target-heap-growth
- target-heap-shrinkage
  target-heap-size
  target-include-file
- target-initial-heap-size
  target-stack-size
  toplevel-lambda-id
  toplevel-scope
  transform-direct-lambdas!
  tree-copy
+ uncommentify
  undefine-shadowed-macros
  unique-id
  unit-name
@@ -288,10 +298,12 @@
  valid-c-identifier?
  valid-compiler-options
  valid-compiler-options-with-argument
+ validate-type
  variable-mark
  variable-visible?
  varnode
  verbose-mode
+ with-debugging-output
  words
  words->bytes
  words-per-flonum
