@@ -1,6 +1,6 @@
 ;;;; compiler-syntax.scm - compiler syntax used internally
 ;
-; Copyright (c) 2009-2010, The Chicken Team
+; Copyright (c) 2009-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -43,11 +43,14 @@
 	(alist-update! name (add1 a) compiler-syntax-statistics)))))
 
 (define (r-c-s names transformer #!optional (se '()))
-  (let ((t (cons (##sys#er-transformer transformer) se)))
+  (let ((t (cons (##sys#ensure-transformer
+		  (##sys#er-transformer transformer)
+		  (car names))
+		 (append se ##sys#default-macro-environment))))
     (for-each
      (lambda (name)
        (##sys#put! name '##compiler#compiler-syntax t) )
-     (if (symbol? names) (list names) names) ) ) )
+     names) ) )
 
 (define-syntax define-internal-compiler-syntax
   (syntax-rules ()
@@ -60,9 +63,10 @@
   (pair?)
   (let ((%let (r 'let))
 	(%if (r 'if))
-	(%loop (r 'loop))
+	(%loop (r 'for-each-loop))
 	(%proc (gensym))
 	(%begin (r 'begin))
+	(%quote (r 'quote))
 	(%and (r 'and))
 	(%pair? (r 'pair?))
 	(%lambda (r 'lambda))
@@ -73,7 +77,7 @@
 	  `(,%let ((,%proc ,(cadr x))
 		   ,@(map list vars lsts))
 		  ,@(map (lambda (var)
-			   `(##core#check (##sys#check-list ,var 'for-each)))
+			   `(##core#check (##sys#check-list ,var (,%quote for-each))))
 			 vars)
 		  (,%let ,%loop ,(map list vars vars)
 			 (,%if (,%and ,@(map (lambda (v) `(,%pair? ,v)) vars))
@@ -86,10 +90,10 @@
 	x)))
 
 (define-internal-compiler-syntax ((map ##sys#map #%map) x r c)
-  (pair?)
+  (pair? cons)
   (let ((%let (r 'let))
 	(%if (r 'if))
-	(%loop (r 'loop))
+	(%loop (r 'map-loop))
 	(%res (gensym))
 	(%cons (r 'cons))
 	(%set! (r 'set!))
@@ -110,7 +114,7 @@
 		   (,%proc ,(cadr x))
 		   ,@(map list vars lsts))		   
 		  ,@(map (lambda (var)
-			   `(##core#check (##sys#check-list ,var 'map)))
+			   `(##core#check (##sys#check-list ,var (,%quote map))))
 			 vars)
 		  (,%let ,%loop ,(map list vars vars)
 			 (,%if (,%and ,@(map (lambda (v) `(,%pair? ,v)) vars))
@@ -133,7 +137,7 @@
   (if (and (fx> (length x) 1)
 	   (memq 'o extended-bindings) ) ; s.a.
       (let ((%tmp (r 'tmp)))
-	`(,(r 'lambda) (,%tmp) ,(fold-right list %tmp (cdr x))))
+	`(,(r 'lambda) (,%tmp) ,(fold-right list %tmp (cdr x)))) ;XXX use foldr
       x))
 
 (define-internal-compiler-syntax ((sprintf #%sprintf format #%format) x r c)
@@ -185,9 +189,9 @@
 	    (define (fail ret? msg . args)
 	      (let ((ln (get-line x)))
 		(warning 
-		 (sprintf "`~a', in format string ~s~a, ~?" 
+		 (sprintf "~a`~a', in format string ~s, ~?" 
+		   (if ln (sprintf "(~a) " ln) "")
 		   func fstr 
-		   (if ln (sprintf "(~a)" ln) "")
 		   msg args) ))
 	      (when ret? (return #f)))
 	    (let ((code '())
@@ -242,7 +246,7 @@
 				  (let* ([fstr (next)]
 					 [lst (next)] )
 				    (push `(##sys#apply ,%fprintf ,%out ,fstr ,lst))))
-				 ((#\~) (push `(,write-char #\~ ,%out)))
+				 ((#\~) (push `(,%write-char #\~ ,%out)))
 				 ((#\% #\N) (push `(,%write-char #\newline ,%out)))
 				 (else
 				  (if (char-whitespace? dchar)
@@ -253,3 +257,51 @@
 				      (fail #t "illegal format-string character `~c'" dchar) ) ) )
 			       (loop '()) )
 			     (loop (cons c chunk)))))))))))))
+
+(define-internal-compiler-syntax ((foldr #%foldr) x r c)
+  (pair?)
+  (if (and (fx= (length x) 4)
+	   (memq 'foldr extended-bindings) ) ; s.a.
+      (let ((f (cadr x))
+	    (z (caddr x))
+	    (lst (cadddr x))
+	    (%let* (r 'let*))
+	    (%let (r 'let))
+	    (%pair? (r 'pair?))
+	    (%if (r 'if))
+	    (loopvar (gensym "foldr"))
+	    (lstvar (gensym)))
+	`(,%let* ((,lstvar ,lst))
+		 (##core#check (##sys#check-list ,lstvar (##core#quote foldr)))
+		 (,%let ,loopvar ((,lstvar ,lstvar))
+			(,%if (,%pair? ,lstvar)
+			      (,f (##sys#slot ,lstvar 0) 
+				     (##core#app ,loopvar (##sys#slot ,lstvar 1)))
+			      ,z))))
+      x))
+
+(define-internal-compiler-syntax ((foldl #%foldl) x r c) 
+  (pair?)
+  (if (and (fx= (length x) 4)
+	   (memq 'foldl extended-bindings) ) ; s.a.
+      (let ((f (cadr x))
+	    (z (caddr x))
+	    (lst (cadddr x))
+	    (%let* (r 'let*))
+	    (%let (r 'let))
+	    (%if (r 'if))
+	    (%pair? (r 'pair?))
+	    (zvar (gensym))
+	    (loopvar (gensym "foldl"))
+	    (lstvar (gensym)))
+	`(,%let* ((,zvar ,z)
+		  (,lstvar ,lst))
+		 (##core#check (##sys#check-list ,lstvar (##core#quote foldl)))
+		 (,%let ,loopvar ((,lstvar ,lstvar) (,zvar ,zvar))
+			(,%if (,%pair? ,lstvar)
+			      (##core#app 
+			       ,loopvar
+			       (##sys#slot ,lstvar 1) 
+			       (,f ,zvar (##sys#slot ,lstvar 0)))
+			      ,zvar))))
+      x))
