@@ -1,6 +1,6 @@
 ;;;; csi.scm - Interpreter stub for CHICKEN
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -31,7 +31,9 @@
   (disable-interrupts)
   (compile-syntax)
   (foreign-declare #<<EOF
-#if (defined(_MSC_VER) && defined(_WIN32)) || defined(HAVE_DIRECT_H)
+#include <signal.h>
+
+#if defined(HAVE_DIRECT_H)
 # include <direct.h>
 #else
 # define _getcwd(buf, len)       NULL
@@ -43,10 +45,10 @@ EOF
 
 (private csi 
   print-usage print-banner
-  run hexdump del 
+  run hexdump
   parse-option-string chop-separator lookup-script-file
   report describe dump hexdump bytevector-data get-config
-  deldups tty-input?
+  tty-input?
   history-list history-count history-add history-ref history-clear history-show)
 
 (declare
@@ -55,7 +57,7 @@ EOF
   (hide parse-option-string bytevector-data member* canonicalize-args 
 	describer-table dirseparator? circular-list? improper-pairs?
 	show-frameinfo selected-frame select-frame copy-from-frame
-	findall command-table default-editor) )
+	findall command-table default-editor csi-eval) )
 
 
 ;;; Parameters:
@@ -89,7 +91,7 @@ usage: csi [FILENAME | OPTION ...]
   one of the following:
 
     -h  -help  --help             display this text and exit
-    -v  -version                  display version and exit
+        -version                  display version and exit
         -release                  print release number and exit
     -i  -case-insensitive         enable case-insensitive reading
     -e  -eval EXPRESSION          evaluate given expression
@@ -126,12 +128,12 @@ EOF
     --                            ignore all following options
 
 EOF
-) )
+) ) ;|  <--- for emacs font-lock
 
 (define (print-banner)
   (newline)
-  #; ;UNUSED
-  (when (and (tty-input?) (##sys#fudge 11))
+  ;;UNUSED BECAUSE IT IS STUPID
+  #;(when (and (tty-input?) (##sys#fudge 11))
     (let* ((t (string-copy +product+))
 	   (len (string-length t))
 	   (c (make-string len #\x08)))
@@ -205,7 +207,7 @@ EOF
 	(and (> (##sys#size name) 0)
 	     (cond [(dirseparator? (string-ref name 0)) (addext name)]
 		   [(string-index dirseparator? name)
-		    (and-let* ([p (_getcwd buf 256)])
+		    (let ((p (_getcwd buf 256)))
 		      (addext (string-append (chop-separator p) "/" name)) ) ]
 		   [(addext name)]
 		   [else
@@ -255,9 +257,14 @@ EOF
 	(##sys#error "history entry index out of range" index) ) ) )
 
 (repl-prompt
- (let ([sprintf sprintf])
+ (let ((sprintf sprintf))
    (lambda ()
-     (sprintf "#;~A> " history-count) ) ) )
+     (sprintf "#;~A~A> "
+       (let ((m (##sys#current-module)))
+	 (if m
+	     (sprintf "~a:" (##sys#module-name m))
+	     ""))
+       history-count))))
 
 (define (tty-input?)
   (or (##sys#fudge 12) (##sys#tty-port? ##sys#standard-input)) )
@@ -269,14 +276,19 @@ EOF
     (lambda ()
       (when (tty-input?) (old)) ) ) )
 
-(define command-table (make-vector 37 '()))
+(define command-table '())
 
 (define (toplevel-command name proc #!optional help)
   (##sys#check-symbol name 'toplevel-command)
   (when help (##sys#check-string help 'toplevel-command))
-  (##sys#hash-table-set! command-table name (cons proc help)) )
+  (cond ((assq name command-table) =>
+	 (lambda (a)
+	   (set-cdr! a (list proc help)) ))
+	(else
+	 (set! command-table (cons (list name proc help) command-table))))
+  (##sys#void))
 
-(set! ##sys#repl-eval-hook
+(set! csi-eval
   (let ((eval eval)
 	(load-noisily load-noisily)
 	(read read)
@@ -295,11 +307,12 @@ EOF
 	    ((and (pair? form)
 		  (eq? 'unquote (##sys#slot form 0)) )
 	     (let ((cmd (cadr form)))
-	       (cond ((and (symbol? cmd) (##sys#hash-table-ref command-table cmd)) =>
+	       (cond ((assq cmd command-table) =>
 		      (lambda (p)
-			((car p))
+			((cadr p))
 			(##sys#void) ) )
 		     (else
+		      ;;XXX use `toplevel-command' to define as many as possible of these
 		      (case cmd
 			((x)
 			 (let ([x (read)])
@@ -325,7 +338,7 @@ EOF
 				[xn (eval n)] )
 			   (dump xe xn) ) )
 			((r) (report))
-			((q) (exit))
+			((q) (##sys#quit-hook))
 			((l)
 			 (let ((fns (string-split (read-line))))
 			   (for-each load fns)
@@ -349,7 +362,7 @@ EOF
 				    (or (editor-command) default-editor)
 				    " " (read-line)))))
 			   (if (not (zero? r))
-			       (printf "Editor returned with non-zero exit status ~a" r))))
+			       (printf "editor returned with non-zero exit status ~a" r))))
 			((ch)
 			 (history-clear)
 			 (##sys#void))
@@ -392,16 +405,16 @@ EOF
  ,g NAME           Get variable NAME from current frame
  ,t EXP            Evaluate form and print elapsed time
  ,x EXP            Pretty print expanded expression EXP\n")
-			 (##sys#hash-table-for-each
-			  (lambda (k v) 
-			    (let ((help (cdr v)))
+			 (for-each
+			  (lambda (a) 
+			    (let ((help (caddr a)))
 			      (if help
 				  (print #\space help)
-				  (print " ," k) ) ) )
+				  (print " ," (car a)) ) ) )
 			  command-table)
 			 (##sys#void) )
 			(else
-			 (printf "Undefined toplevel command ~s - enter `,?' for help~%" form) 
+			 (printf "undefined toplevel command ~s - enter `,?' for help~%" form) 
 			 (##sys#void) ) ) ) ) ) )
 	    (else
 	     (receive rs (eval form)
@@ -409,6 +422,29 @@ EOF
 	       (apply values rs) ) ) ) ) ) )
 
 
+;;; Builtin toplevel commands:
+
+(toplevel-command
+ 'm 
+ (let ((printf printf))
+   (lambda ()
+     (let ((name (read)))
+       (when (string? name)
+	 (set! name (##sys#string->symbol name)))
+       (cond ((not name)
+	      (##sys#switch-module #f)
+	      (printf "; resetting current module to toplevel~%"))
+	     ((not (symbol? name))
+	      (printf "invalid module name `~a'~%" name))
+	     ((##sys#find-module (##sys#resolve-module-name name #f) #f) =>
+	      (lambda (m)
+		(##sys#switch-module m)
+		(printf "; switching current module to `~a'~%" name)))
+	     (else
+	      (printf "undefined module `~a'~%" name))))))
+ ",m MODULE         switch to module with name `MODULE'")
+
+
 ;;; Parse options from string:
 
 (define (parse-option-string str)
@@ -444,15 +480,24 @@ EOF
 	  (let ([sinfo (##sys#symbol-table-info)]
 		[minfo (memory-statistics)] )
 	    (define (shorten n) (/ (truncate (* n 100)) 100))
-	    (printf "Features:\n")
-	    (for-each
-	     (lambda (lst) 
-	       (display "\n  ")
-	       (for-each 
-		(lambda (f)
-		  (printf "~a~a" f (make-string (fxmax 1 (fx- 16 (string-length f))) #\space)) )
-		lst) )
-	     (chop (sort (map keyword->string ##sys#features) string<?) 5))
+	    (printf "Features:~%~%")
+	    (let ((fs (sort (map keyword->string ##sys#features) string<?))
+		  (c 0))
+	      (for-each
+	       (lambda (f)
+		 (printf "  ~a" f)
+		 (let* ((len (string-length f))
+			(pad (- 16 len)))
+		   (set! c (add1 c))
+		   (when (<= pad 0)
+		     (set! c (add1 c))
+		     (set! pad (+ pad 18)))
+		   (cond ((>= c 3)
+			  (display "\n")
+			  (set! c 0))
+			 (else 
+			  (display (make-string pad #\space))))))
+	       fs))
 	    (printf "~%~%~
                    Machine type:    \t~A ~A~%~
                    Software type:   \t~A~%~
@@ -540,7 +585,10 @@ EOF
 		   (let ((v (pref x (fx+ start i))))
 		     (let loop2 ((n 1) (j (fx+ i (fx+ start 1))))
 		       (cond ((fx>= j len)
-			      (fprintf out " ~S: ~S" i v)
+			      (##sys#with-print-length-limit
+			       1000
+			       (lambda ()
+				 (fprintf out " ~S: ~S" i v)))
 			      (if (fx> n 1)
 				  (fprintf out "\t(followed by ~A identical instance~a)~% ...~%" 
 					   (fx- n 1)
@@ -560,7 +608,7 @@ EOF
 	    [(eof-object? x) (fprintf out "end-of-file object~%")]
 	    [(eq? (##sys#void) x) (fprintf out "unspecified object~%")]
 	    [(fixnum? x)
-	     (fprintf out "exact integer ~S, #x~X, #o~O, #b~B" x x x x)
+	     (fprintf out "exact integer ~S~%  #x~X~%  #o~O~%  #b~B" x x x x)
 	     (let ([code (integer->char x)])
 	       (when (fx< x #x10000) (fprintf out ", character ~S" code)) )
 	     (##sys#write-char-0 #\newline ##sys#standard-output) ]
@@ -609,13 +657,9 @@ EOF
 	    [(pair? x) (fprintf out "pair with car ~S~%and cdr ~S~%" (car x) (cdr x))]
 	    [(procedure? x)
 	     (let ([len (##sys#size x)])
-	       (if (and (> len 3)
-			(memq #:tinyclos ##sys#features)
-			(eq? ##tinyclos#entity-tag (##sys#slot x (fx- len 1))) ) ;XXX handle this in tinyclos egg (difficult)
-		   (describe-object x out)
-		   (descseq 
-		    (sprintf "procedure with code pointer ~X" (##sys#peek-unsigned-integer x 0))
-		    ##sys#size ##sys#slot 1) ) ) ]
+	       (descseq 
+		(sprintf "procedure with code pointer 0x~X" (##sys#peek-unsigned-integer x 0))
+		##sys#size ##sys#slot 1) ) ]
 	    [(port? x)
 	     (fprintf out
 		      "~A port of type ~A with name ~S and file pointer ~X~%"
@@ -623,8 +667,6 @@ EOF
 		      (##sys#slot x 7)
 		      (##sys#slot x 3)
 		      (##sys#peek-unsigned-integer x 0) ) ]
-	    [(and (memq #:tinyclos ##sys#features) (instance? x)) ; XXX put into tinyclos egg
-	     (describe-object x out) ]
 	    [(##sys#locative? x)
 	     (fprintf out "locative~%  pointer ~X~%  index ~A~%  type ~A~%"
 		      (##sys#peek-unsigned-integer x 0)
@@ -670,11 +712,13 @@ EOF
 		(let loop ((props (##sys#slot x 2)))
 		  (unless (null? props)
 		    (when (eq? k (caar props))
-		      (fprintf out "\t~s: ~s~%" (cdar props) (cadr props)) )
+		      (##sys#with-print-length-limit
+		       100
+		       (lambda ()
+			 (fprintf out "\t~s: ~s" (cdar props) (cadr props)) ))
+		      (newline out))
 		    (loop (cddr props)) ) ) )
 	      (##sys#slot x 1) ) ]
-	    [(and (##sys#structure? x 'meroon-instance) (provided? 'meroon)) ; XXX put this into meroon egg (really!)
-	     (unveil x out) ]
 	    [(##sys#generic-structure? x)
 	     (let ([st (##sys#slot x 0)])
 	       (cond ((##sys#hash-table-ref describer-table st) => (cut <> x out))
@@ -688,7 +732,7 @@ EOF
       (##sys#void) ) ) )
 
 (define (set-describer! tag proc)
-  (##sys#check-symbol tag 'symbol 'set-describer!)
+  (##sys#check-symbol tag 'set-describer!)
   (##sys#hash-table-set! describer-table tag proc) )
 
 
@@ -816,6 +860,7 @@ EOF
 
 (define copy-from-frame
   (let ((display display)
+	(newline newline)
 	(call/cc call/cc))
     (lambda (name)
       (let* ((ct (or ##sys#repl-recent-call-chain '()))
@@ -824,56 +869,61 @@ EOF
 	      (cond ((symbol? name) (##sys#slot name 1)) ; name
 		    ((string? name) name)
 		    (else 
-		     (display "string or symbol required for `,cf'\n")
+		     (display "string or symbol required for `,g'\n")
 		     #f))))
+	(define (compare sym)
+	  (let ((str (##sys#slot sym 1))) ; name
+	    (string=?
+	     name
+	     (substring str 0 (min (string-length name) (string-length str))))))
 	(if name
 	    (call/cc
 	     (lambda (return)
 	       (define (fail msg)
 		 (display msg)
+		 (newline)
 		 (return (##sys#void)))
-	       (do ((ct ct (cdr ct))
-		    (i (fx- len 1) (fx- i 1)))
-		   ((null? ct) (fail "no environment in frame\n")) 
+	       (do ((ct ct (cdr ct)))
+		   ((null? ct) (fail "no environment in frame")) 
+		 ;;XXX this should be refactored as it duplicates the code above
 		 (let* ((info (car ct))
 			(here (eq? selected-frame info))
 			(data (##sys#slot info 2)) ; cooked2 (cntr/frameinfo)
-			(finfo (##sys#structure? data 'frameinfo))
-			(cntr (if finfo (##sys#slot data 1) data))) ; cntr
+			(finfo (##sys#structure? data 'frameinfo)))
 		   (when (and here finfo)
 		     (for-each
 		      (lambda (e v)
 			(do ((i 0 (fx+ i 1))
 			     (be e (cdr be)))
-			    ((null? be) (fail "no such variable\n"))
-			  (when (string=? name (##sys#slot (car be) 1)) ; name
+			    ((null? be))
+			  (when (compare (car be))
+			    (display "; getting ")
+			    (display (car be))
+			    (newline)
 			    (history-add (list (##sys#slot v i)))
 			    (return (##sys#slot v i)))))
 		      (##sys#slot data 2)	; e
-		      (##sys#slot data 3))))))) ; v
+		      (##sys#slot data 3))	; v
+		     (fail (##sys#string-append "no such variable: " name)))))))
 	    (##sys#void))))))
 
 
+;;; Handle some signals:
+
+(define-foreign-variable _sigint int "SIGINT")
+
+(define-syntax defhandler 
+  (syntax-rules ()
+    ((_ sig handler)
+     (begin
+       (##core#inline "C_establish_signal_handler" sig sig)
+       (##sys#setslot ##sys#signal-vector sig handler)))))
+
+(defhandler _sigint (lambda (n) (##sys#user-interrupt-hook)))
+
+
 ;;; Start interpreting:
 
-(define (del x lst tst)
-  (let loop ([lst lst])
-    (if (null? lst)
-	'()
-	(let ([y (car lst)])
-	  (if (tst x y)
-	      (cdr lst)
-	      (cons y (loop (cdr lst))) ) ) ) ) )
-
-(define (deldups lis . maybe-=)
-  (let ((elt= (optional maybe-= equal?)))
-    (let recur ((lis lis))
-      (if (null? lis) lis
-	  (let* ((x (car lis))
-		 (tail (cdr lis))
-		 (new-tail (recur (del x tail elt=))))
-	    (if (eq? tail new-tail) lis (cons x new-tail)))))))
-
 (define (member* keys set)
   (let loop ((set set))
     (and (pair? set)
@@ -938,7 +988,8 @@ EOF
 	     (##sys#error "missing or invalid script argument"))
 	   (program-name (cadr script))
 	   (command-line-arguments (cddr script))
-	   (register-feature! 'script)
+	   (register-feature! 'script)	; DEPRECATED
+	   (register-feature! 'chicken-script)
 	   (set-cdr! (cdr script) '()) 
 	   (when ##sys#windows-platform
 	     (and-let* ((sname (lookup-script-file (cadr script))))
@@ -951,7 +1002,10 @@ EOF
 	   [batch (or script (member* '("-b" "-batch") args) eval?)]
 	   [quietflag (member* '("-q" "-quiet") args)]
 	   [quiet (or script quietflag eval?)]
-	   [ipath (map chop-separator (string-split (or (get-environment-variable "CHICKEN_INCLUDE_PATH") "") ";"))] )      
+	   [ipath (map chop-separator 
+		       (string-split 
+			(or (get-environment-variable "CHICKEN_INCLUDE_PATH") "") 
+			";"))] )      
       (define (collect-options opt)
 	(let loop ([opts args])
 	  (cond [(member opt opts) 
@@ -976,7 +1030,9 @@ EOF
       (when (member* '("-h" "-help" "--help") args)
 	(print-usage)
 	(exit 0) )
-      (when (member* '("-v" "-version") args)
+      (when (member* '("-v"		; DEPRECATED
+		       "-version") 
+		     args)
 	(print-banner)
 	(exit 0) )
       (when (member "-setup-mode" args)
@@ -998,7 +1054,7 @@ EOF
       (for-each register-feature! (collect-options "-D"))
       (for-each unregister-feature! (collect-options "-no-feature"))
       (set! ##sys#include-pathnames 
-	(deldups
+	(##sys#nodups
 	 (append (map chop-separator (collect-options "-include-path"))
 		 (map chop-separator (collect-options "-I"))
 		 ##sys#include-pathnames
@@ -1025,13 +1081,16 @@ EOF
 	(keyword-style #:none)
 	(parentheses-synonyms #f)
 	(symbol-escape #f) )
-      (unless (or (member* '("-n" "-no-init") args) script) (loadinit))
+      (unless (or (member* '("-n" "-no-init") args) script eval?) (loadinit))
       (when batch 
 	(set! ##sys#notices-enabled #f))
       (do ([args args (cdr args)])
 	  ((null? args)
 	   (unless batch 
-	     (repl)
+	     (call/cc
+	      (lambda (k)
+		(set! ##sys#quit-hook (lambda _ (k #f)))
+		(repl csi-eval)))
 	     (##sys#write-char-0 #\newline ##sys#standard-output) ) )
 	(let* ((arg (car args)))
 	  (cond ((member arg simple-options))
