@@ -1,6 +1,6 @@
 ;;;; data-structures.import.scm - import library for "data-structures" module
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -29,7 +29,8 @@
  '(->string
    alist-ref
    alist-update!
-   always?
+   alist-update
+   always?				; DEPRECATED
    any?
    atom?
    binary-search
@@ -48,30 +49,27 @@
    identity
    intersperse
    join
-   left-section				; DEPRECATED
    list->queue
    list-of?
    make-queue
    merge
    merge!
-   never?
-   none?
-   noop					;DEPRECATED
+   never?				; DEPRECATED
+   none?				; DEPRECATED
    o
-   project
    queue->list
    queue-add!
    queue-empty?
    queue-first
    queue-last
+   queue-length
    queue-push-back!
    queue-push-back-list!
    queue-remove!
    queue?
    rassoc
    reverse-string-append
-   right-section			; DEPRECATED
-   shuffle
+   shuffle				; DEPRECATED
    sort
    sort!
    sorted?
