@@ -1,6 +1,6 @@
 ;;; data-structures.scm - Optional data structures extensions
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without
@@ -26,11 +26,7 @@
 
 
 (declare
- (unit data-structures)
- (foreign-declare #<<EOF
-#define C_mem_compare(to, from, n)   C_fix(C_memcmp(C_c_string(to), C_c_string(from), C_unfix(n)))
-EOF
-) )
+ (unit data-structures))
 
 (include "common-declarations.scm")
 
@@ -42,9 +38,6 @@ EOF
 
 (define (identity x) x)
 
-(define (project n)			; DEPRECATED
-  (lambda args (list-ref args n)) )
-
 (define (conjoin . preds)
   (lambda (x)
     (let loop ([preds preds])
@@ -101,8 +94,6 @@ EOF
 	    [(pred (##sys#slot lst 0)) (loop (##sys#slot lst 1))]
 	    [else #f] ) ) ) )
 
-(define (noop . _) (void))		;DEPRECATED
-
 (define (each . procs)
   (cond ((null? procs) (lambda _ (void)))
 	((null? (##sys#slot procs 1)) (##sys#slot procs 0))
@@ -119,24 +110,9 @@ EOF
 
 (define (any? x) #t)
 
-(define (none? x) #f)
-
-(define (always? . _) #t)
-
-(define (never? . _) #f)
-
-(define (left-section proc . args)	;DEPRECATED
-  (##sys#check-closure proc 'left-section)
-  (lambda xs
-    (##sys#apply proc (##sys#append args xs)) ) )
-
-(define right-section			;DEPRECATED
-  (let ([##sys#reverse reverse])
-    (lambda (proc . args)
-      (##sys#check-closure proc 'right-section)
-      (let ([revdargs (##sys#reverse args)])
-        (lambda xs
-          (##sys#apply proc (##sys#reverse (##sys#append revdargs (##sys#reverse xs)))) ) ) ) ) )
+(define (none? x) #f)			; DEPRECATED
+(define (always? . _) #t)		; DEPRECATED
+(define (never? . _) #f)		; DEPRECATED
 
 
 ;;; List operators:
@@ -180,20 +156,19 @@ EOF
 		 (cons head (loop tail rest)) ) ) ] ) ) )
 
 (define chop
-  (let ([reverse reverse])
-    (lambda (lst n)
-      (##sys#check-exact n 'chop)
-      (when (fx<= n 0) (##sys#error 'chop "invalid numeric argument" n))
-      (let ([len (length lst)])
-	(let loop ([lst lst] [i len])
-	  (cond [(null? lst) '()]
-		[(fx< i n) (list lst)]
-		[else
-		 (do ([hd '() (cons (##sys#slot tl 0) hd)]
-		      [tl lst (##sys#slot tl 1)] 
-		      [c n (fx- c 1)] )
-		     ((fx= c 0)
-		      (cons (reverse hd) (loop tl (fx- i n))) ) ) ] ) ) ) ) ) )
+  (lambda (lst n)
+    (##sys#check-exact n 'chop)
+    (when (fx<= n 0) (##sys#error 'chop "invalid numeric argument" n))
+    (let ([len (length lst)])
+      (let loop ([lst lst] [i len])
+	(cond [(null? lst) '()]
+	      [(fx< i n) (list lst)]
+	      [else
+	       (do ([hd '() (cons (##sys#slot tl 0) hd)]
+		    [tl lst (##sys#slot tl 1)] 
+		    [c n (fx- c 1)] )
+		   ((fx= c 0)
+		    (cons (reverse hd) (loop tl (fx- i n))) ) ) ] ) ) ) ) )
 
 (define (join lsts . lst)
   (let ([lst (if (pair? lst) (car lst) '())])
@@ -223,7 +198,7 @@ EOF
 	       (cons (##sys#slot lst 0) (loop (##sys#slot blst 1) (##sys#slot lst 1)))]
 	      [else (loop (##sys#slot blst 1) (##sys#slot lst 1))] ) ) ) ) )
 
-(define shuffle
+(define shuffle				; DEPRECATED
   (lambda (l random)
     (let ((len (length l)))
       (map cdr
@@ -252,6 +227,19 @@ EOF
 	  lst)
 	(cons (cons x y) lst) ) ) )
 
+(define (alist-update k v lst #!optional (cmp eqv?))
+  (let loop ((lst lst))
+    (if (null? lst)
+        (list (cons k v))
+        (let ((a (##sys#slot lst 0)))
+          (cond ((not (pair? a))
+                 (error 'alist-update "bad argument type" a))
+                ((cmp (##sys#slot a 0) k)
+                 (cons (cons k v) (##sys#slot lst 1)))
+                (else
+                 (cons (cons (##sys#slot a 0) (##sys#slot a 1))
+                       (loop (##sys#slot lst 1)))))))))
+
 (define (alist-ref x lst #!optional (cmp eqv?) (default #f))
   (let* ([aq (cond [(eq? eq? cmp) assq]
 		   [(eq? eqv? cmp) assv]
@@ -285,41 +273,36 @@ EOF
 ; (reverse-string-append l) = (apply string-append (reverse l))
 
 (define (reverse-string-append l)
-
   (define (rev-string-append l i)
     (if (pair? l)
       (let* ((str (car l))
 	     (len (string-length str))
-	     (result (rev-string-append (cdr l) (+ i len))))
-	(let loop ((j 0) (k (- (- (string-length result) i) len)))
-	  (if (< j len)
+	     (result (rev-string-append (cdr l) (fx+ i len))))
+	(let loop ((j 0) (k (fx- (fx- (string-length result) i) len)))
+	  (if (fx< j len)
 	    (begin
 	      (string-set! result k (string-ref str j))
-	      (loop (+ j 1) (+ k 1)))
+	      (loop (fx+ j 1) (fx+ k 1)))
 	    result)))
       (make-string i)))
-
   (rev-string-append l 0))
 
 ;;; Anything->string conversion:
 
 (define ->string 
-  (let ([display display]
-	[string string])
-    (lambda (x)
-      (cond [(string? x) x]
-	    [(symbol? x) (symbol->string x)]
-	    [(char? x) (string x)]
-	    [(number? x) (##sys#number->string x)]
-	    [else 
-	     (let ([o (open-output-string)])
-	       (display x o)
-	       (get-output-string o) ) ] ) ) ) )
+  (lambda (x)
+    (cond [(string? x) x]
+	  [(symbol? x) (symbol->string x)]
+	  [(char? x) (string x)]
+	  [(number? x) (##sys#number->string x)]
+	  [else 
+	   (let ([o (open-output-string)])
+	     (display x o)
+	     (get-output-string o) ) ] ) ) )
 
 (define conc
-  (let ([string-append string-append])
-    (lambda args
-      (apply string-append (map ->string args)) ) ) )
+  (lambda args
+    (apply string-append (map ->string args)) ) )
 
 
 ;;; Search one string inside another:
@@ -365,7 +348,7 @@ EOF
   (let ((len1 (##sys#size s1))
 	(len2 (##sys#size s2)) )
     (let* ((len-diff (fx- len1 len2)) 
-	   (cmp (##core#inline "C_mem_compare" s1 s2 (if (fx< len-diff 0) len1 len2))))
+	   (cmp (##core#inline "C_string_compare" s1 s2 (if (fx< len-diff 0) len1 len2))))
       (if (fx= cmp 0) 
 	  len-diff 
 	  cmp))))
@@ -478,55 +461,53 @@ EOF
 ;;; Translate elements of a string:
 
 (define string-translate 
-  (let ([make-string make-string]
-	[list->string list->string] )
-    (lambda (str from . to)
+  (lambda (str from . to)
 
-      (define (instring s)
-	(let ([len (##sys#size s)])
-	  (lambda (c)
-	    (let loop ([i 0])
-	      (cond [(fx>= i len) #f]
-		    [(eq? c (##core#inline "C_subchar" s i)) i]
-		    [else (loop (fx+ i 1))] ) ) ) ) )
+    (define (instring s)
+      (let ([len (##sys#size s)])
+	(lambda (c)
+	  (let loop ([i 0])
+	    (cond [(fx>= i len) #f]
+		  [(eq? c (##core#inline "C_subchar" s i)) i]
+		  [else (loop (fx+ i 1))] ) ) ) ) )
 
-      (let* ([from
-	      (cond [(char? from) (lambda (c) (eq? c from))]
-		    [(pair? from) (instring (list->string from))]
-		    [else
-		     (##sys#check-string from 'string-translate)
-		     (instring from) ] ) ]
-	     [to
-	      (and (pair? to)
-		   (let ([tx (##sys#slot to 0)])
-		     (cond [(char? tx) tx]
-			   [(pair? tx) (list->string tx)]
-			   [else
-			    (##sys#check-string tx 'string-translate)
-			    tx] ) ) ) ] 
-	     [tlen (and (string? to) (##sys#size to))] )
-	(##sys#check-string str 'string-translate)
-	(let* ([slen (##sys#size str)]
-	       [str2 (make-string slen)] )
-	  (let loop ([i 0] [j 0])
-	    (if (fx>= i slen)
-		(if (fx< j i)
-		    (##sys#substring str2 0 j)
-		    str2)
-		(let* ([ci (##core#inline "C_subchar" str i)]
-		       [found (from ci)] )
-		  (cond [(not found)
-			 (##core#inline "C_setsubchar" str2 j ci)
-			 (loop (fx+ i 1) (fx+ j 1)) ]
-			[(not to) (loop (fx+ i 1) j)]
-			[(char? to)
-			 (##core#inline "C_setsubchar" str2 j to)
-			 (loop (fx+ i 1) (fx+ j 1)) ]
-			[(fx>= found tlen)
-			 (##sys#error 'string-translate "invalid translation destination" i to) ]
-			[else 
-			 (##core#inline "C_setsubchar" str2 j (##core#inline "C_subchar" to found))
-			 (loop (fx+ i 1) (fx+ j 1)) ] ) ) ) ) ) ) ) ) )
+    (let* ([from
+	    (cond [(char? from) (lambda (c) (eq? c from))]
+		  [(pair? from) (instring (list->string from))]
+		  [else
+		   (##sys#check-string from 'string-translate)
+		   (instring from) ] ) ]
+	   [to
+	    (and (pair? to)
+		 (let ([tx (##sys#slot to 0)])
+		   (cond [(char? tx) tx]
+			 [(pair? tx) (list->string tx)]
+			 [else
+			  (##sys#check-string tx 'string-translate)
+			  tx] ) ) ) ] 
+	   [tlen (and (string? to) (##sys#size to))] )
+      (##sys#check-string str 'string-translate)
+      (let* ([slen (##sys#size str)]
+	     [str2 (make-string slen)] )
+	(let loop ([i 0] [j 0])
+	  (if (fx>= i slen)
+	      (if (fx< j i)
+		  (##sys#substring str2 0 j)
+		  str2)
+	      (let* ([ci (##core#inline "C_subchar" str i)]
+		     [found (from ci)] )
+		(cond [(not found)
+		       (##core#inline "C_setsubchar" str2 j ci)
+		       (loop (fx+ i 1) (fx+ j 1)) ]
+		      [(not to) (loop (fx+ i 1) j)]
+		      [(char? to)
+		       (##core#inline "C_setsubchar" str2 j to)
+		       (loop (fx+ i 1) (fx+ j 1)) ]
+		      [(fx>= found tlen)
+		       (##sys#error 'string-translate "invalid translation destination" i to) ]
+		      [else 
+		       (##core#inline "C_setsubchar" str2 j (##core#inline "C_subchar" to found))
+		       (loop (fx+ i 1) (fx+ j 1)) ] ) ) ) ) ) ) ) )
 
 (define (string-translate* str smap)
   (##sys#check-string str 'string-translate*)
@@ -536,7 +517,7 @@ EOF
       (if (fx>= i len)
 	  (##sys#fragments->string
 	   total
-	   (reverse 
+	   (##sys#fast-reverse 
 	    (if (fx> i from) 
 		(cons (##sys#substring str from i) fs)
 		fs) ) )
@@ -788,21 +769,20 @@ EOF
 ;;; Binary search:
 
 (define binary-search
-  (let ([list->vector list->vector])
-    (lambda (vec proc)
-      (if (pair? vec)
-	  (set! vec (list->vector vec))
-	  (##sys#check-vector vec 'binary-search) )
-      (let ([len (##sys#size vec)])
-	(and (fx> len 0)
-	     (let loop ([ps 0]
-			[pe len] )
-	       (let ([p (fx+ ps (##core#inline "C_fixnum_shift_right" (fx- pe ps) 1))])
-		 (let* ([x (##sys#slot vec p)]
-			[r (proc x)] )
-		   (cond [(fx= r 0) p]
-			 [(fx< r 0) (and (not (fx= pe p)) (loop ps p))]
-			 [else (and (not (fx= ps p)) (loop p pe))] ) ) ) ) ) ) ) ) )
+  (lambda (vec proc)
+    (if (pair? vec)
+	(set! vec (list->vector vec))
+	(##sys#check-vector vec 'binary-search) )
+    (let ([len (##sys#size vec)])
+      (and (fx> len 0)
+	   (let loop ([ps 0]
+		      [pe len] )
+	     (let ([p (fx+ ps (##core#inline "C_fixnum_shift_right" (fx- pe ps) 1))])
+	       (let* ([x (##sys#slot vec p)]
+		      [r (proc x)] )
+		 (cond [(fx= r 0) p]
+		       [(fx< r 0) (and (not (fx= pe p)) (loop ps p))]
+		       [else (and (not (fx= ps p)) (loop p pe))] ) ) ) ) ) ) ) )
 
 
 
@@ -820,14 +800,18 @@ EOF
 ; is stored in the queue type so that datums can be added in constant
 ; time.
 
-(define (make-queue) (##sys#make-structure 'queue '() '()))
+(define (make-queue) (##sys#make-structure 'queue '() '() 0))
 (define (queue? x) (##sys#structure? x 'queue))
 
-(define (queue-empty? q)
+(define (queue-length q)		; thread-safe
+  (##sys#check-structure q 'queue 'queue-length)
+  (##sys#slot q 3))
+
+(define (queue-empty? q)		; thread-safe
   (##sys#check-structure q 'queue 'queue-empty?)
   (eq? '() (##sys#slot q 1)) )
 
-(define queue-first
+(define queue-first			; thread-safe
   (lambda (q)
     (##sys#check-structure q 'queue 'queue-first)
     (let ((first-pair (##sys#slot q 1)))
@@ -835,7 +819,7 @@ EOF
 	(##sys#error 'queue-first "queue is empty" q))
       (##sys#slot first-pair 0) ) ) )
 
-(define queue-last
+(define queue-last			; thread-safe
   (lambda (q)
     (##sys#check-structure q 'queue 'queue-last)
     (let ((last-pair (##sys#slot q 2)))
@@ -843,15 +827,16 @@ EOF
 	(##sys#error 'queue-last "queue is empty" q))
       (##sys#slot last-pair 0) ) ) )
 
-(define (queue-add! q datum)
+(define (queue-add! q datum)		; thread-safe
   (##sys#check-structure q 'queue 'queue-add!)
   (let ((new-pair (cons datum '())))
     (cond ((eq? '() (##sys#slot q 1)) (##sys#setslot q 1 new-pair))
 	  (else (##sys#setslot (##sys#slot q 2) 1 new-pair)) )
     (##sys#setslot q 2 new-pair) 
+    (##sys#setislot q 3 (fx+ (##sys#slot q 3) 1))
     (##core#undefined) ) )
 
-(define queue-remove!
+(define queue-remove!			; thread-safe
   (lambda (q)
     (##sys#check-structure q 'queue 'queue-remove!)
     (let ((first-pair (##sys#slot q 1)))
@@ -861,13 +846,17 @@ EOF
 	(##sys#setslot q 1 first-cdr)
 	(if (eq? '() first-cdr)
 	    (##sys#setslot q 2 '()) )
+	(##sys#setislot q 3 (fx- (##sys#slot q 3) 1))
 	(##sys#slot first-pair 0) ) ) ) )
 
 (define (queue->list q)
   (##sys#check-structure q 'queue 'queue->list)
-  (##sys#slot q 1) )
+  (let loop ((lst (##sys#slot q 1)) (lst2 '()))
+    (if (null? lst)
+	(##sys#fast-reverse lst2)
+	(loop (##sys#slot lst 1) (cons (##sys#slot lst 0) lst2)))))
 
-(define (list->queue lst0)
+(define (list->queue lst0)		
   (##sys#check-list lst0 'list->queue)
   (##sys#make-structure 
    'queue lst0
@@ -877,18 +866,20 @@ EOF
 	   ((eq? (##sys#slot lst 1) '()) lst)
 	 (if (or (not (##core#inline "C_blockp" lst))
 		 (not (##core#inline "C_pairp" lst)) )
-	     (##sys#error-not-a-proper-list lst0 'list->queue) ) ) ) ) )
+	     (##sys#error-not-a-proper-list lst0 'list->queue) ) ) )
+   (##sys#length lst0)) )
 
 
 ; (queue-push-back! queue item)
 ; Pushes an item into the first position of a queue.
 
-(define (queue-push-back! q item)
+(define (queue-push-back! q item)	; thread-safe
   (##sys#check-structure q 'queue 'queue-push-back!)
   (let ((newlist (cons item (##sys#slot q 1))))
     (##sys#setslot q 1 newlist)
     (if (eq? '() (##sys#slot q 2))
-	(##sys#setslot q 2 newlist))))
+	(##sys#setslot q 2 newlist))
+    (##sys#setislot q 3 (fx+ (##sys#slot q 3) 1))))
 
 ; (queue-push-back-list! queue item-list)
 ; Pushes the items in item-list back onto the queue,
@@ -906,4 +897,5 @@ EOF
 		       '()
 		       (last-pair newlist))))
     (##sys#setslot q 1 newlist)
-    (##sys#setslot q 2 newtail)))
+    (##sys#setslot q 2 newtail)
+    (##sys#setislot q 3 (fx+ (##sys#slot q 3) (##core#inline "C_i_length" itemlist)))))
