@@ -1,6 +1,6 @@
 ;;;; extras.import.scm - import library for "extras" module
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -33,8 +33,8 @@
    pretty-print-width
    printf
    random
-   random-seed				; DEPRECATED
    randomize
+   read-buffered
    read-byte
    read-file
    read-line
