@@ -1,6 +1,6 @@
 ;;; extras.scm - Optional non-standard extensions
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -41,14 +41,13 @@
 
 (define read-file
   (let ([read read]
-	[reverse reverse] 
 	[call-with-input-file call-with-input-file] )
     (lambda (#!optional (port ##sys#standard-input) (reader read) max)
       (define (slurp port)
 	(do ((x (reader port) (reader port))
 	     (i 0 (fx+ i 1))
 	     (xs '() (cons x xs)) )
-	    ((or (eof-object? x) (and max (fx>= i max))) (reverse xs)) ) )
+	    ((or (eof-object? x) (and max (fx>= i max))) (##sys#fast-reverse xs)) ) )
       (if (port? port)
 	  (slurp port)
 	  (call-with-input-file port slurp) ) ) ) )
@@ -56,16 +55,9 @@
 
 ;;; Random numbers:
 
-(define random-seed			;DEPRECATED
-  (let ((srand (foreign-lambda void "srand" unsigned-integer)))
-    (lambda n
-      (let ((t (if (null? n) (current-seconds) (car n))))
-	(##sys#check-integer t 'random-seed)
-	(srand t) ) ) ) )
-
 (define (randomize . n)
   (let ((nn (if (null? n)
-		(##sys#inexact->exact (fp/ (current-seconds) 1000.0)) ; wall clock time
+		(##sys#flo2fix (fp/ (current-seconds) 1000.0)) ; wall clock time
 		(car n))))
     (##sys#check-exact nn 'randomize)
     (##core#inline "C_randomize" nn) ) )
@@ -80,7 +72,7 @@
 ;;; Line I/O:
 
 (define read-line
-  (let ([make-string make-string])
+  (let ()
     (define (fixup str len)
       (##sys#substring
        str 0
@@ -91,7 +83,7 @@
       (let* ([parg (pair? args)]
 	     [p (if parg (car args) ##sys#standard-input)]
 	     [limit (and parg (pair? (cdr args)) (cadr args))])
-	(##sys#check-port p 'read-line)
+	(##sys#check-input-port p #t 'read-line)
 	(cond ((##sys#slot (##sys#slot p 2) 8) => (lambda (rl) (rl p limit)))
 	      (else
 	       (let* ((buffer-len (if limit limit 256))
@@ -114,32 +106,41 @@
 				      (##sys#substring buffer 0 i) ) ) ]
 			       [else
 				(when (fx>= i buffer-len)
-				  (set! buffer (##sys#string-append buffer (make-string buffer-len)))
+				  (set! buffer
+				    (##sys#string-append buffer (make-string buffer-len)))
 				  (set! buffer-len (fx+ buffer-len buffer-len)) )
 				(##core#inline "C_setsubchar" buffer i c)
 				(loop (fx+ i 1)) ] ) ) ) ) ) ) ) ) ) ) ) )
 
 (define read-lines
-  (let ((call-with-input-file call-with-input-file) 
-	(reverse reverse) )
-    (lambda port-and-max
-      (let* ((port (if (pair? port-and-max) (##sys#slot port-and-max 0) ##sys#standard-input))
-	     (rest (and (pair? port-and-max) (##sys#slot port-and-max 1)))
-	     (max (if (pair? rest) (##sys#slot rest 0) #f)) )
-	(define (doread port)
-	  (let loop ((lns '())
-		     (n (or max 1000000000)) ) ; this is silly
-	    (if (eq? n 0)
-		(reverse lns)
-		(let ((ln (read-line port)))
-		  (if (eof-object? ln)
-		      (reverse lns)
-		      (loop (cons ln lns) (fx- n 1)) ) ) ) ) )
-	(if (string? port)
-	    (call-with-input-file port doread)
-	    (begin
-	      (##sys#check-port port 'read-lines)
-	      (doread port) ) ) ) ) ) )
+  (lambda port-and-max
+    (let* ((port (if (pair? port-and-max) (##sys#slot port-and-max 0) ##sys#standard-input))
+	   (rest (and (pair? port-and-max) (##sys#slot port-and-max 1)))
+	   (max (if (pair? rest) (##sys#slot rest 0) #f)) )
+      (define (doread port)
+	(let loop ((lns '())
+		   (n (or max 1000000000)) ) ; this is silly
+	  (if (eq? n 0)
+	      (##sys#fast-reverse lns)
+	      (let ((ln (read-line port)))
+		(if (eof-object? ln)
+		    (##sys#fast-reverse lns)
+		    (loop (cons ln lns) (fx- n 1)) ) ) ) ) )
+      (if (string? port)
+	  (call-with-input-file port doread)
+	  (begin
+	    (##sys#check-input-port port #t 'read-lines)
+	    (doread port) ) ) ) ) )
+
+(define write-line
+  (lambda (str . port)
+    (let* ((p (if (##core#inline "C_eqp" port '())
+                  ##sys#standard-output
+                  (##sys#slot port 0) ) ))
+      (##sys#check-output-port p #t 'write-line)
+      (##sys#check-string str 'write-line)
+      ((##sys#slot (##sys#slot p 2) 3) p str) ; write-string method
+      (##sys#write-char-0 #\newline p))))
 
 
 ;;; Extended I/O 
@@ -147,9 +148,6 @@
 (define (##sys#read-string! n dest port start)
   (cond ((eq? n 0) 0)
 	(else
-	 (when (##sys#slot port 6)	; peeked?
-	   (##core#inline "C_setsubchar" dest start (##sys#read-char-0 port))
-	   (set! start (fx+ start 1)) )
 	 (let ((rdstring (##sys#slot (##sys#slot port 2) 7)))
            (if rdstring
 	       (let loop ((start start) (n n) (m 0))
@@ -173,12 +171,12 @@
 			 (else (fx+ n2 m))) )))))))
 
 (define (read-string! n dest #!optional (port ##sys#standard-input) (start 0))
-  (##sys#check-port port 'read-string!)
+  (##sys#check-input-port port #t 'read-string!)
   (##sys#check-string dest 'read-string!)
-  (when n
-    (##sys#check-exact n 'read-string!)
-    (when (fx> (fx+ start n) (##sys#size dest))
-      (set! n (fx- (##sys#size dest) start))))
+  (when n (##sys#check-exact n 'read-string!))
+  (let ((dest-size (##sys#size dest)))
+    (unless (and n (fx<= (fx+ start n) dest-size))
+      (set! n (fx- dest-size start))))
   (##sys#check-exact start 'read-string!)
   (##sys#read-string! n dest port start) )
 
@@ -186,7 +184,7 @@
 
 (define ##sys#read-string/port
   (lambda (n p)
-    (##sys#check-port p 'read-string)
+    (##sys#check-input-port p #t 'read-string)
     (cond (n (##sys#check-exact n 'read-string)
 	     (let* ((str (##sys#make-string n))
 		    (n2 (##sys#read-string! n str p 0)) )
@@ -208,10 +206,27 @@
 (define (read-string #!optional n (port ##sys#standard-input))
   (##sys#read-string/port n port) )
 
+;; <procedure>(read-buffered [PORT])</procedure>
+;;
+;; Reads any remaining data buffered after previous read operations on
+;; {{PORT}}. If no remaining data is currently buffered, an empty string
+;; is returned. This procedure will never block. Currently only useful for
+;; string-, process- and tcp ports.
+
+(define (read-buffered #!optional (port ##sys#standard-input))
+  (##sys#check-input-port port #t 'read-buffered)
+  (let ((rb (##sys#slot (##sys#slot port 2) 9))) ; read-buffered method
+    (if rb
+	(rb port)
+	"")))
+
+
+;;; read token of characters that satisfy a predicate
+
 (define read-token
   (lambda (pred . port)
     (let ([port (optional port ##sys#standard-input)])
-      (##sys#check-port port 'read-token)
+      (##sys#check-input-port port #t 'read-token)
       (let ([out (open-output-string)])
 	(let loop ()
 	  (let ([c (##sys#peek-char-0 port)])
@@ -222,35 +237,22 @@
 		(get-output-string out) ) ) ) ) ) ) )
 
 (define write-string 
-  (let ([display display])
-    (lambda (s . more)
-      (##sys#check-string s 'write-string)
-      (let-optionals more ([n #f] [port ##sys#standard-output])
-	(##sys#check-port port 'write-string)
-	(when n (##sys#check-exact n 'write-string))
-	(display 
-	 (if (and n (fx< n (##sys#size s)))
-	     (##sys#substring s 0 n)
-	     s)
-	 port) ) ) ) )
-
-(define write-line
-  (let ((display display)
-	(newline newline) )
-    (lambda (str . port)
-      (let ((p (if (##core#inline "C_eqp" port '())
-		   ##sys#standard-output
-		   (##sys#slot port 0) ) ) )
-	(##sys#check-port p 'write-line)
-	(##sys#check-string str 'write-line)
-	(display str p)
-	(newline p) ) ) ) )
+  (lambda (s . more)
+    (##sys#check-string s 'write-string)
+    (let-optionals more ([n #f] [port ##sys#standard-output])
+      (##sys#check-output-port port #t 'write-string)
+      (when n (##sys#check-exact n 'write-string))
+      ((##sys#slot (##sys#slot port 2) 3) ; write-string
+       port
+       (if (and n (fx< n (##sys#size s)))
+	   (##sys#substring s 0 n)
+	   s)))))
 
 
 ;;; Binary I/O
 
 (define (read-byte #!optional (port ##sys#standard-input))
-  (##sys#check-port port 'read-byte)
+  (##sys#check-input-port port #t 'read-byte)
   (let ((x (##sys#read-char-0 port)))
     (if (eof-object? x)
 	x
@@ -258,7 +260,7 @@
 
 (define (write-byte byte #!optional (port ##sys#standard-output))
   (##sys#check-exact byte 'write-byte)
-  (##sys#check-port port 'write-byte)
+  (##sys#check-output-port port #t 'write-byte)
   (##sys#write-char-0 (integer->char byte) port) )
 
 
@@ -329,16 +331,26 @@
 	    ((string? obj)      (if display?
 				    (out obj col)
 				    (let loop ((i 0) (j 0) (col (out "\"" col)))
-				      (if (and col (< j (string-length obj)))
+				      (if (and col (fx< j (string-length obj)))
 					  (let ((c (string-ref obj j)))
-					    (if (or (char=? c #\\)
-						    (char=? c #\"))
-						(loop j
-						      (+ j 1)
-						      (out "\\"
-							   (out (##sys#substring obj i j)
-								col)))
-						(loop i (+ j 1) col)))
+                                            (if (or (char=? c #\\)
+                                                    (char=? c #\"))
+                                                (loop j
+                                                      (+ j 1)
+                                                      (out "\\"
+                                                           (out (##sys#substring obj i j)
+                                                                col)))
+                                                (cond ((assq c '((#\tab . "\\t")
+                                                                 (#\newline . "\\n")
+                                                                 (#\return . "\\r")))
+                                                       =>
+                                                       (lambda (a)
+                                                         (let ((col2
+                                                                (out (##sys#substring obj i j) col)))
+                                                           (loop (fx+ j 1)
+                                                                 (fx+ j 1)
+                                                                 (out (cdr a) col2)))))
+                                                      (else (loop i (fx+ j 1) col)))))
 					  (out "\""
 					       (out (##sys#substring obj i j) col))))))
 	    ((char? obj)        (if display?
@@ -355,7 +367,6 @@
 					     (out (if (fx> code #xffff) "U" "u") col)
 					     (out (number->string code 16) col) ]
 					    [else (out (make-string 1 obj) col)] ) ) ) )
-	    ((eof-object? obj)  (out "#<eof>" col))
 	    ((##core#inline "C_undefinedp" obj) (out "#<unspecified>" col))
 	    ((##core#inline "C_anypointerp" obj) (out (##sys#pointer->string obj) col))
 	    ((eq? obj (##sys#slot '##sys#arbitrary-unbound-symbol 0))
@@ -366,15 +377,19 @@
 	       (out (get-output-string o) col) ) )
 	    ((port? obj) (out (string-append "#<port " (##sys#slot obj 3) ">") col))
 	    ((##core#inline "C_bytevectorp" obj)
-	     (if (##core#inline "C_permanentp" obj)
-		 (out "#<static blob of size" col)
-		 (out "#<blob of size " col) )
-	     (out (number->string (##core#inline "C_block_size" obj)) col)
-	     (out ">" col) )
+	     (out "#${" col)
+	     (let ((len (##sys#size obj)))
+	       (do ((i 0 (fx+ i 1)))
+		   ((fx>= i len))
+		 (let ((b (##sys#byte obj i)))
+		   (when (fx< b 16)
+		     (out "0" col))
+		   (out (##sys#number->string b 16) col)))
+	       (out "}" col)))
 	    ((##core#inline "C_lambdainfop" obj)
 	     (out "#<lambda info " col)
 	     (out (##sys#lambda-info->string obj) col)
-	     (out "#>" col) )
+	     (out ">" col) )
 	    (else (out "#<unprintable object>" col)) ) )
 
     (define (pp obj col)
@@ -517,7 +532,7 @@
       (define (pp-do expr col extra)
 	(pp-general expr col extra #f pp-expr-list pp-expr-list pp-expr))
 
-					; define formatting style (change these to suit your style)
+      ;; define formatting style (change these to suit your style)
 
       (define indent-general 2)
 
@@ -559,14 +574,11 @@
 ;;; Write simple formatted output:
 
 (define fprintf0
-  (let ((write write)
-	(newline newline)
-	(display display) )
-    (lambda (loc port msg args)
-      (when port (##sys#check-port port loc))
-      (let ((out (if (and port (##sys#tty-port? port))
-		     port
-		     (open-output-string))))
+  (lambda (loc port msg args)
+    (when port (##sys#check-output-port port #t loc))
+    (let ((out (if (and port (##sys#tty-port? port))
+		   port
+		   (open-output-string))))
       (let rec ([msg msg] [args args])
 	(##sys#check-string msg loc)
 	(let ((index 0)
@@ -612,7 +624,7 @@
 		(loop) ) ) ) ) )
       (cond ((not port) (get-output-string out))
 	    ((not (eq? out port))
-	     (##sys#print (get-output-string out) #f port) ) ) ) ) ) )
+	     (##sys#print (get-output-string out) #f port) ) ) ) ) )
 
 (define (fprintf port fstr . args)
   (fprintf0 'fprintf port fstr args) )
