@@ -1,6 +1,6 @@
 ;;;; files.scm - File and pathname operations
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -36,7 +36,8 @@
 
 (declare
   (unit files)
-  (uses regex data-structures)
+  (uses irregex data-structures)
+  (fixnum)
   (hide chop-pds absolute-pathname-root root-origin root-directory split-directory)
   (disable-interrupts) 
   (foreign-declare #<<EOF
@@ -67,95 +68,68 @@ EOF
     (and (file-exists? file) (delete-file file)) ) )
 
 ;;; file-copy and file-move : they do what you'd think.
-(define (file-copy origfile newfile #!optional (clobber #f) (blocksize 1024))
-    (##sys#check-string origfile 'file-copy)
-    (##sys#check-string newfile 'file-copy)
-    (##sys#check-number blocksize 'file-copy)
-    (or (and (integer? blocksize) (> blocksize 0))
-        (##sys#error (string-append
-                         "invalid blocksize given: not a positive integer - "
-                         (number->string blocksize))))
-    (or (file-exists? origfile)
-        (##sys#error (string-append "origfile does not exist - " origfile)))
-    (and (file-exists? newfile)
-         (or clobber
-             (##sys#error (string-append
-                              "newfile exists but clobber is false - "
-                              newfile))))
-    (let* ((i   (condition-case (open-input-file origfile)
-                    (val ()
-                        (##sys#error (string-append
-                                         "could not open origfile for read - "
-                                         origfile)))))
-           (o   (condition-case (open-output-file newfile)
-                    (val ()
-                        (##sys#error (string-append
-                                         "could not open newfile for write - "
-                                         newfile)))))
-           (s   (make-string blocksize)))
-        (let loop ((d (read-string! blocksize s i))
-                   (l 0))
-            (if (fx= 0 d)
-                (begin
-                    (close-input-port i)
-                    (close-output-port o)
-                    l)
-                (begin
-                    (condition-case (write-string s d o)
-                        (val ()
-                            (close-input-port i)
-                            (close-output-port o)
-                            (##sys#error (string-append
-                                             "error writing file starting at "
-                                             (number->string l)))))
-                    (loop (read-string! blocksize s i) (fx+ d l)))))))
 
-(define (file-move origfile newfile #!optional (clobber #f) (blocksize 1024))
-  (##sys#check-string origfile 'file-move)
-  (##sys#check-string newfile 'file-move)
-  (##sys#check-number blocksize 'file-move)
-  (or (and (integer? blocksize) (> blocksize 0))
-      (##sys#error (string-append
-		    "invalid blocksize given: not a positive integer - "
-		    (number->string blocksize))))
-  (or (file-exists? origfile)
-      (##sys#error (string-append "origfile does not exist - " origfile)))
+(define (file-copy origfile newfile #!optional (clobber #f) (blocksize 1024))
+  (##sys#check-string origfile 'file-copy)
+  (##sys#check-string newfile 'file-copy)
+  (##sys#check-number blocksize 'file-copy)
+  (unless (and (integer? blocksize) (> blocksize 0))
+    (##sys#error 
+     'file-copy 
+     "invalid blocksize given: not a positive integer"
+     blocksize))
   (and (file-exists? newfile)
        (or clobber
-	   (##sys#error (string-append
-			 "newfile exists but clobber is false - "
-			 newfile))))
-  (let* ((i   (condition-case (open-input-file origfile)
-		(val ()
-		     (##sys#error (string-append
-				   "could not open origfile for read - "
-				   origfile)))))
-	 (o   (condition-case (open-output-file newfile)
-		(val ()
-		     (##sys#error (string-append
-				   "could not open newfile for write - "
-				   newfile)))))
-	 (s   (make-string blocksize)))
+	   (##sys#error 
+	    'file-copy
+	    "newfile exists but clobber is false"
+	     newfile)))
+  (when (directory-exists? origfile)
+    (##sys#error 'file-copy "can not copy directories" origfile))
+  (let* ((i (open-input-file origfile #:binary))
+	 (o (open-output-file newfile #:binary))
+	 (s (make-string blocksize)))
     (let loop ((d (read-string! blocksize s i))
 	       (l 0))
       (if (fx= 0 d)
 	  (begin
 	    (close-input-port i)
 	    (close-output-port o)
-	    (condition-case (delete-file origfile)
-	      (val ()
-		   (##sys#error (string-append
-				 "could not remove origfile - "
-				 origfile))))
 	    l)
 	  (begin
-	    (condition-case (write-string s d o)
-	      (val ()
-		   (close-input-port i)
-		   (close-output-port o)
-		   (##sys#error (string-append
-				 "error writing file starting at "
-				 (number->string l)))))
+	    (write-string s d o)
+	    (loop (read-string! blocksize s i) (fx+ d l)))))))
+
+(define (file-move origfile newfile #!optional (clobber #f) (blocksize 1024))
+  (##sys#check-string origfile 'file-move)
+  (##sys#check-string newfile 'file-move)
+  (##sys#check-number blocksize 'file-move)
+  (unless (and (integer? blocksize) (> blocksize 0))
+    (##sys#error
+     'file-move
+     "invalid blocksize given: not a positive integer"
+     blocksize))
+  (when (directory-exists? origfile)
+    (##sys#error 'file-move "can not move directories" origfile))
+  (and (file-exists? newfile)
+       (or clobber
+	   (##sys#error 
+	    'file-move
+	    "newfile exists but clobber is false"
+	    newfile)))
+  (let* ((i (open-input-file origfile #:binary))
+	 (o (open-output-file newfile #:binary))
+	 (s (make-string blocksize)))
+    (let loop ((d (read-string! blocksize s i))
+	       (l 0))
+      (if (fx= 0 d)
+	  (begin
+	    (close-input-port i)
+	    (close-output-port o)
+	    (delete-file origfile)
+	    l)
+	  (begin
+	    (write-string s d o)
 	    (loop (read-string! blocksize s i) (fx+ d l)))))))
 
 ;;; Pathname operations:
@@ -170,71 +144,64 @@ EOF
 (define root-directory)
 
 (if ##sys#windows-platform
-    (let ((rx (regexp "([A-Za-z]:)?([\\/\\\\]).*")))
-      (set! absolute-pathname-root (lambda (pn) (string-match rx pn)))
-      (set! root-origin (lambda (rt) (and rt (cadr rt))))
-      (set! root-directory (lambda (rt) (and rt (caddr rt)))) )
-    (let ((rx (regexp "([\\/\\\\]).*")))
-      (set! absolute-pathname-root (lambda (pn) (string-match rx pn)))
+    (let ((rx (irregex "([A-Za-z]:)?([\\/\\\\]).*")))
+      (set! absolute-pathname-root (lambda (pn) (irregex-match rx pn)))
+      (set! root-origin (lambda (rt) (and rt (irregex-match-substring rt 1))))
+      (set! root-directory (lambda (rt) (and rt (irregex-match-substring rt 2)))) )
+    (let ((rx (irregex "([\\/\\\\]).*")))
+      (set! absolute-pathname-root (lambda (pn) (irregex-match rx pn)))
       (set! root-origin (lambda (rt) #f))
-      (set! root-directory (lambda (rt) (and rt (cadr rt)))) ) )
+      (set! root-directory (lambda (rt) (and rt (irregex-match-substring rt 1)))) ) )
 
 (define (absolute-pathname? pn)
   (##sys#check-string pn 'absolute-pathname?)
-  (pair? (absolute-pathname-root pn)) )
+  (irregex-match-data? (absolute-pathname-root pn)) )
 
 (define-inline (*char-pds? ch) (memq ch '(#\\ #\/)))
 
-(define (chop-pds str pds)
+(define (chop-pds str)
   (and str
-       (let ((len (##sys#size str))
-	     (pdslen (if pds (##sys#size pds) 1)))
+       (let ((len (##sys#size str)))
 	 (if (and (fx>= len 1)
-		  (if pds
-		      (##core#inline "C_substring_compare" str pds (fx- len pdslen) 0 pdslen)
-		      (*char-pds? (##core#inline "C_subchar" str (fx- len pdslen)) ) ) )
-	     (##sys#substring str 0 (fx- len pdslen))
+		  (*char-pds? (##core#inline "C_subchar" str (fx- len 1)) ) )
+	     (##sys#substring str 0 (fx- len 1))
 	     str) ) ) )
 
 (define make-pathname)
 (define make-absolute-pathname)
 
-(let ([string-append string-append]
-      [def-pds "/"] )
+(let ()
 
-  (define (conc-dirs dirs pds)
+  (define (conc-dirs dirs)
     (##sys#check-list dirs 'make-pathname)
-    (let loop ([strs dirs])
+    (let loop ((strs dirs))
       (if (null? strs)
 	  ""
 	  (let ((s1 (car strs)))
 	    (if (zero? (string-length s1))
 		(loop (cdr strs))
 		(string-append 
-		 (chop-pds (car strs) pds)
-		 (or pds def-pds)
+		 (chop-pds (car strs))
+		 "/"
 		 (loop (cdr strs))) ) ) ) ) )
 
-  (define (canonicalize-dirs dirs pds)
-    (cond [(or (not dirs) (null? dirs)) ""]
-	  [(string? dirs) (conc-dirs (list dirs) pds)]
-	  [else           (conc-dirs dirs pds)] ) )
+  (define (canonicalize-dirs dirs)
+    (cond ((or (not dirs) (null? dirs)) "")
+	  ((string? dirs) (conc-dirs (list dirs)))
+	  (else           (conc-dirs dirs)) ) )
 
-  (define (_make-pathname loc dir file ext pds)
-    (let ([ext (or ext "")]
-	  [file (or file "")]
-	  [pdslen (if pds (##sys#size pds) 1)] )
+  (define (_make-pathname loc dir file ext)
+    (let ((ext (or ext ""))
+	  (file (or file "")))
       (##sys#check-string dir loc)
       (##sys#check-string file loc)
       (##sys#check-string ext loc)
-      (when pds (##sys#check-string pds loc))
       (string-append
        dir
-       (if (and (fx>= (##sys#size file) pdslen)
-		(if pds
-                    (##core#inline "C_substring_compare" pds file 0 0 pdslen)
-                    (*char-pds? (##core#inline "C_subchar" file 0))))
-	   (##sys#substring file pdslen (##sys#size file))
+       (if (and (fx>= (##sys#size dir) 1)
+		(fx>= (##sys#size file) 1)
+		(*char-pds? (##core#inline "C_subchar" file 0)))
+	   (##sys#substring file 1 (##sys#size file))
 	   file)
        (if (and (fx> (##sys#size ext) 0)
 		(not (char=? (##core#inline "C_subchar" ext 0) #\.)) )
@@ -244,39 +211,45 @@ EOF
 
   (set! make-pathname
     (lambda (dirs file #!optional ext)
-      (_make-pathname 'make-pathname (canonicalize-dirs dirs def-pds) file ext def-pds)))
+      (_make-pathname 'make-pathname (canonicalize-dirs dirs) file ext)))
 
   (set! make-absolute-pathname
     (lambda (dirs file #!optional ext)
       (_make-pathname
        'make-absolute-pathname
-       (let ([dir (canonicalize-dirs dirs def-pds)])
+       (let ((dir (canonicalize-dirs dirs)))
 	 (if (absolute-pathname? dir)
 	     dir
-	     (##sys#string-append def-pds dir)) )
-       file ext def-pds) ) ) )
+	     (##sys#string-append "/"dir)) )
+       file ext) ) ) )
 
 (define decompose-pathname
   (let* ([patt1 "^(.*[\\/\\\\])?([^\\/\\\\]+)(\\.([^\\/\\\\.]+))$"]
 	 [patt2 "^(.*[\\/\\\\])?((\\.)?[^\\/\\\\]+)$"]
-	 [rx1 (regexp patt1)]
-	 [rx2 (regexp patt2)]
+	 [rx1 (irregex patt1)]
+	 [rx2 (irregex patt2)]
 	 [strip-pds
 	  (lambda (dir)
 	    (and dir
 		 (if (member dir '("/" "\\"))
 		     dir
-		     (chop-pds dir #f) ) ) )] )
+		     (chop-pds dir) ) ) )] )
     (lambda (pn)
       (##sys#check-string pn 'decompose-pathname)
       (if (fx= 0 (##sys#size pn))
 	  (values #f #f #f)
-	  (let ([ms (string-match rx1 pn)])
+	  (let ([ms (irregex-search rx1 pn)])
 	    (if ms
-		(values (strip-pds (cadr ms)) (caddr ms) (car (cddddr ms)))
-		(let ([ms (string-match rx2 pn)])
+		(values 
+		 (strip-pds (irregex-match-substring ms 1))
+		 (irregex-match-substring ms 2)
+		 (irregex-match-substring ms 4))
+		(let ([ms (irregex-search rx2 pn)])
 		  (if ms
-		      (values (strip-pds (cadr ms)) (caddr ms) #f)
+		      (values
+		       (strip-pds (irregex-match-substring ms 1))
+		       (irregex-match-substring ms 2) 
+		       #f)
 		      (values (strip-pds pn) #f #f) ) ) ) ) ) ) ) )
 
 (define pathname-directory
@@ -322,9 +295,9 @@ EOF
 (define create-temporary-file)
 (define create-temporary-directory)
 
-(let ((call-with-output-file call-with-output-file)
-      (temp #f)
-      (temp-prefix "temp"))
+(let ((temp #f)
+      (temp-prefix "temp")
+      (string-append string-append))
   (define (tempdir)
     (or temp
 	(let ((tmp 
@@ -341,9 +314,12 @@ EOF
 	(let* ((n (##core#inline "C_random_fixnum" #x10000))
 	       (pn (make-pathname 
 		    (tempdir)
-		    (##sys#string-append 
+		    (string-append 
 		     temp-prefix
-		     (number->string n 16)) ext)) )
+		     (number->string n 16)
+		     "."
+		     (##sys#number->string (##sys#fudge 33))) ; PID
+		    ext)) )
 	  (if (file-exists? pn)
 	      (loop)
 	      (call-with-output-file pn (lambda (p) pn)) ) ) ) ) )
@@ -355,7 +331,9 @@ EOF
 		    (tempdir)
 		    (string-append
 		     temp-prefix
-		     (number->string n 16)))))
+		     (number->string n 16)
+		     "."
+		     (##sys#number->string (##sys#fudge 33)))))) ; PID
 	  (if (directory-exists? pn) 
 	      (loop)
 	      (let ((r (##core#inline "C_mkdir" (##sys#make-c-string pn 'create-temporary-directory))))
@@ -370,12 +348,14 @@ EOF
 ;;; normalize pathname for a particular platform
 
 (define normalize-pathname
-  (let ((reverse reverse)
-	(display display)
-	(bldplt (if (memq (build-platform) '(msvc mingw32)) 'windows 'unix)) )
+  (let ((bldplt (if (eq? (build-platform) 'mingw32) 'windows 'unix)) )
     (define (addpart part parts)
       (cond ((string=? "." part) parts)
-            ((string=? ".." part) (if (null? parts) '("..") (cdr parts)))
+            ((string=? ".." part) 
+	     (if (or (null? parts)
+		     (string=? ".." (car parts)))
+		 (cons part parts)
+		 (cdr parts)))
             (else (cons part parts) ) ) )
     (lambda (path #!optional (platform bldplt))
       (let ((sep (if (eq? platform 'windows) #\\ #\/)))
@@ -388,11 +368,14 @@ EOF
 		   (when (fx> i prev)
 		     (set! parts (addpart (##sys#substring path prev i) parts)))
 		   (if (null? parts)
-		       (if abspath
-		           (##sys#string-append (string sep) ".")
-		           (##sys#string-append "." (string sep)) )
+		       (let ((r (if abspath
+				    (##sys#string-append (string sep) ".")
+				    (##sys#string-append "." (string sep)) )))
+			 (if drive
+			     (##sys#string-append drive r)
+			     r))
 		       (let ((out (open-output-string))
-			     (parts (reverse parts)))
+			     (parts (##sys#fast-reverse parts)))
 			 (display (car parts) out)
 			 (for-each
 			  (lambda (p)
