@@ -1,6 +1,6 @@
 ;;;; irregex.import.scm - import library for "regex" module (irregex API)
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -26,11 +26,48 @@
 
 (##sys#register-primitive-module
  'irregex
- '(irregex string->irregex sre->irregex string->sre
-    irregex? irregex-match-data?
-    irregex-new-matches irregex-reset-matches!
-    irregex-match-start irregex-match-end irregex-match-substring
-    irregex-search irregex-search/matches irregex-match irregex-match-string
-    irregex-fold irregex-replace irregex-replace/all irregex-apply-match
-    irregex-dfa irregex-dfa/search irregex-dfa/extract
-    irregex-nfa irregex-flags irregex-submatches irregex-lengths irregex-names))
+ '(irregex 
+   irregex-apply-match
+   irregex-dfa 
+   irregex-dfa/extract
+   irregex-dfa/search
+   irregex-extract
+   irregex-flags
+   irregex-fold
+   irregex-fold/chunked
+   irregex-lengths
+   irregex-match
+   irregex-match?
+   irregex-match-data?
+   irregex-match-end-chunk
+   irregex-match-end-index
+   irregex-match-names
+   irregex-match-num-submatches
+   irregex-match-start-chunk
+   irregex-match-start-index
+   irregex-match-string
+   irregex-match-subchunk
+   irregex-match-substring
+   irregex-match-valid-index?
+   irregex-match/chunked
+   irregex-names
+   irregex-new-matches
+   irregex-nfa
+   irregex-num-submatches
+   irregex-opt
+   irregex-quote
+   irregex-replace
+   irregex-replace/all
+   irregex-reset-matches!
+   irregex-search
+   irregex-search/chunked
+   irregex-search/matches
+   irregex-split
+   irregex?
+   make-irregex-chunker
+   maybe-string->sre
+   sre->irregex
+   sre->string
+   string->irregex 
+   string->sre
+   ))
