@@ -1,6 +1,6 @@
 ;;;; lolevel.import.scm - import library for "lolevel" module
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -34,21 +34,19 @@
    extend-procedure
    extended-procedure?
    free
-   global-bound?
-   global-make-unbound!
-   global-ref
-   global-set!
    locative->object
    locative-ref
    locative-set!
    locative?
    make-locative
    make-record-instance
+   make-pointer-vector
    make-weak-locative
    move-memory!
-   mutate-procedure
+   mutate-procedure			; DEPRECATED
+   mutate-procedure!
    null-pointer
-   null-pointer?
+   null-pointer?			; DEPRECATED
    number-of-bytes
    number-of-slots
    object->pointer
@@ -75,6 +73,11 @@
    pointer-s32-set!
    pointer-s8-ref
    pointer-s8-set!
+   pointer-vector
+   pointer-vector?
+   pointer-vector-length
+   pointer-vector-ref
+   pointer-vector-set!
    pointer-tag
    pointer-u16-ref
    pointer-u16-set!
