@@ -1,6 +1,6 @@
 ;;;; lolevel.scm - Low-level routines for CHICKEN
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -32,7 +32,8 @@
    ##sys#check-block
    ##sys#check-become-alist
    ##sys#check-generic-structure
-   ##sys#check-generic-vector )
+   ##sys#check-generic-vector
+   pv-buf-ref pv-buf-set!)
   (not inline ipc-hook-0 ##sys#invalid-procedure-call-hook)
   (foreign-declare #<<EOF
 #if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
@@ -221,9 +222,9 @@ EOF
   (##sys#check-special ptr 'pointer->address)
   (##sys#pointer->address ptr) )
 
-(define null-pointer ##sys#null-pointer)
+(define null-pointer ##sys#null-pointer) ; DEPRECATED
 
-(define (null-pointer? ptr)
+(define (null-pointer? ptr)		; DEPRECATED
   (##sys#check-special ptr 'null-pointer?)
   (##core#inline "C_null_pointerp" ptr))
 
@@ -244,8 +245,6 @@ EOF
   (foreign-lambda* nonnull-c-pointer ([c-pointer ptr] [integer off])
     "C_return((unsigned char *)ptr + off);") )
 
-(define pointer-offset pointer+)	; DEPRECATED
-
 (define align-to-word
   (let ([align (foreign-lambda integer "C_align" integer)])
     (lambda (x)
@@ -403,14 +402,13 @@ EOF
 	 (##sys#slot d 1) ) ) )
 
 (define set-procedure-data!
-  (let ((extend-procedure extend-procedure))
-    (lambda (proc x)
-      (let ((p2 (extend-procedure proc x)))
-	(if (eq? p2 proc)
-	    proc
-	    (##sys#signal-hook
-	     #:type-error 'set-procedure-data!
-	     "bad argument type - not an extended procedure" proc) ) ) ) ) )
+  (lambda (proc x)
+    (let ((p2 (extend-procedure proc x)))
+      (if (eq? p2 proc)
+	  proc
+	  (##sys#signal-hook
+	   #:type-error 'set-procedure-data!
+	   "bad argument type - not an extended procedure" proc) ) ) ) )
 
 
 ;;; Accessors for arbitrary vector-like block objects:
@@ -486,8 +484,6 @@ EOF
 
 ;;; Evict objects into static memory:
 
-(define-constant evict-table-size 301)
-
 (define (object-evicted? x) (##core#inline "C_permanentp" x))
 
 (define (object-evict x . allocator)
@@ -565,7 +561,9 @@ EOF
 		 (do ([i (if (##core#inline "C_specialp" x) 1 0) (fx+ i 1)])
 		     [(fx>= i n)]
 		   (release (##sys#slot x i))) )
-	       (free (##sys#address->pointer (##core#inline_allocate ("C_block_address" 4) x))) ) ] ) ) ) )
+	       (free 
+		(##sys#address->pointer
+		 (##core#inline_allocate ("C_block_address" 4) x))) ) ] ) ) ) )
 
 (define (object-size x)
   (let ([tab (make-hash-table eq?)])
@@ -616,31 +614,83 @@ EOF
   (##sys#check-become-alist alst 'object-become!)
   (##sys#become! alst) )
 
-(define (mutate-procedure old proc)
-  (##sys#check-closure old 'mutate-procedure)
-  (##sys#check-closure proc 'mutate-procedure)
+(define (mutate-procedure! old proc)
+  (##sys#check-closure old 'mutate-procedure!)
+  (##sys#check-closure proc 'mutate-procedure!)
   (let* ([n (##sys#size old)]
 	 [words (##core#inline "C_words" n)]
 	 [new (##core#inline "C_copy_block" old (##sys#make-vector words))] )
     (##sys#become! (list (cons old (proc new))))
     new ) )
 
+(define mutate-procedure mutate-procedure!) ; DEPRECATED
 
-;;; Access computed globals:
 
-(define (global-ref sym)
-  (##sys#check-symbol sym 'global-ref)
-  (##core#inline "C_retrieve" sym) )
+;;; pointer vectors
 
-(define (global-set! sym x)
-  (##sys#check-symbol sym 'global-set!)
-  (##sys#setslot sym 0 x) )
+(define make-pointer-vector
+  (let ((unset (list 'unset)))
+    (lambda (n #!optional (init unset))
+      (##sys#check-exact n 'make-pointer-vector)
+      (let* ((mul (##sys#fudge 7))	; wordsize
+	     (size (fx* n mul))
+	     (buf (##sys#make-blob size)))
+	(unless (eq? init unset)
+	  (when init
+	    (##sys#check-pointer init 'make-pointer-vector))
+	  (do ((i 0 (fx+ i 1)))
+	      ((fx>= i n))
+	    (pv-buf-set! buf i init)))
+	(##sys#make-structure 'pointer-vector n buf)))))
 
-(define (global-bound? sym)
-  (##sys#check-symbol sym 'global-bound?)
-  (##sys#symbol-has-toplevel-binding? sym) )
+(define (pointer-vector? x) 
+  (##sys#structure? x 'pointer-vector))
 
-(define (global-make-unbound! sym)
-  (##sys#check-symbol sym 'global-make-unbound!)
-  (##sys#setslot sym 0 (##sys#slot '##sys#arbitrary-unbound-symbol 0))
-  sym )
+(define (pointer-vector . ptrs)
+  (let* ((n (length ptrs))
+	 (pv (make-pointer-vector n))
+	 (buf (##sys#slot pv 2)))	; buf
+    (do ((ptrs ptrs (cdr ptrs))
+	 (i 0 (fx+ i 1)))
+	((null? ptrs) pv)
+      (let ((ptr (car ptrs)))
+	(##sys#check-pointer ptr 'pointer-vector)
+	(pv-buf-set! buf i ptr)))))
+
+(define (pointer-vector-fill! pv ptr)
+  (##sys#check-structure pv 'pointer-vector 'pointer-vector-fill!)
+  (when ptr (##sys#check-pointer ptr 'pointer-vector-fill!))
+  (let ((buf (##sys#slot pv 2))		; buf
+	(n (##sys#slot pv 1)))		; n
+    (do ((i 0 (fx+ i 1)))
+	((fx>= i n))
+      (pv-buf-set! buf i ptr))))
+
+(define pv-buf-ref
+  (foreign-lambda* c-pointer ((scheme-object buf) (unsigned-int i))
+    "C_return(*((void **)C_data_pointer(buf) + i));"))
+
+(define pv-buf-set!
+  (foreign-lambda* void ((scheme-object buf) (unsigned-int i) (c-pointer ptr))
+    "*((void **)C_data_pointer(buf) + i) = ptr;"))
+
+(define (pointer-vector-set! pv i ptr)
+  (##sys#check-structure pv 'pointer-vector 'pointer-vector-ref)
+  (##sys#check-exact i 'pointer-vector-ref)
+  (##sys#check-range i 0 (##sys#slot pv 1)) ; len
+  (when ptr (##sys#check-pointer ptr 'pointer-vector-set!))
+  (pv-buf-set! (##sys#slot pv 2) i ptr))
+
+(define pointer-vector-ref
+  (getter-with-setter
+   (lambda (pv i)
+     (##sys#check-structure pv 'pointer-vector 'pointer-vector-ref)
+     (##sys#check-exact i 'pointer-vector-ref)
+     (##sys#check-range i 0 (##sys#slot pv 1)) ; len
+     (pv-buf-ref (##sys#slot pv 2) i))	; buf
+   pointer-vector-set!
+   "(pointer-vector-ref pv i)"))
+
+(define (pointer-vector-length pv)
+  (##sys#check-structure pv 'pointer-vector 'pointer-vector-length)
+  (##sys#slot pv 1))
