@@ -1,6 +1,6 @@
 ;;;; posixwin.scm - Miscellaneous file- and process-handling routines, available on Windows
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -34,6 +34,7 @@
 ; set-signal-mask!  signal-mask	 signal-masked?	 signal-mask!  signal-unmask!
 ; user-information group-information  get-groups  set-groups!  initialize-groups
 ; errno/wouldblock
+; change-directory*
 ; change-file-owner
 ; current-user-id  current-group-id  current-effective-user-id	current-effective-group-id
 ; current-effective-user-name
@@ -63,7 +64,7 @@
 
 (declare
   (unit posix)
-  (uses scheduler regex extras utils files ports)
+  (uses scheduler irregex extras utils files ports)
   (disable-interrupts)
   (hide $quote-args-list $exec-setup $exec-teardown)
   (not inline ##sys#interrupt-hook ##sys#user-interrupt-hook)
@@ -72,7 +73,7 @@
 # define WIN32_LEAN_AND_MEAN
 #endif
 
-#if (_MSC_VER > 1300) || (defined(HAVE_WINSOCK2_H) && defined(HAVE_WS2TCPIP_H))
+#if (defined(HAVE_WINSOCK2_H) && defined(HAVE_WS2TCPIP_H))
 # include <winsock2.h>
 # include <ws2tcpip.h>
 #else
@@ -222,14 +223,7 @@ readdir(DIR * dir)
 #define open_text_output_pipe(a, n, name)    open_binary_output_pipe(a, n, name)
 #define close_pipe(p)			     C_fix(_pclose(C_port_file(p)))
 
-#define C_set_file_ptr(port, ptr)  (C_set_block_item(port, 0, (C_block_item(ptr, 0))), C_SCHEME_UNDEFINED)
-
-#define C_getpid	    getpid
 #define C_chmod(fn, m)	    C_fix(chmod(C_data_pointer(fn), C_unfix(m)))
-#define C_fdopen(a, n, fd, m) C_mpointer(a, fdopen(C_unfix(fd), C_c_string(m)))
-#define C_C_fileno(p)	    C_fix(fileno(C_port_file(p)))
-#define C_dup(x)	    C_fix(dup(C_unfix(x)))
-#define C_dup2(x, y)	    C_fix(dup2(C_unfix(x), C_unfix(y)))
 #define C_setvbuf(p, m, s)  C_fix(setvbuf(C_port_file(p), NULL, C_unfix(m), C_unfix(s)))
 #define C_test_access(fn, m)	    C_fix(access((char *)C_data_pointer(fn), C_unfix(m)))
 #define C_pipe(d, m)	    C_fix(_pipe(C_pipefds, PIPE_BUF, C_unfix(m)))
@@ -268,6 +262,7 @@ C_set_arg_string(char **where, int i, char *dat, int len)
 	ptr = (char *)C_malloc(len + 1);
 	C_memcpy(ptr, dat, len);
 	ptr[ len ] = '\0';
+        /* Can't barf() here, so the NUL byte check happens in Scheme */
     }
     else
 	ptr = NULL;
@@ -304,6 +299,8 @@ C_free_arg_string(char **where) {
 
 #define C_flushall()	    C_fix(_flushall())
 
+#define C_umask(m)          C_fix(_umask(C_unfix(m)))
+
 #define C_ctime(n)	    (C_secs = (n), ctime(&C_secs))
 
 #define C_tm_set_08(v) \
@@ -327,9 +324,6 @@ C_free_arg_string(char **where) {
 static char C_time_string [TIME_STRING_MAXLENGTH + 1];
 #undef TIME_STRING_MAXLENGTH
 
-#define C_strftime(v, f) \
-        (strftime(C_time_string, sizeof(C_time_string), C_c_string(f), C_tm_set(v)) ? C_time_string : NULL)
-
 /*
   mapping from Win32 error codes to errno
 */
@@ -837,6 +831,7 @@ C_process(const char * app, const char * cmdlin, const char ** env,
 		pb += strlen(*p) + 1;
 	    }
 	    *pb = '\0';
+            /* This _should_ already have been checked for embedded NUL bytes */
 	}
 	else
 	    success = FALSE;
@@ -986,18 +981,17 @@ EOF
       (##sys#signal-hook #:file-error 'file-close "cannot close file" fd) ) ) )
 
 (define file-read
-  (let ([make-string make-string] )
-    (lambda (fd size . buffer)
-      (##sys#check-exact fd 'file-read)
-      (##sys#check-exact size 'file-read)
-      (let ([buf (if (pair? buffer) (car buffer) (make-string size))])
-	(unless (and (##core#inline "C_blockp" buf) (##core#inline "C_byteblockp" buf))
-	  (##sys#signal-hook #:type-error 'file-read "bad argument type - not a string or blob" buf) )
-	(let ([n (##core#inline "C_read" fd buf size)])
-	  (when (eq? -1 n)
-	    (##sys#update-errno)
-	    (##sys#signal-hook #:file-error 'file-read "cannot read from file" fd size) )
-	  (list buf n) ) ) ) ) )
+  (lambda (fd size . buffer)
+    (##sys#check-exact fd 'file-read)
+    (##sys#check-exact size 'file-read)
+    (let ([buf (if (pair? buffer) (car buffer) (make-string size))])
+      (unless (and (##core#inline "C_blockp" buf) (##core#inline "C_byteblockp" buf))
+	(##sys#signal-hook #:type-error 'file-read "bad argument type - not a string or blob" buf) )
+      (let ([n (##core#inline "C_read" fd buf size)])
+	(when (eq? -1 n)
+	  (##sys#update-errno)
+	  (##sys#signal-hook #:file-error 'file-read "cannot read from file" fd size) )
+	(list buf n) ) ) ) )
 
 (define file-write
   (lambda (fd buffer . size)
@@ -1013,16 +1007,15 @@ EOF
 	n) ) ) )
 
 (define file-mkstemp
-  (let ([string-length string-length])
-    (lambda (template)
-      (##sys#check-string template 'file-mkstemp)
-      (let* ([buf (##sys#make-c-string template 'file-mkstemp)]
-	     [fd (##core#inline "C_mkstemp" buf)]
-	     [path-length (string-length buf)])
-	(when (eq? -1 fd)
-	  (##sys#update-errno)
-	  (##sys#signal-hook #:file-error 'file-mkstemp "cannot create temporary file" template) )
-	(values fd (##sys#substring buf 0 (fx- path-length 1) ) ) ) ) ) )
+  (lambda (template)
+    (##sys#check-string template 'file-mkstemp)
+    (let* ([buf (##sys#make-c-string template 'file-mkstemp)]
+	   [fd (##core#inline "C_mkstemp" buf)]
+	   [path-length (string-length buf)])
+      (when (eq? -1 fd)
+	(##sys#update-errno)
+	(##sys#signal-hook #:file-error 'file-mkstemp "cannot create temporary file" template) )
+      (values fd (##sys#substring buf 0 (fx- path-length 1) ) ) ) ) )
 
 
 ;;; File attribute access:
@@ -1091,16 +1084,9 @@ EOF
     (##sys#signal-hook #:file-error 'create-directory
 		       "cannot create directory" name)))
 
-(define-inline (create-directory-check name)
-    (if (file-exists? name)
-        (let ((i   (##sys#file-info name)))
-            (and i
-                 (fx= 1 (##sys#slot i 4))))
-        #f))
-
 (define-inline (create-directory-helper-silent name)
-    (unless (create-directory-check name)
-      (create-directory-helper name)))
+  (unless (##sys#file-exists? name #f #t #f)
+    (create-directory-helper name)))
 
 (define-inline (create-directory-helper-parents name)
   (let* ((l   (string-split name "/\\"))
@@ -1165,52 +1151,58 @@ EOF
 	   (else (badmode m)) ) ) ) ) )
   (set! close-input-pipe
     (lambda (port)
-      (##sys#check-port port 'close-input-pipe)
+      (##sys#check-input-port port #t 'close-input-pipe)
       (let ((r (##core#inline "close_pipe" port)))
 	(##sys#update-errno)
-	(when (eq? -1 r) (##sys#signal-hook #:file-error 'close-input-pipe "error while closing pipe" port)) ) ) )
-  (set! close-output-pipe close-input-pipe) )
+	(when (eq? -1 r)
+	  (##sys#signal-hook #:file-error 'close-input-pipe "error while closing pipe" port) )
+	r)))
+  (set! close-output-pipe
+    (lambda (port)
+      (##sys#check-output-port port #t 'close-output-pipe)
+      (let ((r (##core#inline "close_pipe" port)))
+	(##sys#update-errno)
+	(when (eq? -1 r)
+	  (##sys#signal-hook #:file-error 'close-output-pipe "error while closing pipe" port) )
+	r))))
 
-(let ([open-input-pipe open-input-pipe]
-      [open-output-pipe open-output-pipe]
-      [close-input-pipe close-input-pipe]
-      [close-output-pipe close-output-pipe] )
-  (set! call-with-input-pipe
-    (lambda (cmd proc . mode)
-      (let ([p (apply open-input-pipe cmd mode)])
+(define call-with-input-pipe
+  (lambda (cmd proc . mode)
+    (let ([p (apply open-input-pipe cmd mode)])
+      (##sys#call-with-values
+       (lambda () (proc p))
+       (lambda results
+	 (close-input-pipe p)
+	 (apply values results) ) ) ) ) )
+
+(define call-with-output-pipe
+  (lambda (cmd proc . mode)
+    (let ([p (apply open-output-pipe cmd mode)])
+      (##sys#call-with-values
+       (lambda () (proc p))
+       (lambda results
+	 (close-output-pipe p)
+	 (apply values results) ) ) ) ) )
+
+(define with-input-from-pipe
+  (lambda (cmd thunk . mode)
+    (let ([p (apply open-input-pipe cmd mode)])
+      (fluid-let ((##sys#standard-input p))
 	(##sys#call-with-values
-	 (lambda () (proc p))
+	 thunk
 	 (lambda results
 	   (close-input-pipe p)
-	   (apply values results) ) ) ) ) )
-  (set! call-with-output-pipe
-    (lambda (cmd proc . mode)
-      (let ([p (apply open-output-pipe cmd mode)])
+	   (apply values results) ) ) ) ) ) )
+
+(define with-output-to-pipe
+  (lambda (cmd thunk . mode)
+    (let ([p (apply open-output-pipe cmd mode)])
+      (fluid-let ((##sys#standard-output p))
 	(##sys#call-with-values
-	 (lambda () (proc p))
+	 thunk
 	 (lambda results
 	   (close-output-pipe p)
-	   (apply values results) ) ) ) ) )
-  (set! with-input-from-pipe
-    (lambda (cmd thunk . mode)
-      (let ([old ##sys#standard-input]
-	    [p (apply open-input-pipe cmd mode)] )
-	(set! ##sys#standard-input p)
-	(##sys#call-with-values thunk
-	  (lambda results
-	    (close-input-pipe p)
-	    (set! ##sys#standard-input old)
-	    (apply values results) ) ) ) ) )
-  (set! with-output-to-pipe
-    (lambda (cmd thunk . mode)
-      (let ([old ##sys#standard-output]
-	    [p (apply open-output-pipe cmd mode)] )
-	(set! ##sys#standard-output p)
-	(##sys#call-with-values thunk
-	  (lambda results
-	    (close-output-pipe p)
-	    (set! ##sys#standard-output old)
-	    (apply values results) ) ) ) ) ) )
+	   (apply values results) ) ) ) ) ) )
 
 
 ;;; Pipe primitive:
@@ -1219,11 +1211,11 @@ EOF
 (define-foreign-variable _pipefd1 int "C_pipefds[ 1 ]")
 
 (define create-pipe
-    (lambda (#!optional (mode (fxior open/binary open/noinherit)))
-      (when (fx< (##core#inline "C_pipe" #f mode) 0)
-	(##sys#update-errno)
-	(##sys#signal-hook #:file-error 'create-pipe "cannot create pipe") )
-      (values _pipefd0 _pipefd1) ) )
+  (lambda (#!optional (mode (fxior open/binary open/noinherit)))
+    (when (fx< (##core#inline "C_pipe" #f mode) 0)
+      (##sys#update-errno)
+      (##sys#signal-hook #:file-error 'create-pipe "cannot create pipe") )
+    (values _pipefd0 _pipefd1) ) )
 
 ;;; Signal processing:
 
@@ -1268,68 +1260,9 @@ EOF
     signal/term signal/int signal/fpe signal/ill
     signal/segv signal/abrt signal/break))
 
-(let ([oldhook ##sys#interrupt-hook]
-      [sigvector (make-vector 256 #f)] )
-  (set! signal-handler
-    (lambda (sig)
-      (##sys#check-exact sig 'signal-handler)
-      (##sys#slot sigvector sig) ) )
-  (set! set-signal-handler!
-    (lambda (sig proc)
-      (##sys#check-exact sig 'set-signal-handler!)
-      (##core#inline "C_establish_signal_handler" sig (and proc sig))
-      (vector-set! sigvector sig proc) ) )
-  (set! ##sys#interrupt-hook
-    (lambda (reason state)
-      (let ([h (##sys#slot sigvector reason)])
-	(if h
-	    (begin
-	      (h reason)
-	      (##sys#context-switch state) )
-	    (oldhook reason state) ) ) ) ) )
 
 ;;; More errno codes:
 
-(define-foreign-variable _errno int "errno")
-
-(define-foreign-variable _eperm int "EPERM")
-(define-foreign-variable _enoent int "ENOENT")
-(define-foreign-variable _esrch int "ESRCH")
-(define-foreign-variable _eintr int "EINTR")
-(define-foreign-variable _eio int "EIO")
-(define-foreign-variable _enoexec int "ENOEXEC")
-(define-foreign-variable _ebadf int "EBADF")
-(define-foreign-variable _echild int "ECHILD")
-(define-foreign-variable _enomem int "ENOMEM")
-(define-foreign-variable _eacces int "EACCES")
-(define-foreign-variable _efault int "EFAULT")
-(define-foreign-variable _ebusy int "EBUSY")
-(define-foreign-variable _eexist int "EEXIST")
-(define-foreign-variable _enotdir int "ENOTDIR")
-(define-foreign-variable _eisdir int "EISDIR")
-(define-foreign-variable _einval int "EINVAL")
-(define-foreign-variable _emfile int "EMFILE")
-(define-foreign-variable _enospc int "ENOSPC")
-(define-foreign-variable _espipe int "ESPIPE")
-(define-foreign-variable _epipe int "EPIPE")
-(define-foreign-variable _eagain int "EAGAIN")
-(define-foreign-variable _erofs int "EROFS")
-(define-foreign-variable _enxio int "ENXIO")
-(define-foreign-variable _e2big int "E2BIG")
-(define-foreign-variable _exdev int "EXDEV")
-(define-foreign-variable _enodev int "ENODEV")
-(define-foreign-variable _enfile int "ENFILE")
-(define-foreign-variable _enotty int "ENOTTY")
-(define-foreign-variable _efbig int "EFBIG")
-(define-foreign-variable _emlink int "EMLINK")
-(define-foreign-variable _edom int "EDOM")
-(define-foreign-variable _erange int "ERANGE")
-(define-foreign-variable _edeadlk int "EDEADLK")
-(define-foreign-variable _enametoolong int "ENAMETOOLONG")
-(define-foreign-variable _enolck int "ENOLCK")
-(define-foreign-variable _enosys int "ENOSYS")
-(define-foreign-variable _enotempty int "ENOTEMPTY")
-(define-foreign-variable _eilseq int "EILSEQ")
 
 (define errno/perm _eperm)
 (define errno/noent _enoent)
@@ -1435,7 +1368,7 @@ EOF
 
 (define port->fileno
   (lambda (port)
-    (##sys#check-port port 'port->fileno)
+    (##sys#check-open-port port 'port->fileno)
     (if (not (zero? (##sys#peek-unsigned-integer port 0)))
 	(let ([fd (##core#inline "C_C_fileno" port)])
 	  (when (fx< fd 0)
@@ -1473,8 +1406,7 @@ EOF
   (##core#undefined) )
 
 (define get-environment-variables
-  (let ([get (foreign-lambda c-string "C_getenventry" int)]
-	[substring substring] )
+  (let ([get (foreign-lambda c-string "C_getenventry" int)])
     (lambda ()
       (let loop ([i 0])
 	(let ([entry (get i)])
@@ -1488,54 +1420,12 @@ EOF
 
 ;;; Time related things:
 
-(define (check-time-vector loc tm)
-  (##sys#check-vector tm loc)
-  (when (fx< (##sys#size tm) 10)
-    (##sys#error loc "time vector too short" tm) ) )
-
-(define (seconds->local-time #!optional (secs (current-seconds)))
-  (##sys#check-number secs 'seconds->local-time)
-  (##sys#decode-seconds secs #f) )
-
-(define (seconds->utc-time #!optional (secs (current-seconds)))
-  (##sys#check-number secs 'seconds->utc-time)
-  (##sys#decode-seconds secs #t) )
-
-(define seconds->string
-  (let ([ctime (foreign-lambda c-string "C_ctime" integer)])
-    (lambda (#!optional (secs (current-seconds)))
-      (let ([str (ctime secs)])
-        (if str
-            (##sys#substring str 0 (fx- (##sys#size str) 1))
-            (##sys#error 'seconds->string "cannot convert seconds to string" secs) ) ) ) ) )
-
-(define time->string
-  (let ([asctime (foreign-lambda c-string "C_asctime" scheme-object)]
-        [strftime (foreign-lambda c-string "C_strftime" scheme-object scheme-object)])
-    (lambda (tm #!optional fmt)
-      (check-time-vector 'time->string tm)
-      (if fmt
-          (begin
-            (##sys#check-string fmt 'time->string)
-            (or (strftime tm (##sys#make-c-string fmt 'time->string))
-                (##sys#error 'time->string "time formatting overflows buffer" tm)) )
-          (let ([str (asctime tm)])
-            (if str
-                (##sys#substring str 0 (fx- (##sys#size str) 1))
-                (##sys#error 'time->string "cannot convert time vector to string" tm) ) ) ) ) ) )
-
-(define (local-time->seconds tm)
-  (check-time-vector 'local-time->seconds tm)
-  (let ((t (##core#inline_allocate ("C_a_mktime" 4) tm)))
-    (if (fp= t -1.0)
-	(##sys#error 'local-time->seconds "cannot convert time vector to seconds" tm) 
-	t)))
-
 (define local-timezone-abbreviation
   (foreign-lambda* c-string ()
    "char *z = (_daylight ? _tzname[1] : _tzname[0]);\n"
    "C_return(z);") )
 
+
 ;;; Other things:
 
 (define _exit
@@ -1544,7 +1434,7 @@ EOF
       (ex0 (if (pair? code) (car code) 0)) ) ) )
 
 (define (terminal-port? port)
-  (##sys#check-port port 'terminal-port?)
+  (##sys#check-open-port port 'terminal-port?)
   (let ([fp (##sys#peek-unsigned-integer port 0)])
     (and (not (eq? 0 fp)) (##core#inline "C_tty_portp" port) ) ) )
 
@@ -1559,20 +1449,20 @@ EOF
 (define-foreign-variable _bufsiz int "BUFSIZ")
 
 (define set-buffering-mode!
-    (lambda (port mode . size)
-      (##sys#check-port port 'set-buffering-mode!)
-      (let ([size (if (pair? size) (car size) _bufsiz)]
-	    [mode (case mode
-		    [(###full) _iofbf]
-		    [(###line) _iolbf]
-		    [(###none) _ionbf]
-		    [else (##sys#error 'set-buffering-mode! "invalid buffering-mode" mode port)] ) ] )
-	(##sys#check-exact size 'set-buffering-mode!)
-	(when (fx< (if (eq? 'stream (##sys#slot port 7))
-		       (##core#inline "C_setvbuf" port mode size)
-		       -1)
-		   0)
-	  (##sys#error 'set-buffering-mode! "cannot set buffering mode" port mode size) ) ) ) )
+  (lambda (port mode . size)
+    (##sys#check-open-port port 'set-buffering-mode!)
+    (let ([size (if (pair? size) (car size) _bufsiz)]
+	  [mode (case mode
+		  [(###full) _iofbf]
+		  [(###line) _iolbf]
+		  [(###none) _ionbf]
+		  [else (##sys#error 'set-buffering-mode! "invalid buffering-mode" mode port)] ) ] )
+      (##sys#check-exact size 'set-buffering-mode!)
+      (when (fx< (if (eq? 'stream (##sys#slot port 7))
+		     (##core#inline "C_setvbuf" port mode size)
+		     -1)
+		 0)
+	(##sys#error 'set-buffering-mode! "cannot set buffering mode" port mode size) ) ) ) )
 
 ;;; Process handling:
 
@@ -1592,36 +1482,35 @@ EOF
 ; arguments with embedded whitespace will parse incorrectly. Must
 ; string-quote such arguments.
 (define $quote-args-list
-  (let ([char-whitespace? char-whitespace?]
-	[string-length string-length]
-	[string-ref string-ref]
-	[string-append string-append])
-    (lambda (lst exactf)
-      (if exactf
+  (lambda (lst exactf)
+    (if exactf
 	lst
 	(let ([needs-quoting?
-		; This is essentially (string-any char-whitespace? s) but we don't
-		; want a SRFI-13 dependency. (Do we?)
-		(lambda (s)
-		  (let ([len (string-length s)])
-		    (let loop ([i 0])
-		      (cond
-			[(fx= i len) #f]
-			[(char-whitespace? (string-ref s i)) #t]
-			[else (loop (fx+ i 1))]))))])
-	    (let loop ([ilst lst] [olst '()])
-	      (if (null? ilst)
-		(reverse olst)
+					; This is essentially (string-any char-whitespace? s) but we don't
+					; want a SRFI-13 dependency. (Do we?)
+	       (lambda (s)
+		 (let ([len (string-length s)])
+		   (let loop ([i 0])
+		     (cond
+		      [(fx= i len) #f]
+		      [(char-whitespace? (string-ref s i)) #t]
+		      [else (loop (fx+ i 1))]))))])
+	  (let loop ([ilst lst] [olst '()])
+	    (if (null? ilst)
+		(##sys#fast-reverse olst)
 		(let ([str (car ilst)])
 		  (loop
-		    (cdr ilst)
-		    (cons
-		      (if (needs-quoting? str) (string-append "\"" str "\"") str)
-		      olst)) ) ) ) ) ) ) ) )
+		   (cdr ilst)
+		   (cons
+		    (if (needs-quoting? str) (string-append "\"" str "\"") str)
+		    olst)) ) ) ) ) ) ) )
 
 (define $exec-setup
-  (let ([setarg (foreign-lambda void "C_set_exec_arg" int scheme-pointer int)]
-	[setenv (foreign-lambda void "C_set_exec_env" int scheme-pointer int)]
+  ;; NOTE: We use c-string here instead of scheme-object.
+  ;; Because set_exec_* make a copy, this implies a double copy.
+  ;; At least it's secure, we can worry about performance later, if at all
+  (let ([setarg (foreign-lambda void "C_set_exec_arg" int c-string int)]
+	[setenv (foreign-lambda void "C_set_exec_env" int c-string int)]
 	[build-exec-argvec
 	  (lambda (loc lst argvec-setter idx)
 	    (if lst
@@ -1661,8 +1550,6 @@ EOF
     ($exec-teardown 'process-spawn "cannot spawn process" filename
       (if envlst (##core#inline "C_spawnvpe" mode prg) (##core#inline "C_spawnvp" mode prg))) ) )
 
-(define current-process-id (foreign-lambda int "C_getpid"))
-
 (define-foreign-variable _shlcmd c-string "C_shlcmd")
 
 (define (##sys#shell-command)
@@ -1702,9 +1589,10 @@ EOF
 ; where stdin-input-port?, etc. is a port or #f, indicating no port created.
 
 (define ##sys#process
+  ;; XXX TODO: When environment is implemented, check for embedded NUL bytes!
   (let ([c-process
 	  (foreign-lambda bool "C_process" c-string c-string c-pointer
-	    (pointer int) (pointer int) (pointer int) (pointer int) int)])
+	    (c-pointer int) (c-pointer int) (c-pointer int) (c-pointer int) int)])
     ; The environment list must be sorted & include current directory
     ; information for the system drives. i.e !C:=...
     ; For now any environment is ignored.
@@ -1729,6 +1617,7 @@ EOF
 
 (define process)
 (define process*)
+
 (let ([%process
 	(lambda (loc err? cmd args env exactf)
 	  (let ([chkstrlst
@@ -1761,21 +1650,7 @@ EOF
     (values pid #t _exstatus)
     (values -1 #f #f) ) )
 
-(define process-wait
-  (lambda (pid . args)
-    (let-optionals* args ([nohang #f])
-      (##sys#check-exact pid 'process-wait)
-      (receive [epid enorm ecode] (##sys#process-wait pid nohang)
-	(if (fx= epid -1)
-	  (begin
-	    (##sys#update-errno)
-	    (##sys#signal-hook #:process-error 'process-wait "waiting for child process failed" pid) )
-	  (values epid enorm ecode) ) ) ) ) )
-
-(define sleep
-  (lambda (t)
-    (##core#inline "C_sleep" t)
-    0) )
+(define sleep (foreign-lambda int "C_sleep" int))
 
 (define-foreign-variable _hostname c-string "C_hostname")
 (define-foreign-variable _osver c-string "C_osver")
@@ -1817,6 +1692,7 @@ EOF
      (define (?name . _)
        (error '?name (##core#immutable '"this function is not available on this platform")) ) ] ) )
 
+(define-unimplemented change-directory*)
 (define-unimplemented change-file-owner)
 (define-unimplemented create-fifo)
 (define-unimplemented create-session)
