@@ -1,6 +1,6 @@
 ;;;; setup-api.scm - build + installation API for eggs
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -24,7 +24,7 @@
 ; POSSIBILITY OF SUCH DAMAGE.
 
 
-(require-library srfi-1 regex utils posix srfi-13 extras ports data-structures files)
+(require-library srfi-1 irregex utils posix srfi-13 extras ports data-structures files)
 
 ; This code is partially quite messy and the API is not overly consistent,
 ; mainly because it has grown "organically" while the old chicken-setup program
@@ -39,12 +39,13 @@
     ((run execute)
      compile
      standard-extension
-     make make/proc
+     make make/proc			; DEPRECATED
      host-extension
      install-extension install-program install-script
      setup-verbose-mode setup-install-mode deployment-mode
      installation-prefix
      destination-prefix
+     runtime-prefix
      chicken-prefix
      find-library find-header 
      program-path remove-file* 
@@ -53,7 +54,7 @@
      test-compile try-compile run-verbose
      extra-features extra-nonfeatures
      copy-file move-file
-     required-chicken-version required-extension-version
+     required-chicken-version required-extension-version ;DEPRECATED
      sudo-install keep-intermediates
      version>=?
      extension-name-and-version
@@ -67,7 +68,7 @@
      setup-error-handling)
   
   (import scheme chicken foreign
-	  regex utils posix ports extras data-structures
+	  irregex utils posix ports extras data-structures
 	  srfi-1 srfi-13 files)
 
 ;;; Constants, variables and parameters
@@ -197,7 +198,7 @@
 	     (let loop ()
 	       (let ((ln (read-line)))
 		 (unless (eof-object? ln)
-		   (write-line (string-substitute rx subst ln #t)) 
+		   (write-line (irregex-replace/all rx ln subst))
 		   (loop) ) ) ) ) ) ) )
       (let ((tmp (create-temporary-file)))
 	(patch (list tmp tmp) rx subst)
@@ -230,28 +231,35 @@
   (reg "chicken-status" (foreign-value "C_CHICKEN_STATUS_PROGRAM" c-string))
   (reg "chicken-bug" (foreign-value "C_CHICKEN_BUG_PROGRAM" c-string)))
 
+(define (target-prefix fname)
+  (and-let* ((tp (runtime-prefix)))
+    (make-pathname tp fname)))
+
 (define (fixpath prg)
-  (if (string=? prg "csc")
-      (string-intersperse 
-       (cons*
-	(shellpath (find-program "csc"))
-	"-feature" "compiling-extension" 
-	(if (and (feature? #:cross-chicken)
-		 (not (host-extension)))
-	    "" "-setup-mode")
-	(if (keep-intermediates) "-k" "")
-	(if (host-extension) "-host" "")
-	(if (deployment-mode) "-deployed" "")
-	(append
-	 (map (lambda (f)
-		(string-append "-feature " (symbol->string f)))
-	      (extra-features))
-	 (map (lambda (f)
-		(string-append "-no-feature " (symbol->string f)))
-	      (extra-nonfeatures))
-	 *csc-options*) )
-       " ")
-      (find-program prg)))
+  (cond ((string=? prg "csc")
+	 (string-intersperse 
+	  (cons*
+	   (shellpath (find-program "csc"))
+	   "-feature" "compiling-extension" 
+	   (if (or (deployment-mode)
+		   (and (feature? #:cross-chicken)
+			(not (host-extension))))
+	       "" "-setup-mode")
+	   (if (keep-intermediates) "-k" "")
+	   (if (host-extension) "-host" "")
+	   (if (deployment-mode) "-deployed" "")
+	   (append
+	    (map (lambda (f)
+		   (string-append "-feature " (symbol->string f)))
+		 (extra-features))
+	    (map (lambda (f)
+		   (string-append "-no-feature " (symbol->string f)))
+		 (extra-nonfeatures))
+	    *csc-options*) )
+	  " ") )
+	((and (string-prefix? "./" prg) *windows-shell*)
+	 (shellpath (substring prg 2)))
+	(else (find-program prg))))
 
 (define (fixmaketarget file)
   (if (and (equal? "so" (pathname-extension file))
@@ -282,6 +290,9 @@
 
 ;;; "make" functionality
 
+;;; DEPRECATED
+;;; vvv
+
 (define (make:find-matching-line str spec)
   (let ((match? (lambda (s) (string=? s str))))
     (let loop ((lines spec))
@@ -373,8 +384,8 @@
 					 ((string? reason)
 					  (string-append " because " reason " changed"))
 					 (else
-					  (string-append (sprintf " just because (reason: ~a date: ~a)" 
-								  reason date)))) ) )
+					  (sprintf " just because (reason: ~a date: ~a)" 
+					    reason date)))) )
 			      (handle-exceptions exn
 				  (begin
 				    (printf "make: Failed to make ~a: ~a~%"
@@ -435,6 +446,9 @@
 		'('())
 		(cddr form)))))))
 
+;;;^^^
+;;; DEPRECATED
+
 
 ;;; Processing setup scripts
 
@@ -442,6 +456,7 @@
   (make-pathname rpath fn setup-file-extension) )
 
 (define destination-prefix (make-parameter #f))
+(define runtime-prefix (make-parameter #f))
 
 (define installation-prefix
   (let ((prefix (get-environment-variable "CHICKEN_INSTALL_PREFIX")))
@@ -469,6 +484,7 @@
     (when (setup-verbose-mode) (printf "writing info ~A -> ~S ...~%" id info))
     (let* ((sid (->string id))
 	   (setup-file (make-setup-info-pathname sid (repo-path #t))))
+      (ensure-directory setup-file)
       (cond (*sudo*
 	     (let ((tmp (create-temporary-file)))
 	       (with-output-to-file tmp (cut pp info))
@@ -485,8 +501,17 @@
 		      to-path
 		      (make-pathname prefix to-path) )
 		  to-path))))
-    (ensure-directory to)
-    (run (,*copy-command* ,(shellpath from) ,(shellpath to)))
+    (let walk ((from from) (to to))
+      (cond ((directory? from)
+	     (for-each
+	      (lambda (f)
+		(walk (make-pathname from f) (make-pathname to f)))
+	      (directory from)))
+	    (else
+	     (ensure-directory to)
+	     (run (,*copy-command* 
+		   ,(shellpath from)
+		   ,(shellpath to))))))
     to))
 
 (define (path-prefix? pref path)
@@ -534,22 +559,28 @@
 	    "unknown"))))
 
 (define (supply-version info version)
-  (if (assq 'version info)
-      info
-      (cons `(version ,(what-version version)) info)))
+  (cond ((assq 'version info) => 
+	 (lambda (a)
+	   (cons 
+	    `(egg-name ,(extension-name))
+	    info)))
+	(else
+	 (let ((v (what-version version)))
+	   (cons*
+	    `(version ,v)
+	    `(egg-name ,(extension-name))
+	    info)))))
 
 
 ;;; Convenience function
 
-(define (standard-extension name version #!key (static #t) (info '()))
+(define (standard-extension name #!optional version #!key static (info '()))
+  ;; `static' is ignored
   (let* ((sname (->string name))
 	 (fname (make-pathname #f sname "scm"))
 	 (iname (make-pathname #f sname "import.scm"))
 	 (ilname (make-pathname #f sname "inline")))
     (compile -dynamic -optimize-level 3 -debug-level 1 ,fname -emit-import-library ,name)
-    (when static
-      (compile -c -optimize-level 3 -debug-level 1 ,fname -emit-import-library
-	       ,name -unit ,name))
     (compile -dynamic -optimize-level 3 -debug-level 0 ,iname)
     (install-extension
      name
@@ -558,8 +589,7 @@
        ,@(if (file-exists? ilname)
 	     (list ilname)
 	     '()))
-     `(,@(supply-version info version)
-       ,@(if static `((static ,(make-pathname #f fname "o"))) '())))))
+     `(,@(supply-version info version)))))
 
 
 ;;; Installation
@@ -572,9 +602,6 @@
 	   (dests (map (lambda (f)
 			 (let ((from (if (pair? f) (car f) f))
 			       (to (make-dest-pathname rpathd f)) )
-			   (when (and (not *windows*) 
-				      (equal? "so" (pathname-extension to)))
-			     (run (,*remove-command* ,(shellpath to)) ))
 			   (copy-file from to)
 			   (unless *windows-shell*
 			     (run (,*chmod-command* a+r ,(shellpath to))))
@@ -583,7 +610,9 @@
 					(equal? (cadr static) from) 
 					(equal? (pathname-extension to) "a"))
 			       (run (,*ranlib-command* ,(shellpath to)) ) ))
-			   to))
+			   (if (deployment-mode)
+			       f
+			       (or (target-prefix to) to))))
 		       files) ) )
       (write-info id dests (supply-version info #f)) ) ) )
 
@@ -595,7 +624,7 @@
   (when (setup-install-mode)
     (let* ((files (check-filelist (if (list? files) files (list files))))
 	   (pre (installation-prefix))
-	   (ppath (ensure-directory (make-pathname pre "bin")))
+	   (ppath (ensure-directory (make-pathname pre "bin") #t))
 	   (files (if *windows*
                       (map (lambda (f)
                              (if (list? f) 
@@ -617,7 +646,7 @@
   (when (setup-install-mode)
     (let* ((files (check-filelist (if (list? files) files (list files))))
 	   (pre (installation-prefix))
-	   (ppath (ensure-directory (make-pathname pre "bin")))
+	   (ppath (ensure-directory (make-pathname pre "bin") #t))
 	   (pfiles (map (lambda (f)
 			  (let ((from (if (pair? f) (car f) f))
 				(to (make-dest-pathname ppath f)) )
@@ -644,11 +673,11 @@
 			  (sprintf "lib/chicken/~a" (##sys#fudge 42)))
 			 (repository-path)))) ; otherwise use repo-path
 	       (repository-path))) )
-    (ensure-directory p)
+    (ensure-directory p #t)
     p) )
 
-(define (ensure-directory path)
-  (and-let* ((dir (pathname-directory path)))
+(define (ensure-directory path #!optional full)
+  (and-let* ((dir (if full path (pathname-directory path))))
     (if (file-exists? dir)
 	(unless (directory? dir)
 	  (error "cannot create directory: a file with the same name already exists") )
@@ -681,7 +710,7 @@
     (ignore-errors ($system (sprintf "~A ~A" *remove-command* (shellpath fname))))
     (zero? r) ) )
 
-(define (required-chicken-version v)
+(define (required-chicken-version v)	;DEPRECATED
   (when (version>=? v (chicken-version) ) 
     (error (sprintf "CHICKEN version ~a or higher is required" v)) ) )
 
@@ -691,7 +720,7 @@
     "the currently installed extension `~s' ~a - please run~%~%  chicken-install ~a~a~%~%and repeat the current installation operation."
     ext msg ext (if version (conc ":" version) "")) ) )
 
-(define (required-extension-version . args)
+(define (required-extension-version . args) ;DEPRECATED
   (let loop ((args args))
     (cond ((null? args) #f)
 	  ((and (list? args) (>= (length args) 2))
@@ -702,11 +731,13 @@
 	     (if info
 		 (let ((ver (and (assq 'version info) (cadr (assq 'version info)))))
 		   (cond ((not ver) (upgrade-message ext "has no associated version information"))
-			 ((and (version>=? version ver) (not (string=? (->string version) (->string ver))))
+			 ((and (version>=? version ver) 
+			       (not (string=? (->string version) (->string ver))))
 			  (upgrade-message 
 			   ext
-			   (sprintf "is older than ~a, which is the minimum version that this extension requires"
-				    version) 
+			   (sprintf
+			       "is older than ~a, which is the minimum version that this extension requires"
+			     version) 
                            version) )
 			 (else (loop more)) ) ) 
 		 (upgrade-message ext "is not installed") ) ) )
@@ -728,7 +759,7 @@
 (define (version>=? v1 v2)
   (define (version->list v)
     (map (lambda (x) (or (string->number x) x))
-	 (string-split-fields "[-\\._]" (->string v) #:infix)))
+	 (irregex-split "[-\\._]" (->string v))))
   (let loop ((p1 (version->list v1))
 	     (p2 (version->list v2)))
     (cond ((null? p1) (null? p2))
@@ -755,8 +786,7 @@
                    [ensure-string (lambda (x) (if (or (not x) (null? x)) "" (->string x)))])
                (list (ensure-string nam) (ensure-string ver)) ) ]
             [else
-             (warning "invalid extension-name-and-version" x)
-             (extension-name-and-version) ] ) ) ) )
+             (error "invalid extension-name-and-version" x)]))))
 
 (define (extension-name)
   (car (extension-name-and-version)) )
