@@ -1,6 +1,6 @@
 ;;;; srfi-13.import.scm - import library for "srfi-13" module
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -110,21 +110,23 @@
    xsubstring)
  `((let-string-start+end 
     ()
-    ,(##sys#er-transformer
-      (lambda (form r c)
-	(##sys#check-syntax 'let-string-start+end form '(_ _ _ _ _ . _))
-	(let ((s-e-r (cadr form))
-	      (proc (caddr form))
-	      (s-exp (cadddr form))
-	      (args-exp (car (cddddr form)))
-	      (body (cdr (cddddr form)))
-	      (%receive (r 'receive))
-	      (%string-parse-start+end (r 'string-parse-start+end))
-	      (%string-parse-final-start+end (r 'string-parse-final-start+end)))
-	  (if (pair? (cddr s-e-r))
-	      `(,%receive (,(caddr s-e-r) ,(car s-e-r) ,(cadr s-e-r))
-			  (,%string-parse-start+end ,proc ,s-exp ,args-exp)
-			  ,@body)
-	      `(,%receive ,s-e-r
-			  (,%string-parse-final-start+end ,proc ,s-exp ,args-exp)
-			  ,@body) ) ))))))
+    ,(##sys#ensure-transformer
+      (##sys#er-transformer
+       (lambda (form r c)
+	 (##sys#check-syntax 'let-string-start+end form '(_ _ _ _ _ . _))
+	 (let ((s-e-r (cadr form))
+	       (proc (caddr form))
+	       (s-exp (cadddr form))
+	       (args-exp (car (cddddr form)))
+	       (body (cdr (cddddr form)))
+	       (%receive (r 'receive))
+	       (%string-parse-start+end (r 'string-parse-start+end))
+	       (%string-parse-final-start+end (r 'string-parse-final-start+end)))
+	   (if (pair? (cddr s-e-r))
+	       `(,%receive (,(caddr s-e-r) ,(car s-e-r) ,(cadr s-e-r))
+			   (,%string-parse-start+end ,proc ,s-exp ,args-exp)
+			   ,@body)
+	       `(,%receive ,s-e-r
+			   (,%string-parse-final-start+end ,proc ,s-exp ,args-exp)
+			   ,@body) ) )))
+      'let-string-start+end))))
