@@ -5,46 +5,15 @@
   (unit srfi-13)
   (uses srfi-14)
   (fixnum)
-  (disable-warning redef)
   (hide %string-prefix? %string-hash %finish-string-concatenate-reverse %string-suffix-length %string-prefix-length
 	%string-map %string-copy! %string-compare %substring/shared %string-suffix? %multispan-repcopy!
 	%string-prefix-length-ci %string-suffix-length-ci %string-prefix-ci? %string-suffix-ci?
 	##srfi13#traverse
 	%string-titlecase! %string-map! %string-compare-ci ##srfi13#string-fill!)
-  (standard-bindings not boolean? apply call-with-current-continuation eq? eqv? equal? pair? cons car cdr caar cadr
-		     cdar cddr caaar caadr cadar caddr cdaar cdadr cddar cdddr caaaar caaadr caadar caaddr cadaar
-		     cadadr caddar cadddr cdaaar cdaadr cdadar cdaddr cddaar cddadr cdddar cddddr set-car! set-cdr!
-		     null? list list? length zero? * - error + / - > < >= <= current-output-port current-input-port
-		     write-char newline write display append symbol->string char? char->integer
-		     integer->char eof-object? vector-length string-length string-ref string-set! vector-ref 
-		     vector-set! char=? char<? char>? char>=? char<=? gcd lcm reverse symbol? string->symbol
-		     number? complex? real? integer? rational? odd? even? positive? negative? exact? inexact?
-		     max min quotient remainder modulo floor ceiling truncate round exact->inexact inexact->exact
-		     exp log sin expt sqrt cos tan asin acos atan number->string string->number char-ci=?
-		     char-ci<? char-ci>? char-ci>=? char-ci<=? char-alphabetic? char-whitespace? char-numeric?
-		     char-lower-case? char-upper-case? char-upcase char-downcase string? string=? string>? string<?
-		     string>=? string<=? string-ci=? string-ci<? string-ci>? string-ci<=? string-ci>=?
-		     string-append list->string vector? vector->list list->vector string read map for-each
-		     read-char substring vector-fill! make-string make-vector open-input-file
-		     open-output-file call-with-input-file call-with-output-file close-input-port close-output-port
-		     port? values call-with-values vector procedure? memq memv assq assv member assoc) 
-  (extended-bindings)
+  (not standard-bindings string-copy string->list string-fill!)
   (disable-interrupts) )
 
-(cond-expand
- [paranoia]
- [else
-  (declare
-    (no-procedure-checks-for-usual-bindings)
-    (bound-to-procedure
-     string-concatenate check-substring-spec ##srfi13#string-fill! string-parse-final-start+end
-     ##sys#substring string-index-right string-skip-right substring/shared
-     string-concatenate/shared make-kmp-restart-vector string-ci= string= char-set?
-     char-set-contains? string-fold char-set string-skip string-index string-downcase! char->int
-     string-parse-start+end substring-spec-ok?)
-    (no-bound-checks) ) ] )
-
-(include "unsafe-declarations.scm")
+(include "common-declarations.scm")
 
 (register-feature! 'srfi-13)
 
@@ -814,7 +783,7 @@
 	 (or (and (eq? s1 s2) (= start1 start2))		; Fast path
 	     (%string-compare s1 start1 end1 s2 start2 end2	; Real test
 			      (lambda (i) #f)
-			      values
+			      (lambda (i) (if i #t #f))
 			      (lambda (i) #f))))))
 
 (define (string<> s1 s2 . maybe-starts+ends)
@@ -823,9 +792,9 @@
     (or (not (= (- end1 start1) (- end2 start2)))		; Fast path
 	(and (not (and (eq? s1 s2) (= start1 start2)))		; Quick filter
 	     (%string-compare s1 start1 end1 s2 start2 end2	; Real test
-			      values
+			      (lambda (i) (if i #t #f))
 			      (lambda (i) #f)
-			      values)))))
+			      (lambda (i) (if i #t #f)))))))
 
 (define (string< s1 s2 . maybe-starts+ends)
   (let-string-start+end2 (start1 end1 start2 end2) 
@@ -834,7 +803,7 @@
 	(< end1 end2)
 
 	(%string-compare s1 start1 end1 s2 start2 end2 		; Real test
-			 values
+			 (lambda (i) (if i #t #f))
 			 (lambda (i) #f)
 			 (lambda (i) #f)))))
 
@@ -847,7 +816,7 @@
 	(%string-compare s1 start1 end1 s2 start2 end2 		; Real test
 			 (lambda (i) #f)
 			 (lambda (i) #f)
-			 values))))
+			 (lambda (i) (if i #t #f))))))
 
 (define (string<= s1 s2 . maybe-starts+ends)
   (let-string-start+end2 (start1 end1 start2 end2) 
@@ -856,8 +825,8 @@
 	(<= end1 end2)
 
 	(%string-compare s1 start1 end1 s2 start2 end2 		; Real test
-			 values
-			 values
+			 (lambda (i) (if i #t #f))
+			 (lambda (i) (if i #t #f))
 			 (lambda (i) #f)))))
 
 (define (string>= s1 s2 . maybe-starts+ends)
@@ -868,8 +837,8 @@
 
 	(%string-compare s1 start1 end1 s2 start2 end2 		; Real test
 			 (lambda (i) #f)
-			 values
-			 values))))
+			 (lambda (i) (if i #t #f))
+			 (lambda (i) (if i #t #f))))))
 
 (define (string-ci= s1 s2 . maybe-starts+ends)
   (let-string-start+end2 (start1 end1 start2 end2) 
@@ -878,7 +847,7 @@
 	 (or (and (eq? s1 s2) (= start1 start2))		; Fast path
 	     (%string-compare-ci s1 start1 end1 s2 start2 end2	; Real test
 				 (lambda (i) #f)
-				 values
+				 (lambda (i) (if i #t #f))
 				 (lambda (i) #f))))))
 
 (define (string-ci<> s1 s2 . maybe-starts+ends)
@@ -887,9 +856,9 @@
     (or (not (= (- end1 start1) (- end2 start2)))		; Fast path
 	(and (not (and (eq? s1 s2) (= start1 start2)))		; Quick filter
 	     (%string-compare-ci s1 start1 end1 s2 start2 end2	; Real test
-				 values
+				 (lambda (i) (if i #t #f))
 				 (lambda (i) #f)
-				 values)))))
+				 (lambda (i) (if i #t #f)))))))
 
 (define (string-ci< s1 s2 . maybe-starts+ends)
   (let-string-start+end2 (start1 end1 start2 end2) 
@@ -898,7 +867,7 @@
 	(< end1 end2)
 
 	(%string-compare-ci s1 start1 end1 s2 start2 end2	; Real test
-			    values
+			    (lambda (i) (if i #t #f))
 			    (lambda (i) #f)
 			    (lambda (i) #f)))))
 
@@ -911,7 +880,7 @@
 	(%string-compare-ci s1 start1 end1 s2 start2 end2	; Real test
 			    (lambda (i) #f)
 			    (lambda (i) #f)
-			    values))))
+			    (lambda (i) (if i #t #f))))))
 
 (define (string-ci<= s1 s2 . maybe-starts+ends)
   (let-string-start+end2 (start1 end1 start2 end2) 
@@ -920,8 +889,8 @@
 	(<= end1 end2)
 
 	(%string-compare-ci s1 start1 end1 s2 start2 end2	; Real test
-			    values
-			    values
+			    (lambda (i) (if i #t #f))
+			    (lambda (i) (if i #t #f))
 			    (lambda (i) #f)))))
 
 (define (string-ci>= s1 s2 . maybe-starts+ends)
@@ -932,8 +901,8 @@
 
 	(%string-compare-ci s1 start1 end1 s2 start2 end2	; Real test
 			    (lambda (i) #f)
-			    values
-			    values))))
+			    (lambda (i) (if i #t #f))
+			    (lambda (i) (if i #t #f))))))
 
 
 ;;; Hash
@@ -1797,10 +1766,16 @@
 	    ((= 1 slen)		; Fast path for 1-char replication.
 	     (make-string anslen (string-ref s start)))
 
+	    ;; CHICKEN compiles this file with (declare (fixnum)), so
+	    ;; flonum operations are not reliable.  Since this clause
+	    ;; just provides a shorter path to avoid calling
+	    ;; %multispan-repcopy!, we comment it out and leave the
+	    ;; fixnum declaration.
+	    ;;
 	    ;; Selected text falls entirely within one span.
-	    ((= (floor (/ from slen)) (floor (/ to slen)))
-	     (##sys#substring s (+ start (modulo from slen))
-			  (+ start (modulo to   slen))))
+	    ;; ((= (floor (/ from slen)) (floor (/ to slen)))
+	    ;;  (##sys#substring s (+ start (modulo from slen))
+	    ;; 		  (+ start (modulo to   slen))))
 
 	    ;; Selected text requires multiple spans.
 	    (else (let ((ans (make-string anslen)))
@@ -1844,11 +1819,17 @@
 	    ((= 1 slen)			; Fast path for 1-char replication.
 	     (##srfi13#string-fill! target (string-ref s start) tstart tend))
 
+	    ;; CHICKEN compiles this file with (declare (fixnum)), so
+	    ;; flonum operations are not reliable.  Since this clause
+	    ;; just provides a shorter path to avoid calling
+	    ;; %multispan-repcopy!, we comment it out and leave the
+	    ;; fixnum declaration.
+	    ;;
 	    ;; Selected text falls entirely within one span.
-	    ((= (floor (/ sfrom slen)) (floor (/ sto slen)))
-	     (%string-copy! target tstart s 
-			    (+ start (modulo sfrom slen))
-			    (+ start (modulo sto   slen))))
+	    ;; ((= (floor (/ sfrom slen)) (floor (/ sto slen)))
+	    ;;  (%string-copy! target tstart s
+	    ;; 		    (+ start (modulo sfrom slen))
+	    ;; 		    (+ start (modulo sto   slen))))
 
 	    ;; Multi-span copy.
 	    (else (%multispan-repcopy! target tstart s sfrom sto start end))))))
