@@ -1,6 +1,6 @@
 ;;;; srfi-18.scm - Simple thread unit - felix
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -29,7 +29,7 @@
  (unit srfi-18)
  (uses scheduler)
  (disable-interrupts)
- (hide ##sys#compute-time-limit) )
+ (hide compute-time-limit) )
 
 (include "common-declarations.scm")
 
@@ -45,7 +45,7 @@
 
 ;;; Helper routines:
 
-(define (##sys#compute-time-limit tm loc)
+(define (compute-time-limit tm loc)
   (cond ((not tm) #f)
 	((##sys#structure? tm 'time) (##sys#slot tm 1))
 	((number? tm) (+ (current-milliseconds) (* tm 1000)))
@@ -57,28 +57,16 @@
 (define (current-time)
   (##sys#make-structure 'time (current-milliseconds)))
 
-(define srfi-18:current-time current-time)
-
 (define (time->seconds tm)
   (##sys#check-structure tm 'time 'time->seconds)
-  (fp* (##sys#slot tm 1) 1000.0))
-
-(define (time->milliseconds tm)		; DEPRECATED
-  (##sys#check-structure tm 'time 'time->milliseconds)
-  (##sys#slot tm 1))
+  (fp/ (##sys#slot tm 1) 1000.0))
 
 (define (seconds->time n)
   (##sys#check-number n 'seconds->time)
   (##sys#make-structure 'time (fp* (##sys#exact->inexact n) 1000.0)))
 
-(define (milliseconds->time nms)	; DEPRECATED
-  (##sys#check-number nms 'milliseconds->time)
-  (##sys#make-structure 'time (##sys#exact->inexact nms)))
-
 (define (time? x) (##sys#structure? x 'time))
 
-(define srfi-18:time? time?)
-
 
 ;;; Exception handling:
 
@@ -132,14 +120,17 @@
   (##sys#check-structure thread 'thread 'thread-state)
   (##sys#slot thread 3) )
 
-(define (thread-specific thread)
-  (##sys#check-structure thread 'thread 'thread-specific)
-  (##sys#slot thread 10) )
-
 (define (thread-specific-set! thread x)
   (##sys#check-structure thread 'thread 'thread-specific-set!)
   (##sys#setslot thread 10 x) )
 
+(define thread-specific
+  (getter-with-setter
+   (lambda (thread)
+     (##sys#check-structure thread 'thread 'thread-specific)
+     (##sys#slot thread 10) )
+   thread-specific-set!))
+
 (define (thread-quantum thread)
   (##sys#check-structure thread 'thread 'thread-quantum)
   (##sys#slot thread 9) )
@@ -170,7 +161,7 @@
   (lambda (thread . timeout)
     (##sys#check-structure thread 'thread 'thread-join!)
     (let* ((limit (and (pair? timeout) 
-		       (##sys#compute-time-limit (##sys#slot timeout 0) 'thread-join!)))
+		       (compute-time-limit (##sys#slot timeout 0) 'thread-join!)))
 	   (rest (and (pair? timeout) (##sys#slot timeout 1)))
 	   (tosupplied (and rest (pair? rest)))
 	   (toval (and tosupplied (##sys#slot rest 0))) )
@@ -213,7 +204,7 @@
 (define (thread-suspend! thread)
   (##sys#check-structure thread 'thread 'thread-suspend!)
   (##sys#setslot thread 3 'suspended)
-  (when (eq? thread ##sys#current-thread)
+  (when (eq? thread ##sys#current-thread) ;XXX what if thread is ready or blocked?
     (##sys#call-with-current-continuation
      (lambda (return)
        (##sys#setslot thread 1 (lambda () (return (##core#undefined))))
@@ -221,7 +212,7 @@
 
 (define (thread-resume! thread)
   (##sys#check-structure thread 'thread 'thread-resume!)
-  (when (eq? (##sys#slot thread 3) 'suspended)
+  (when (eq? (##sys#slot thread 3) 'suspended) ;XXX what if thread is ready or blocked?
     (##sys#setslot thread 3 'ready)
     (##sys#add-to-ready-queue thread) ) )
 
@@ -234,18 +225,15 @@
 	 (##sys#thread-block-for-timeout! ct limit)
 	 (##sys#schedule) ) ) ) )
   (unless tm (##sys#signal-hook #:type-error 'thread-sleep! "invalid timeout argument" tm))
-  (sleep (##sys#compute-time-limit tm 'thread-sleep!)) )
+  (sleep (compute-time-limit tm 'thread-sleep!)) )
 
 
 ;;; Mutexes:
 
 (define (mutex? x) (##sys#structure? x 'mutex))
 
-(define make-mutex
-  (lambda id
-    (let* ((id (if (pair? id) (car id) (gensym 'mutex)))
-	   (m (##sys#make-mutex id ##sys#current-thread)) )
-      m) ) )
+(define (make-mutex #!optional (id (gensym 'mutex)))
+  (##sys#make-mutex id #f))
 
 (define (mutex-name x)
   (##sys#check-structure x 'mutex 'mutex-name) 
@@ -269,7 +257,7 @@
   (lambda (mutex . ms-and-t)
     (##sys#check-structure mutex 'mutex 'mutex-lock!)
     (let* ([limitsup (pair? ms-and-t)]
-	   [limit (and limitsup (##sys#compute-time-limit (car ms-and-t) 'mutex-lock!))]
+	   [limit (and limitsup (compute-time-limit (car ms-and-t) 'mutex-lock!))]
 	   [threadsup (fx> (length ms-and-t) 1)]
 	   [thread (and threadsup (cadr ms-and-t))] )
       (when thread (##sys#check-structure thread 'thread 'mutex-lock!))
@@ -334,21 +322,21 @@
       (##sys#call-with-current-continuation
        (lambda (return)
 	 (let ([waiting (##sys#slot mutex 3)]
-	       [limit (and timeout (##sys#compute-time-limit timeout 'mutex-unlock!))] )
-	   (##sys#setislot mutex 4 #f)
-	   (##sys#setislot mutex 5 #f)
+	       [limit (and timeout (compute-time-limit timeout 'mutex-unlock!))] )
+	   (##sys#setislot mutex 4 #f)	; abandoned
+	   (##sys#setislot mutex 5 #f)	; blocked
 	   (let ((t (##sys#slot mutex 2)))
 	     (when t
-	       (##sys#setslot t 8 (##sys#delq mutex (##sys#slot t 8)))))
+	       (##sys#setslot t 8 (##sys#delq mutex (##sys#slot t 8))))) ; unown from owner
 	   (when cvar
 	     (##sys#setslot cvar 2 (##sys#append (##sys#slot cvar 2) (##sys#list ct)))
-	     (##sys#setslot ct 11 cvar)
+	     (##sys#setslot ct 11 cvar)	; block object
 	     (cond (limit
 		    (##sys#setslot 
 		     ct 1
 		     (lambda () 
 		       (##sys#setslot cvar 2 (##sys#delq ct (##sys#slot cvar 2)))
-		       (##sys#setslot ct 11 #f)
+		       (##sys#setslot ct 11 #f) ; block object
 		       (if (##sys#slot ct 13) ; unblocked by timeout
 			   (return #f)
 			   (begin
@@ -449,14 +437,13 @@
 
 ;;; Don't block in the repl: (by Chris Double)
 
-(unless (eq? (build-platform) 'msvc)
-  (set! ##sys#read-prompt-hook
-    (let ([old ##sys#read-prompt-hook])
-      (lambda ()
-	(when (or (##sys#fudge 12) (##sys#tty-port? ##sys#standard-input))
-	  (old)
-	  (##sys#thread-block-for-i/o! ##sys#current-thread 0 #t)
-	  (thread-yield!)))) ) )
+(set! ##sys#read-prompt-hook
+  (let ([old ##sys#read-prompt-hook])
+    (lambda ()
+      (when (or (##sys#fudge 12) (##sys#tty-port? ##sys#standard-input))
+	(old)
+	(##sys#thread-block-for-i/o! ##sys#current-thread 0 #:input)
+	(thread-yield!)))) )
 
 
 ;;; Waiting for I/O on file-descriptor
