@@ -1,6 +1,6 @@
 ;;;; tcp.scm - Networking stuff
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -34,11 +34,11 @@
   (foreign-declare #<<EOF
 #include <errno.h>
 #ifdef _WIN32
-# if _MSC_VER > 1300
-# include <winsock2.h>
-# include <ws2tcpip.h>
+# if (defined(HAVE_WINSOCK2_H) && defined(HAVE_WS2TCPIP_H))
+#  include <winsock2.h>
+#  include <ws2tcpip.h>
 # else
-# include <winsock.h>
+#  include <winsock.h>
 # endif
 /* Beware: winsock2.h must come BEFORE windows.h */
 # define socklen_t       int
@@ -71,6 +71,10 @@ static WSADATA wsa;
 #include <sys/sockio.h>
 #endif
 
+#ifndef h_addr
+# define h_addr  h_addr_list[ 0 ]
+#endif
+
 static char addr_buffer[ 20 ];
 EOF
 ) )
@@ -95,6 +99,8 @@ EOF
 (define-foreign-variable _ipproto_tcp int "IPPROTO_TCP")
 (define-foreign-variable _invalid_socket int "INVALID_SOCKET")
 (define-foreign-variable _ewouldblock int "EWOULDBLOCK")
+(define-foreign-variable _eagain int "EAGAIN")
+(define-foreign-variable _eintr int "EINTR")
 (define-foreign-variable _einprogress int "EINPROGRESS")
 
 (define ##net#socket (foreign-lambda int "socket" int int int))
@@ -275,7 +281,7 @@ EOF
 	   (##sys#string-append "cannot bind to socket - " strerror) s port) )
 	(values s addr) ) ) ) )
 
-(define-constant default-backlog 10)
+(define-constant default-backlog 100)
 
 (define (tcp-listen port . more)
   (let-optionals more ((w default-backlog) (host #f))
@@ -322,8 +328,7 @@ EOF
   (set! tcp-accept-timeout (make-parameter #f (check 'tcp-accept-timeout))) )
 
 (define ##net#io-ports
-  (let ((tbs tcp-buffer-size)
-	(make-string make-string) )
+  (let ((tbs tcp-buffer-size))
     (lambda (fd)
       (unless (##net#make-nonblocking fd)
 	(##sys#update-errno)
@@ -344,18 +349,21 @@ EOF
 		(let loop ()
 		  (let ((n (##net#recv fd buf +input-buffer-size+ 0)))
 		    (cond ((eq? -1 n)
-			   (cond ((eq? errno _ewouldblock) 
+			   (cond ((or (eq? errno _ewouldblock) 
+				      (eq? errno _eagain))
 				  (when tmr
 				    (##sys#thread-block-for-timeout! 
 				     ##sys#current-thread
 				     (+ (current-milliseconds) tmr) ) )
-				  (##sys#thread-block-for-i/o! ##sys#current-thread fd #t)
+				  (##sys#thread-block-for-i/o! ##sys#current-thread fd #:input)
 				  (yield)
 				  (when (##sys#slot ##sys#current-thread 13)
 				    (##sys#signal-hook
 				     #:network-timeout-error
 				     "read operation timed out" tmr fd) )
 				  (loop) )
+				 ((eq? errno _eintr)
+				  (##sys#dispatch-interrupt loop))
 				 (else
 				  (##sys#update-errno)
 				  (##sys#signal-hook 
@@ -431,7 +439,7 @@ EOF
 			       (set! bufindex next)
 			       (cond ((eq? pos2 limit) ; no line-terminator, hit limit
 				      (if str (##sys#string-append str dest) dest))
-				     ((eq? pos2 next)  ; no line-terminator, hit buflen
+				     ((eq? pos2 next) ; no line-terminator, hit buflen
 				      (read-input)
 				      (if (fx>= bufindex buflen)
 					  (or str "")
@@ -444,7 +452,14 @@ EOF
 			  (read-input)
 			  (if (fx< bufindex buflen)
 			      (loop str limit)
-			      #!eof) ) ) ) ) ) )
+			      #!eof) ) ) ) )
+	       (lambda (p)		; read-buffered
+		 (if (fx>= bufindex buflen)
+		     ""
+		     (let ((str (##sys#substring buf bufpos buflen)))
+		       (set! bufpos buflen)
+		       str)))
+	       ) )
 	     (output
 	      (lambda (s)
 		(let loop ((len (##sys#size s))
@@ -452,18 +467,22 @@ EOF
 		  (let* ((count (fxmin +output-chunk-size+ len))
 			 (n (##net#send fd s offset count 0)) )
 		    (cond ((eq? -1 n)
-			   (cond ((eq? errno _ewouldblock)
+			   (cond ((or (eq? errno _ewouldblock)
+				      (eq? errno _eagain))
 				  (when tmw
 				    (##sys#thread-block-for-timeout! 
 				     ##sys#current-thread
 				     (+ (current-milliseconds) tmw) ) )
-				  (##sys#thread-block-for-i/o! ##sys#current-thread fd #f)
+				  (##sys#thread-block-for-i/o! ##sys#current-thread fd #:output)
 				  (yield) 
 				  (when (##sys#slot ##sys#current-thread 13)
 				    (##sys#signal-hook
 				     #:network-timeout-error
 				     "write operation timed out" tmw fd) )
 				  (loop len offset) )
+				 ((eq? errno _eintr)
+				  (##sys#dispatch-interrupt 
+				   (cut loop len offset)))
 				 (else
 				  (##sys#update-errno)
 				  (##sys#signal-hook 
@@ -514,18 +533,22 @@ EOF
     (let loop ()
       (if (eq? 1 (##net#select fd))
 	  (let ((fd (##net#accept fd #f #f)))
-	    (when (eq? -1 fd)
-	      (##sys#update-errno)
-	      (##sys#signal-hook 
-	       #:network-error 'tcp-accept (##sys#string-append "could not accept from listener - " strerror) 
-	       tcpl) )
-	    (##net#io-ports fd) )
+	    (cond ((not (eq? -1 fd)) (##net#io-ports fd))
+		  ((eq? errno _eintr)
+		   (##sys#dispatch-interrupt loop))
+		  (else
+		   (##sys#update-errno)
+		   (##sys#signal-hook 
+		    #:network-error
+		    'tcp-accept
+		    (##sys#string-append "could not accept from listener - " strerror)
+		    tcpl))))
 	  (begin
 	    (when tma
 	      (##sys#thread-block-for-timeout! 
 	       ##sys#current-thread
 	       (+ (current-milliseconds) tma) ) )
-	    (##sys#thread-block-for-i/o! ##sys#current-thread fd #t)
+	    (##sys#thread-block-for-i/o! ##sys#current-thread fd #:input)
 	    (yield)
 	    (when (##sys#slot ##sys#current-thread 13)
 	      (##sys#signal-hook
@@ -549,7 +572,7 @@ EOF
     "int err, optlen;"
     "optlen = sizeof(err);"
     "if (typecorrect_getsockopt(socket, SOL_SOCKET, SO_ERROR, &err, (socklen_t *)&optlen) == -1)"
-    "C_return(-1);"
+    "  C_return(-1);"
     "C_return(err);"))
 
 (define general-strerror (foreign-lambda c-string "strerror" int))
@@ -580,25 +603,29 @@ EOF
       (unless (##net#make-nonblocking s)
 	(##sys#update-errno)
 	(##sys#signal-hook #:network-error 'tcp-connect (##sys#string-append "fcntl() failed - " strerror)) )
-      (when (eq? -1 (##net#connect s addr _sockaddr_in_size))
-	(if (eq? errno _einprogress)
-	    (let loop ()
-	      (let ((f (##net#select-write s)))
-		(when (eq? f -1) (fail))
-		(unless (eq? f 1)
-		  (when tmc
-		    (##sys#thread-block-for-timeout!
-		     ##sys#current-thread
-		     (+ (current-milliseconds) tmc) ) )
-		  (##sys#thread-block-for-i/o! ##sys#current-thread s #:all)
-		  (yield)
-		  (when (##sys#slot ##sys#current-thread 13)
-		    (##sys#signal-hook
-		     #:network-timeout-error
-		     'tcp-connect
-		     "connect operation timed out" tmc s) )
-		  (loop) ) ) )
-	    (fail) ) )
+      (let loop ()
+	(when (eq? -1 (##net#connect s addr _sockaddr_in_size))
+	  (cond ((eq? errno _einprogress)
+		 (let loop2 ()
+		   (let ((f (##net#select-write s)))
+		     (when (eq? f -1) (fail))
+		     (unless (eq? f 1)
+		       (when tmc
+			 (##sys#thread-block-for-timeout!
+			  ##sys#current-thread
+			  (+ (current-milliseconds) tmc) ) )
+		       (##sys#thread-block-for-i/o! ##sys#current-thread s #:all)
+		       (yield)
+		       (when (##sys#slot ##sys#current-thread 13)
+			 (##net#close s)
+			 (##sys#signal-hook
+			  #:network-timeout-error
+			  'tcp-connect
+			  "connect operation timed out" tmc s) )
+		       (loop2) ) ) ))
+		((eq? errno _eintr)
+		 (##sys#dispatch-interrupt loop))
+		(else (fail) ) )))
       (let ((err (get-socket-error s)))
 	(cond ((fx= err -1)
 	       (##net#close s)
@@ -619,7 +646,7 @@ EOF
 	(error '##sys#tcp-port->fileno "argument does not appear to be a TCP port" p))))
 
 (define (tcp-addresses p)
-  (##sys#check-port p 'tcp-addresses)
+  (##sys#check-open-port p 'tcp-addresses)
   (let ((fd (##sys#tcp-port->fileno p)))
     (values 
      (or (##net#getsockname fd)
@@ -632,17 +659,19 @@ EOF
 	  (##sys#string-append "cannot compute remote address - " strerror) p) ) ) ) )
 
 (define (tcp-port-numbers p)
-  (##sys#check-port p 'tcp-port-numbers)
+  (##sys#check-open-port p 'tcp-port-numbers)
   (let ((fd (##sys#tcp-port->fileno p)))
-    (values
-     (or (##net#getsockport fd)
-	 (##sys#signal-hook 
-	  #:network-error 'tcp-port-numbers
-	  (##sys#string-append "cannot compute local port - " strerror) p) )
-     (or (##net#getpeerport fd)
-	 (##sys#signal-hook
-	  #:network-error 'tcp-port-numbers
-	  (##sys#string-append "cannot compute remote port - " strerror) p) ) ) ) )
+    (let ((sp (##net#getsockport fd))
+	  (pp (##net#getpeerport fd)))
+      (when (eq? -1 sp)
+	(##sys#signal-hook 
+	 #:network-error 'tcp-port-numbers
+	 (##sys#string-append "cannot compute local port - " strerror) p))
+      (when (eq? -1 pp)
+	(##sys#signal-hook
+	 #:network-error 'tcp-port-numbers
+	 (##sys#string-append "cannot compute remote port - " strerror) p) )
+      (values sp pp))))
 
 (define (tcp-listener-port tcpl)
   (##sys#check-structure tcpl 'tcp-listener 'tcp-listener-port)
@@ -655,10 +684,10 @@ EOF
     port) )
 
 (define (tcp-abandon-port p)
-  (##sys#check-port p 'tcp-abandon-port)
+  (##sys#check-open-port p 'tcp-abandon-port)
   (##sys#setislot
    (##sys#port-data p)
-   (if (##sys#slot p 1) 2 1)
+   (if (##sys#slot p 1) 1 2)
    #t) )
 
 (define (tcp-listener-fileno l)
