@@ -1,22 +1,39 @@
-(declare (standard-bindings)
-	 (extended-bindings)
-	 (block)
-	 (not safe)
-	 )
+;;;; fft.scm - fft benchmark, by Bradley Lucier
+
+
+(cond-expand
+  ((not chicken)			; disable in CHICKEN to test specialization in safe mode
+   (declare
+     (standard-bindings)
+     (extended-bindings)
+     (block)
+     (not safe)))
+  (else))
 
 ;;; All the following redefinitions are *ignored* by the Gambit compiler
 ;;; because of the declarations above.
 
+(define-syntax defalias
+  (syntax-rules ()
+    ((_ one two) 
+     (define-syntax one
+       (syntax-rules ()
+	 ((_ . args) (two . args))))))) 
+
 (cond-expand
+ (generic
+  (begin
+    (defalias fixnum->flonum exact->inexact)
+    (defalias fxodd? odd?)
+    (defalias fxeven? even?)
+    (defalias fxarithmetic-shift-right fxshr)
+    (defalias fxarithmetic-shift-left  fxshl)
+    (defalias fl* *)
+    (defalias fl/ /)
+    (defalias fl+ +)
+    (defalias fl- -)))
  (chicken
   (begin
-    (use srfi-4)
-    (define-syntax defalias
-      (syntax-rules ()
-	((_ one two) 
-	 (define-syntax one
-	   (syntax-rules ()
-	     ((_ . args) (two . args))))))) 
     (defalias fixnum->flonum exact->inexact)
     (defalias fxodd? odd?)
     (defalias fxeven? even?)
@@ -37,6 +54,8 @@
     (defalias f64vector-ref vector-ref)
     (defalias list->f64vector list->vector)
     (defalias f64vector-length vector-length)) )
+ (chicken
+  (use srfi-4))
  (else) )
 
 ;;; end of *ignored* definitions
@@ -2059,12 +2078,20 @@
 	 (expt 2 n))
 	(table
 	 (make-w (fx- n 1))))
-    (display (fx* two^n 2))(newline)
+    ;(display (fx* two^n 2))(newline)
     (let ((a
 	   (make-f64vector (fx* two^n 2) 0.)))
       (do ((i 0 (fx+ i 1)))
-	  ((fx= i iters)); (write table) (newline))
+	  ((fx= i iters)
+	   ;(write table) (newline)
+	   )
 	(direct-fft-recursive-4 a table)
 	(inverse-fft-recursive-4 a table)))))
 
-(test 2000 11)
+(cond-expand
+  (chicken
+   (let-optionals (command-line-arguments)
+       ((iters "2000")
+	(n "11"))
+     (test (string->number iters) (string->number n))))
+  (else (test 2000 11)))
