@@ -230,28 +230,6 @@
 
 ; object-unevict
 
-; global-ref
-
-(assert (pointer? (global-ref 'some-chunk)))
-
-; global-set!
-
-(global-set! 'some-chunk 34)
-
-(assert (not (pointer? (global-ref 'some-chunk))))
-
-(assert (atom? (global-ref 'some-chunk)))
-
-; global-bound?
-
-(assert (global-bound? 'some-chunk))
-
-; global-make-unbound!
-
-(global-make-unbound! 'some-chunk)
-
-(assert (not (global-bound? 'some-chunk)))
-
 ; object-become!
 
 (define some-foo '#(1 2 3))
@@ -273,3 +251,31 @@
 (assert (not (eq? foo new-foo)))
 
 (assert (equal? '(hello 1 2) (foo 1 2)))
+
+; pointer vectors
+
+(define pv (make-pointer-vector 42 #f))
+(assert (= 42 (pointer-vector-length pv)))
+(assert (not (pointer-vector-ref pv 0)))
+(pointer-vector-set! pv 1 (address->pointer 999))
+(set! (pointer-vector-ref pv 40) (address->pointer 777))
+(assert (not (pointer-vector-ref pv 0)))
+(assert (not (pointer-vector-ref pv 41)))
+(assert (= (pointer->address (pointer-vector-ref pv 1)) 999))
+(assert (= (pointer->address (pointer-vector-ref pv 40)) 777))
+(pointer-vector-fill! pv (address->pointer 1))
+(assert (= 1 (pointer->address (pointer-vector-ref pv 0))))
+
+#+(not csi)
+(begin
+  (define pv1
+    (foreign-lambda* bool ((pointer-vector pv))
+      "C_return(pv == NULL);"))
+  (define pv2
+    (foreign-lambda* c-pointer ((pointer-vector pv) (bool f))
+      "static void *xx = (void *)123;"
+      "if(f) pv[ 0 ] = xx;"
+      "C_return(xx);"))
+  (assert (eq? #t (pv1 #f)))
+  (define p (pv2 pv #t))
+  (assert (pointer=? p (pv2 pv #f))))
