@@ -196,5 +196,107 @@
  "syntax defined in module that is the result of an expansion"
  'abc (abc))
 
+(module m17 (a) (import scheme) (define a 1))
+(module m18 = m17)
+(module m19 (a) (import scheme) (define a 2))
+
+(test-equal
+ "global module alias scope (1)"
+ (module m20 ()
+   (import scheme) 
+   (import m18)
+   a)
+ 1)
+
+(test-equal
+ "local module alias scope"
+ (module m21 ()
+   (import scheme)
+   (module m18 = m19)
+   (import m18)
+   a)
+ 2)
+
+(test-equal
+ "global module alias scope (2)"
+ (module m20 ()
+   (import scheme) 
+   (import m18)
+   a)
+ 1)
+
+;; #865 - "*" export list needs special treatment when using "export"
+;; (fix contributed by "megane")
+
+(module
+ m22
+ *
+ (import chicken scheme)
+ (define b 2))
+
+(module
+ m23
+ *
+ (import chicken scheme)
+ (import m22)
+ (export b) )
+
+(test-equal
+ "`*' export-list + explicit export"
+ (module m24 ()
+   (import m23)
+   b)
+ 2)
+
+;; (contributed by "megane")
+
+(module m25 *
+  (import chicken scheme)
+  (define foo 1))
+
+(module m26 (bar)
+  (import chicken scheme)
+  (reexport m25)
+  (define bar 2))
+
+(module m27 *
+  (import chicken scheme)
+  (reexport m25) ;; <- oops, bar not exported anymore
+  (define bar 2))
+
+(test-equal
+ "handle star-exporting module with reexport"
+ (module m28 ()
+   (import scheme chicken)
+   (import (prefix m26 b/))
+   (import (prefix m27 c/))
+   (print b/foo)
+   (print c/foo)
+   (print b/bar)
+   c/bar) ;; <- Error: unbound variable: c/bar
+ 2)
+
+;; somewhat related, but with syntax (#882, found by megane):
+
+(module m29 *
+  (import chicken scheme)
+  (define-syntax m29-baz
+    (lambda _
+      ''foo)))
+
+(module m30 *
+  (import chicken scheme)
+  (import m29)
+  (export m29-baz))
+
+(test-equal
+ "star-export with explicit re-export of syntax"
+ (module m31 ()
+   (import scheme chicken)
+   (import m30)
+   (m29-baz))
+ 'foo)
+
+
 (test-end "modules")
 
