@@ -1,30 +1,26 @@
 (use files)
 
-(assert (equal? "/" (pathname-directory "/")))
-(assert (equal? "/" (pathname-directory "/abc")))
-(assert (equal? "abc" (pathname-directory "abc/")))
-(assert (equal? "abc" (pathname-directory "abc/def")))
-(assert (equal? "abc" (pathname-directory "abc/def.ghi")))
-(assert (equal? "abc" (pathname-directory "abc/.def.ghi")))
-(assert (equal? "abc" (pathname-directory "abc/.ghi")))
-(assert (equal? "/abc" (pathname-directory "/abc/")))
-(assert (equal? "/abc" (pathname-directory "/abc/def")))
-(assert (equal? "/abc" (pathname-directory "/abc/def.ghi")))
-(assert (equal? "/abc" (pathname-directory "/abc/.def.ghi")))
-(assert (equal? "/abc" (pathname-directory "/abc/.ghi")))
-(assert (equal? "q/abc" (pathname-directory "q/abc/")))
-(assert (equal? "q/abc" (pathname-directory "q/abc/def")))
-(assert (equal? "q/abc" (pathname-directory "q/abc/def.ghi")))
-(assert (equal? "q/abc" (pathname-directory "q/abc/.def.ghi")))
-(assert (equal? "q/abc" (pathname-directory "q/abc/.ghi")))
-
 (define-syntax test
   (syntax-rules ()
-    ((_ expected exp)
-     (let ((result exp)
-	   (expd expected))
-       (unless (equal? result expd)
-	 (error "test failed" result expd 'exp))))))
+    ((_ r x) (let ((y x)) (print y) (assert (equal? r y))))))
+
+(test "/" (pathname-directory "/"))
+(test "/" (pathname-directory "/abc"))
+(test "abc" (pathname-directory "abc/"))
+(test "abc" (pathname-directory "abc/def"))
+(test "abc" (pathname-directory "abc/def.ghi"))
+(test "abc" (pathname-directory "abc/.def.ghi"))
+(test "abc" (pathname-directory "abc/.ghi"))
+(test "/abc" (pathname-directory "/abc/"))
+(test "/abc" (pathname-directory "/abc/def"))
+(test "/abc" (pathname-directory "/abc/def.ghi"))
+(test "/abc" (pathname-directory "/abc/.def.ghi"))
+(test "/abc" (pathname-directory "/abc/.ghi"))
+(test "q/abc" (pathname-directory "q/abc/"))
+(test "q/abc" (pathname-directory "q/abc/def"))
+(test "q/abc" (pathname-directory "q/abc/def.ghi"))
+(test "q/abc" (pathname-directory "q/abc/.def.ghi"))
+(test "q/abc" (pathname-directory "q/abc/.ghi"))
 
 (test "./" (normalize-pathname "" 'unix))
 (test ".\\" (normalize-pathname "" 'windows))
@@ -53,12 +49,15 @@
 (test "a/b" (normalize-pathname "a/./././b" 'unix))
 (test "a/b" (normalize-pathname "a/b/c/d/../.." 'unix))
 (test "a/b/" (normalize-pathname "a/b/c/d/../../" 'unix))
+(test "../../foo" (normalize-pathname "../../foo" 'unix))
+(test "c:\\." (normalize-pathname "c:\\" 'windows))
 
 (define home (get-environment-variable "HOME"))
 
-(test (string-append home "/foo") (normalize-pathname "~/foo" 'unix))
-(test "c:~/foo" (normalize-pathname "c:~/foo" 'unix))
-(test (string-append home "\\foo") (normalize-pathname "c:~\\foo" 'windows))
+(when home
+  (test (string-append home "/foo") (normalize-pathname "~/foo" 'unix))
+  (test "c:~/foo" (normalize-pathname "c:~/foo" 'unix))
+  (test (string-append home "\\foo") (normalize-pathname "c:~\\foo" 'windows)))
 
 (assert (directory-null? "/.//"))
 (assert (directory-null? ""))
@@ -71,3 +70,14 @@
 (test '(#f "/" ("foo" "bar")) (receive (decompose-directory "//foo//bar")))
 (test '(#f #f (" " "foo" "bar")) (receive (decompose-directory " //foo//bar")))
 (test '(#f #f ("foo" "bar")) (receive (decompose-directory "foo//bar/")))
+
+(test "x/y/z.q" (make-pathname "x/y" "z" "q"))
+(test "x/y/z.q" (make-pathname "x/y" "z.q"))
+(test "x/y/z.q" (make-pathname "x/y/" "z.q"))
+(test "x/y/z.q" (make-pathname "x/y/" "z.q"))
+(test "x/y/z.q" (make-pathname "x/y\\" "z.q"))
+(test "x//y/z.q" (make-pathname "x//y/" "z.q"))
+(test "x\\y/z.q" (make-pathname "x\\y" "z.q"))
+(test 'error (handle-exceptions _ 'error (make-pathname '(#f) "foo")))
+(test "/x/y/z" (make-pathname #f "/x/y/z"))
+(test "/x/y/z" (make-pathname "/x" "/y/z"))
