@@ -1,6 +1,6 @@
-;;;; tweaks.scm - Some inline-routines and declarations
+;;;; tweaks.scm - Some inline-routines and declarations for the compiler 
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -40,9 +40,13 @@
 
 (define-inline (node? x) (##sys#structure? x 'node))
 (define-inline (make-node c p s) (##sys#make-structure 'node c p s))
-(define-inline (node-class n) (##sys#slot n 1))
-(define-inline (node-parameters n) (##sys#slot n 2))
-(define-inline (node-subexpressions n) (##sys#slot n 3))
+
+(cond-expand
+  ((not debugbuild)
+   (define-inline (node-class n) (##sys#slot n 1))
+   (define-inline (node-parameters n) (##sys#slot n 2))
+   (define-inline (node-subexpressions n) (##sys#slot n 3)))
+  (else))
 
 (define-inline (intrinsic? sym) (##sys#get sym '##compiler#intrinsic))
 
