@@ -1,6 +1,6 @@
 ;;;; utils.scm - Utilities for scripting and file stuff
 ;
-; Copyright (c) 2008-2010, The Chicken Team
+; Copyright (c) 2008-2013, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -27,7 +27,7 @@
 
 (declare
   (unit utils)
-  (uses extras srfi-13 posix files regex)
+  (uses extras srfi-13 posix files irregex)
   (fixnum)
   (hide chop-pds)
   (disable-interrupts) )
@@ -53,14 +53,14 @@
   (let ([file (optional file ##sys#standard-input)])
     (if (port? file)
 	(read-string #f file)
-	(with-input-from-file file (cut read-string #f)) ) ) )
+	(with-input-from-file file (cut read-string #f) #:binary) ) ) )
 
 
 ;;; Quote string for shell
 
 (define (qs str #!optional (platform (build-platform)))
   (case platform
-    ((mingw32 msvc)
+    ((mingw32)
      (string-append "\"" str "\""))
     (else
      (if (zero? (string-length str))
@@ -68,8 +68,8 @@
 	 (string-concatenate
 	  (map (lambda (c)
 		 (if (or (char-whitespace? c)
-			 (memq c '(#\# #\" #\' #\` #\ #\~ #\& #\% #\$ #\! #\* #\; #\< #\> #\\
-				   #\( #\) #\[ #\] #\{ #\})))
+			 (memq c '(#\# #\" #\' #\` #\ #\~ #\& #\% #\$ #\! #\* #\;
+				   #\< #\> #\\ #\( #\) #\[ #\] #\{ #\} #\?)))
 		     (string #\\ c)
 		     (string c)))
 	       (string->list str)))))))
@@ -83,31 +83,35 @@
   (let ((csc (foreign-value "C_CSC_PROGRAM" c-string))
 	(load-file load)
 	(path (foreign-value "C_INSTALL_BIN_HOME" c-string)) )
-    (lambda (filename #!key (options '()) output-file (load #t))
-      (let ((cscpath (or (file-exists? (make-pathname path csc)) "csc"))
-	    (tmpfile (and (not output-file) (create-temporary-file "so")))
-	    (crapshell (memq (build-platform) '(mingw32 msvc))))
-	(print "; compiling " filename " ...")
-	(system* 
-	 "~a~a -s ~a ~a -o ~a~a" 
-	 (if crapshell "\"" "")
-	 (qs cscpath)
-	 (string-intersperse (append (compile-file-options) options) " ")
-	 (qs filename)
-	 (qs (or output-file tmpfile))
-	 (if crapshell "\"" ""))
-	(unless output-file 
-	  (on-exit
-	   (lambda ()
-	     (handle-exceptions ex #f (delete-file* tmpfile)))))
-	(when load
-	  (let ((f (or output-file tmpfile)))
-	    (handle-exceptions ex
-		(begin
-		  (delete-file* f)
-		  (abort ex))
-	      (load-file f)
-	      f)))))))
+    (lambda (filename #!key options output-file (load #t) verbose)
+      (let* ((cscpath (or (file-exists? (make-pathname path csc)) "csc"))
+	     (tmpfile (and (not output-file) (create-temporary-file "so")))
+	     (crapshell (eq? (build-platform) 'mingw32))
+	     (cmd (sprintf "~a~a -s ~a ~a -o ~a~a" 
+		    (if crapshell "\"" "")
+		    (qs cscpath)
+		    (string-intersperse
+		     (or options
+			 (compile-file-options)))
+		    (qs filename)
+		    (qs (or output-file tmpfile))
+		    (if crapshell "\"" ""))))
+	(when verbose (print "  " cmd))
+	(let ((status (system cmd)))
+	  (cond ((zero? status)
+		 (unless output-file 
+		   (on-exit
+		    (lambda ()
+		      (handle-exceptions ex #f (delete-file* tmpfile)))))
+		 (when load
+		   (let ((f (or output-file tmpfile)))
+		     (handle-exceptions ex
+			 (begin
+			   (delete-file* f)
+			   (abort ex))
+		       (load-file f)
+		       f))))
+		(else #f)))))))
 
 
 ;;; Scan lines until regex or predicate matches
@@ -116,7 +120,7 @@
   (lambda (rx #!optional (port ##sys#standard-input))
     (let ((rx (if (procedure? rx)
 		  rx
-		  (cut string-search (regexp rx) <>))))
+		  (cute irregex-search (irregex rx) <>))))
       (let loop ()
 	(let ((ln (read-line port)))
 	  (and (not (eof-object? ln))
@@ -195,7 +199,7 @@ C_confirmation_dialog(char *msg, char *caption, int def, int abort) { return -1;
 		  ((and abort (string-ci=? "abort" ln)) (abort))
 		  (else
 		   (if abort
-		       (printf "~%Please enter \"yes\" or \"no\".~%")
-		       (printf "~%Please enter \"yes\", \"no\" or \"abort\".~%"))
+		       (printf "~%Please enter \"yes\", \"no\" or \"abort\".~%")
+		       (printf "~%Please enter \"yes\" or \"no\".~%"))
 		   (loop) ) ) ) ) ) ) ) )
   
